/*
 * GDateTime methods tests
 * Testing GDateTime methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;

/*
 * Name: g_date_set_dmy
 * Params: GDate *date, GDateDay day, GDateMonth month, GDateYear y 
 * Returns: void
 * Assertion: Sets the value of a GDate from a day, month, and year. The day-month-year triplet must be valid; if you aren't sure it is, call g_date_valid_dmy() to check before you set it.
 *            date : a GDate.
 *            day : day.
 *            month : month.
 *            y : year.
 * Params Values: 10, G_DATE_AUGUST, 2005
 * Expected: store the date  
 */
static void test_purpose_1()
{
    #define DAY     10
    #define MON     G_DATE_AUGUST
    #define YEAR    2005
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate test_date;
    g_date_set_dmy(&test_date, DAY, MON, YEAR);
    if ((test_date.day != DAY) || (test_date.month != MON) || (test_date.year != YEAR)){
        TEST_FAIL("g_date_set_dmy: set date failed", NULL);
    }
    
    #undef DATE
    #undef DAY
    #undef MON
    #undef YEAR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_set_dmy
 * Params: GDate *date, GDateDay day, GDateMonth month, GDateYear y 
 * Returns: void
 * Assertion: Sets the value of a GDate from a day, month, and year. The day-month-year triplet must be valid; if you aren't sure it is, call g_date_valid_dmy() to check before you set it.
 *            date : a GDate.
 *            day : day.
 *            month : month.
 *            y : year.
 * Params Values: 29, G_DATE_FEBRUARY, 2004
 * Expected: store the date  
 */
static void test_purpose_2()
{
    #define DAY     29
    #define MON     G_DATE_FEBRUARY
    #define YEAR    2004
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate test_date;
    g_date_set_dmy(&test_date, DAY, MON, YEAR);
    if ((test_date.day != DAY) || (test_date.month != MON) || (test_date.year != YEAR)){
        TEST_FAIL("g_date_set_dmy: set date failed", NULL);
    }
    
    #undef DATE
    #undef DAY
    #undef MON
    #undef YEAR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_get_julian
 * Params: const GDate *date
 * Returns: guint32
 * Assertion: Returns the Julian day or "serial number" of the GDate. The Julian day is simply the number of days since January 1, Year 1; i.e., January 1, Year 1 is Julian day 1; January 2, Year 1 is Julian day 2, etc. The date must be valid.
 *            date : a GDate to extract the Julian day from.
 *            Returns : Julian day.
 * Params Values: Day = 10, Month = G_DATE_AUGUST, Year = 2005
 * Expected: 732168  
 */
static void test_purpose_3()
{
    #define DAY            10
    #define MON            G_DATE_AUGUST
    #define YEAR           2005
    #define JULIAN_DATE    732168
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate date;
    guint32 julian_date;
    
    g_date_set_dmy(&date, DAY, MON, YEAR);
    julian_date = g_date_get_julian(&date);
    if (julian_date != JULIAN_DATE){
        TEST_FAIL("g_date_get_julian: julian date should be %d, now is %d", JULIAN_DATE, julian_date);
    }
    
    #undef DAY
    #undef MON
    #undef YEAR
    #undef JULIAN_DATE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_get_julian
 * Params: const GDate *date
 * Returns: guint32
 * Assertion: Returns the Julian day or "serial number" of the GDate. The Julian day is simply the number of days since January 1, Year 1; i.e., January 1, Year 1 is Julian day 1; January 2, Year 1 is Julian day 2, etc. The date must be valid.
 *            date : a GDate to extract the Julian day from.
 *            Returns : Julian day.
 * Params Values: Day = 20, Month = G_DATE_JANUARY, Year = 1
 * Expected: 20  
 */
static void test_purpose_4()
{
    #define DAY            20
    #define MON            G_DATE_JANUARY
    #define YEAR           1
    #define JULIAN_DATE    20
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate date;
    guint32 julian_date;
    
    g_date_set_dmy(&date, DAY, MON, YEAR);
    julian_date = g_date_get_julian(&date);
    if (julian_date != JULIAN_DATE){
        TEST_FAIL("g_date_get_julian: julian date should be %d, now is %d", JULIAN_DATE, julian_date);
    }
    
    #undef DAY
    #undef MON
    #undef YEAR
    #undef JULIAN_DATE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_get_weekday
 * Params: const GDate *date
 * Returns: GDateWeekday
 * Assertion: Returns the day of the week for a GDate. The date must be valid.
 *            date : a GDate.
 *            Returns : day of the week as a GDateWeekday. 
 * Params Values: Day = 10, Month = G_DATE_AUGUST, Year = 2005
 * Expected: G_DATE_WEDNESDAY  
 */
static void test_purpose_5()
{
    #define DAY    10
    #define MON    G_DATE_AUGUST
    #define YEAR   2005
    #define WEEK   G_DATE_WEDNESDAY
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate date;
    GDateWeekday week;
    
    g_date_set_dmy(&date, DAY, MON, YEAR);
    week = g_date_get_weekday(&date);
    if (week != WEEK){
        TEST_FAIL("g_date_get_weekday: weekday should be %d, now is %d", WEEK, week);
    }
    
    #undef DAY
    #undef MON
    #undef YEAR
    #undef WEEK
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_get_weekday
 * Params: const GDate *date
 * Returns: GDateWeekday
 * Assertion: Returns the day of the week for a GDate. The date must be valid.
 *            date : a GDate.
 *            Returns : day of the week as a GDateWeekday. 
 * Params Values: Day = 1, Month = G_DATE_JANUARY, Year = 1980
 * Expected: G_DATE_TUESDAY  
 */
static void test_purpose_6()
{
    #define DAY    1
    #define MON    G_DATE_JANUARY
    #define YEAR   1980
    #define WEEK   G_DATE_TUESDAY
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate date;
    GDateWeekday week;
    
    g_date_set_dmy(&date, DAY, MON, YEAR);
    week = g_date_get_weekday(&date);
    if (week != WEEK){
        TEST_FAIL("g_date_get_weekday: weekday should be %d, now is %d", WEEK, week);
    }
    
    #undef DAY
    #undef MON
    #undef YEAR
    #undef WEEK
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_clear
 * Params: const GDate *date, guint n_dates
 * Returns: void
 * Assertion: Initializes one or more GDate structs to a sane but invalid state. The cleared dates will not represent an existing date, but will not contain garbage. Useful to init a date declared on the stack.
 *            date : pointer to one or more dates to clear.
 *            n_dates : number of dates to clear. 
 * Params Values: n_dates = 1
 * Expected: 0  
 */
static void test_purpose_7()
{
    #define NDATES    1
    #define RESULT    0
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate *gDate = NULL;
    
    gDate = g_date_new();
    g_date_clear(gDate, NDATES);
    if (g_date_valid(gDate)){
        TEST_FAIL("g_date_clear: should be invalid", NULL);
    }
    g_date_free(gDate);
    
    #undef NDATES
    #undef RESULT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_is_leap_year
 * Params: GDateYear year
 * Returns: gboolean
 * Assertion: Returns TRUE if the year is a leap year.
 *            year : year to check.
 *            Returns : TRUE if the year is a leap year. 
 * Params Values: Year = 2004
 * Expected: TRUE  
 */
static void test_purpose_8()
{
    #define YEAR    2004
    #define RET     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    gboolean ret;
    
    ret = g_date_is_leap_year(YEAR);
    if (ret != RET){
        TEST_FAIL("g_date_is_leap_year: failed", NULL);
    }
    
    #undef YEAR
    #undef RET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_is_leap_year
 * Params: GDateYear year
 * Returns: gboolean
 * Assertion: Returns TRUE if the year is a leap year.
 *            year : year to check.
 *            Returns : TRUE if the year is a leap year. 
 * Params Values: Year = 2005
 * Expected: FALSE  
 */
static void test_purpose_9()
{
    #define YEAR    2005
    #define RET     FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    gboolean ret;
    
    ret = g_date_is_leap_year(YEAR);
    if (ret != RET){
        TEST_FAIL("g_date_is_leap_year: failed", NULL);
    }
    
    #undef YEAR
    #undef RET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_get_year
 * Params: const GDate *date
 * Returns: GDateYear
 * Assertion: Returns the year of a GDate. The date must be valid.
 *            date : a GDate.
 *            Returns : year in which the date falls. 
 * Params Values: Year = 2004
 * Expected: 2004  
 */
static void test_purpose_10()
{
    #define YEAR    2004
    #define RET     2004
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate gdate;
    GDateYear date_year;
    
    g_date_clear(&gdate, 1);
    g_date_set_dmy(&gdate, 1, 1, YEAR);
    date_year = g_date_get_year(&gdate);
    
    if (date_year != RET){
        TEST_FAIL("g_date_get_year: failed", NULL);
    }
    
    #undef YEAR
    #undef RET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_get_year
 * Params: const GDate *date
 * Returns: GDateYear
 * Assertion: Returns the year of a GDate. The date must be valid.
 *            date : a GDate.
 *            Returns : year in which the date falls. 
 * Params Values: Year = 1985
 * Expected: 1985  
 */
static void test_purpose_11()
{
    #define YEAR    1985
    #define RET     1985
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate gdate;
    GDateYear date_year;
    
    g_date_clear(&gdate, 1);
    g_date_set_dmy(&gdate, 1, 1, YEAR);
    date_year = g_date_get_year(&gdate);
    
    if (date_year != RET){
        TEST_FAIL("g_date_get_year: failed", NULL);
    }
    
    #undef YEAR
    #undef RET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_get_month
 * Params: const GDate *date
 * Returns: GDateMonth
 * Assertion: Returns the month of the year. The date must be valid.
 *            date : a GDate to get the month from.
 *            Returns : month of the year as a GDateMonth. 
 * Params Values: Month = 12
 * Expected: G_DATE_DECEMBER  
 */
static void test_purpose_12()
{
    #define MON     12
    #define RET     G_DATE_DECEMBER
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate gdate;
    GDateMonth date_mon;
    
    g_date_clear(&gdate, 1);
    g_date_set_dmy(&gdate, 1, MON, 2005);
    date_mon = g_date_get_month(&gdate);
    
    if (date_mon != RET){
        TEST_FAIL("g_date_get_month: failed", NULL);
    }
    
    #undef MON
    #undef RET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_get_month
 * Params: const GDate *date
 * Returns: GDateMonth
 * Assertion: Returns the month of the year. The date must be valid.
 *            date : a GDate to get the month from.
 *            Returns : month of the year as a GDateMonth. 
 * Params Values: Month = 2
 * Expected: G_DATE_FEBRUARY  
 */
static void test_purpose_13()
{
    #define MON     2
    #define RET     G_DATE_FEBRUARY
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate gdate;
    GDateMonth date_mon;
    
    g_date_clear(&gdate, 1);
    g_date_set_dmy(&gdate, 1, MON, 2005);
    date_mon = g_date_get_month(&gdate);
    
    if (date_mon != RET){
        TEST_FAIL("g_date_get_month: failed", NULL);
    }
    
    #undef MON
    #undef RET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_get_day
 * Params: const GDate *date
 * Returns: GDateDay
 * Assertion: Returns the day of the month. The date must be valid.
 *            date : a GDate to extract the day of the month from.
 *            Returns : day of the month.
 * Params Values: Day = 20
 * Expected: 20  
 */
static void test_purpose_14()
{
    #define DAY     20
    #define RET     20
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate gdate;
    GDateDay date_day;
    
    g_date_clear(&gdate, 1);
    g_date_set_dmy(&gdate, DAY, 2, 2005);
    date_day = g_date_get_day(&gdate);
    
    if (date_day != RET){
        TEST_FAIL("g_date_get_day: failed", NULL);
    }
    
    #undef DAY
    #undef RET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_get_day
 * Params: const GDate *date
 * Returns: GDateDay
 * Assertion: Returns the day of the month. The date must be valid.
 *            date : a GDate to extract the day of the month from.
 *            Returns : day of the month.
 * Params Values: Day = 2
 * Expected: 2  
 */
static void test_purpose_15()
{
    #define DAY     2
    #define RET     2
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate gdate;
    GDateDay date_day;
    
    g_date_clear(&gdate, 1);
    g_date_set_dmy(&gdate, DAY, 2, 2005);
    date_day = g_date_get_day(&gdate);
    
    if (date_day != RET){
        TEST_FAIL("g_date_get_day: failed", NULL);
    }
    
    #undef DAY
    #undef RET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_valid
 * Params: const GDate *date
 * Returns: gboolean
 * Assertion: Returns TRUE if the GDate represents an existing day. The date must not contain garbage; it should have been initialized with g_date_clear() if it wasn't allocated by one of the g_date_new() variants.
 *            date : a GDate to check.
 *            Returns : Whether the date is valid.
 * Params Values: Day = 30, Month = G_DATE_FEBRUARY, Year = 1980
 * Expected: FALSE  
 */
static void test_purpose_16()
{
    #define DAY     30
    #define MON     G_DATE_FEBRUARY
    #define YEAR    1980
    #define RET     FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate *pgdate = NULL;
    
    pgdate = g_date_new();
    g_date_clear(pgdate, 1);
    g_date_set_dmy(pgdate, DAY, MON, YEAR);
    if (RET != g_date_valid(pgdate)){
        TEST_FAIL("g_date_valid: valid failed", NULL);
    }
    g_date_free(pgdate);
    
    #undef DAY 
    #undef MON 
    #undef YEAR
    #undef RET 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_valid
 * Params: const GDate *date
 * Returns: gboolean
 * Assertion: Returns TRUE if the GDate represents an existing day. The date must not contain garbage; it should have been initialized with g_date_clear() if it wasn't allocated by one of the g_date_new() variants.
 *            date : a GDate to check.
 *            Returns : Whether the date is valid.
 * Params Values: Day = 21, Month = G_DATE_JANUARY, Year = 2005
 * Expected: TRUE  
 */
static void test_purpose_17()
{
    #define DAY     21
    #define MON     G_DATE_JANUARY
    #define YEAR    2005
    #define RET     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate *pgdate = NULL;
    
    pgdate = g_date_new();
    g_date_clear(pgdate, 1);
    g_date_set_dmy(pgdate, DAY, MON, YEAR);
    if (RET != g_date_valid(pgdate)){
        TEST_FAIL("g_date_valid: valid failed", NULL);
    }
    g_date_free(pgdate);
    
    #undef DAY 
    #undef MON 
    #undef YEAR
    #undef RET 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_valid_dmy
 * Params: GDateDay day, GDateMonth month, GDateYear year
 * Returns: gboolean
 * Assertion: Returns TRUE if the day-month-year triplet forms a valid, existing day in the range of days GDate understands (Year 1 or later, no more than a few thousand years in the future).
 *            day : day.
 *            month : month.
 *            year : year.
 *            Returns : TRUE if the date is a valid one.
 * Params Values: Day = 30, Month = G_DATE_FEBRUARY, Year = 1980           
 * Expected: FALSE  
 */
static void test_purpose_18()
{
    #define DAY     30
    #define MON     G_DATE_FEBRUARY
    #define YEAR    1980
    #define RET     FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (RET != g_date_valid_dmy(DAY, MON, YEAR)){
        TEST_FAIL("g_date_valid: valid failed", NULL);
    }
    
    #undef DAY 
    #undef MON 
    #undef YEAR
    #undef RET 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_valid_dmy
 * Params: GDateDay day, GDateMonth month, GDateYear year
 * Returns: gboolean
 * Assertion: Returns TRUE if the day-month-year triplet forms a valid, existing day in the range of days GDate understands (Year 1 or later, no more than a few thousand years in the future).
 *            day : day.
 *            month : month.
 *            year : year.
 *            Returns : TRUE if the date is a valid one.
 * Params Values: Day = 21, Month = G_DATE_JANUARY, Year = 2005           
 * Expected: TRUE  
 */
static void test_purpose_19()
{
    #define DAY     21
    #define MON     G_DATE_JANUARY
    #define YEAR    2005
    #define RET     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (RET != g_date_valid_dmy(DAY, MON, YEAR)){
        TEST_FAIL("g_date_valid: valid failed", NULL);
    }
    
    #undef DAY 
    #undef MON 
    #undef YEAR
    #undef RET 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_subtract_days
 * Params: GDate *date, guint n_days
 * Returns: void
 * Assertion: Moves a date some number of days into the past. To move by weeks, just move by weeks*7 days. The date must be valid.
 *            date : a GDate to decrement.
 *            n_days : number of days to move.
 * Params Values: Day = 21, Month = G_DATE_JANUARY, Year = 1980, n_days = 10         
 * Expected: Get the correct julian day  
 */
static void test_purpose_20()
{
    #define DAY     21
    #define MON     G_DATE_JANUARY
    #define YEAR    1980
    #define NDAYS   10
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate *pgdate = NULL;
    guint32 julian_date;
    
    pgdate = g_date_new();
    g_date_clear(pgdate, 1);
    g_date_set_dmy(pgdate, DAY, MON, YEAR);
    julian_date = g_date_get_julian(pgdate);
    
    g_date_subtract_days(pgdate, NDAYS);
    if (pgdate->julian_days != (julian_date - NDAYS)){
        TEST_FAIL("g_date_subtract_days: failed", NULL);
    }
    g_date_free(pgdate);
    
    #undef DAY  
    #undef MON  
    #undef YEAR 
    #undef NDAYS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_subtract_days
 * Params: GDate *date, guint n_days
 * Returns: void
 * Assertion: Moves a date some number of days into the past. To move by weeks, just move by weeks*7 days. The date must be valid.
 *            date : a GDate to decrement.
 *            n_days : number of days to move.
 * Params Values: Day = 1, Month = G_DATE_APRIL, Year = 2005, n_days = 10         
 * Expected: Get the correct julian day  
 */
static void test_purpose_21()
{
    #define DAY     1
    #define MON     G_DATE_APRIL
    #define YEAR    2005
    #define NDAYS   10
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate *pgdate = NULL;
    guint32 julian_date;
    
    pgdate = g_date_new();
    g_date_clear(pgdate, 1);
    g_date_set_dmy(pgdate, DAY, MON, YEAR);
    julian_date = g_date_get_julian(pgdate);
    
    g_date_subtract_days(pgdate, NDAYS);
    if (pgdate->julian_days != (julian_date - NDAYS)){
        TEST_FAIL("g_date_subtract_days: failed", NULL);
    }
    g_date_free(pgdate);
    
    #undef DAY  
    #undef MON  
    #undef YEAR 
    #undef NDAYS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_set_day
 * Params: GDate *date, GDateDay day
 * Returns: void
 * Assertion: Sets the day of the month for a GDate. If the resulting day-month-year triplet is invalid, the date will be invalid.
 *            date : a GDate.
 *            day : day to set.
 * Params Values: Day = 10, Month = G_DATE_JULY, Year = 2003, n_day = 21         
 * Expected: 21  
 */
static void test_purpose_22()
{
    #define DAY     10
    #define MON     G_DATE_JULY
    #define YEAR    2003
    #define NDAY    21
    #define RET     21
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate gdate;
    
    g_date_clear(&gdate, 1);
    g_date_set_dmy(&gdate, DAY, MON, YEAR);
    g_date_set_day(&gdate, NDAY);
    if (gdate.day != RET){
        TEST_FAIL("g_date_set_day: set day failed", NULL);
    }
    
    #undef DAY  
    #undef MON  
    #undef YEAR 
    #undef NDAY
    #undef RET  
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_compare
 * Params: const GDate *lhs, const GDate *rhs
 * Returns: gint
 * Assertion: qsort()-style comparsion function for dates. Both dates must be valid.
 *            lhs : first date to compare.
 *            rhs : second date to compare.
 *            Returns : 0 for equal, less than zero if lhs is less than rhs, greater than zero if lhs is greater than rhs.
 * Params Values: Dayl = 2, Monthl = G_DATE_JANUARY, Yearl = 2005
 * Expected: 0  
 */
static void test_purpose_23()
{
    #define DAYL     2
    #define MONL     G_DATE_JANUARY
    #define YEARL    2005
    #define RET      0
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate *pgdatel = NULL, *pgdater = NULL;
    gint ret = 0;
    
    pgdatel = g_date_new();
    pgdater = g_date_new();
    g_date_clear(pgdatel, 1);
    g_date_set_dmy(pgdatel, DAYL, MONL, YEARL);
    
    g_date_clear(pgdater, 1);
    g_date_set_dmy(pgdater, 2, G_DATE_JANUARY, 2005);
    
    ret = g_date_compare(pgdatel, pgdater);
    if (ret != RET){
        TEST_FAIL("g_date_compare: compare failed", NULL);
    }
    
    g_date_free(pgdatel);
    g_date_free(pgdater);
    
    #undef DAYL  
    #undef MONL  
    #undef YEARL 
    #undef RET  
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_compare
 * Params: const GDate *lhs, const GDate *rhs
 * Returns: gint
 * Assertion: qsort()-style comparsion function for dates. Both dates must be valid.
 *            lhs : first date to compare.
 *            rhs : second date to compare.
 *            Returns : 0 for equal, less than zero if lhs is less than rhs, greater than zero if lhs is greater than rhs.
 * Params Values: Dayl = 1, Monthl = G_DATE_JANUARY, Yearl = 2005
 * Expected: -1  
 */
static void test_purpose_24()
{
    #define DAYL     1
    #define MONL     G_DATE_JANUARY
    #define YEARL    2005
    #define RET      -1
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate *pgdatel = NULL, *pgdater = NULL;
    gint ret = 0;
    
    pgdatel = g_date_new();
    pgdater = g_date_new();
    g_date_clear(pgdatel, 1);
    g_date_set_dmy(pgdatel, DAYL, MONL, YEARL);
    
    g_date_clear(pgdater, 1);
    g_date_set_dmy(pgdater, 2, G_DATE_JANUARY, 2005);
    
    ret = g_date_compare(pgdatel, pgdater);
    if (ret != RET){
        TEST_FAIL("g_date_compare: compare failed", NULL);
    }
    
    g_date_free(pgdatel);
    g_date_free(pgdater);
    
    #undef DAYL  
    #undef MONL  
    #undef YEARL 
    #undef RET  
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_compare
 * Params: const GDate *lhs, const GDate *rhs
 * Returns: gint
 * Assertion: qsort()-style comparsion function for dates. Both dates must be valid.
 *            lhs : first date to compare.
 *            rhs : second date to compare.
 *            Returns : 0 for equal, less than zero if lhs is less than rhs, greater than zero if lhs is greater than rhs.
 * Params Values: Dayl = 2, Monthl = G_DATE_JANUARY, Yearl = 2006
 * Expected: 1  
 */
static void test_purpose_25()
{
    #define DAYL     2
    #define MONL     G_DATE_JANUARY
    #define YEARL    2006
    #define RET      1
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate *pgdatel = NULL, *pgdater = NULL;
    gint ret = 0;
    
    pgdatel = g_date_new();
    pgdater = g_date_new();
    g_date_clear(pgdatel, 1);
    g_date_set_dmy(pgdatel, DAYL, MONL, YEARL);
    
    g_date_clear(pgdater, 1);
    g_date_set_dmy(pgdater, 2, G_DATE_JANUARY, 2005);
    
    ret = g_date_compare(pgdatel, pgdater);
    if (ret != RET){
        TEST_FAIL("g_date_compare: compare failed", NULL);
    }
    
    g_date_free(pgdatel);
    g_date_free(pgdater);
    
    #undef DAYL  
    #undef MONL  
    #undef YEARL 
    #undef RET  
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_add_days
 * Params: GDate *date, guint n_days
 * Returns: void
 * Assertion: Increments a date some number of days. To move forward by weeks, add weeks*7 days. The date must be valid.
 *            date : a GDate to increment.
 *            n_days : number of days to move the date forward.
 * Params Values: Day = 21, Month = G_DATE_JANUARY, Year = 1980, n_days = 10         
 * Expected: Get the correct julian day  
 */
static void test_purpose_26()
{
    #define DAY     21
    #define MON     G_DATE_JANUARY
    #define YEAR    1980
    #define NDAYS   10
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate *pgdate = NULL;
    guint32 julian_date;
    
    pgdate = g_date_new();
    g_date_clear(pgdate, 1);
    g_date_set_dmy(pgdate, DAY, MON, YEAR);
    julian_date = g_date_get_julian(pgdate);
    
    g_date_add_days(pgdate, NDAYS);
    if (pgdate->julian_days != (julian_date + NDAYS)){
        TEST_FAIL("g_date_add_days: failed", NULL);
    }
    g_date_free(pgdate);
    
    #undef DAY  
    #undef MON  
    #undef YEAR 
    #undef NDAYS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_date_add_days
 * Params: GDate *date, guint n_days
 * Returns: void
 * Assertion: Increments a date some number of days. To move forward by weeks, add weeks*7 days. The date must be valid.
 *            date : a GDate to increment.
 *            n_days : number of days to move the date forward.
 * Params Values: Day = 1, Month = G_DATE_APRIL, Year = 2005, n_days = 10         
 * Expected: Get the correct julian day  
 */
static void test_purpose_27()
{
    #define DAY     1
    #define MON     G_DATE_APRIL
    #define YEAR    2005
    #define NDAYS   10
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDate *pgdate = NULL;
    guint32 julian_date;
    
    pgdate = g_date_new();
    g_date_clear(pgdate, 1);
    g_date_set_dmy(pgdate, DAY, MON, YEAR);
    julian_date = g_date_get_julian(pgdate);
    
    g_date_add_days(pgdate, NDAYS);
    if (pgdate->julian_days != (julian_date + NDAYS)){
        TEST_FAIL("g_date_add_days: failed", NULL);
    }
    g_date_free(pgdate);
    
    #undef DAY  
    #undef MON  
    #undef YEAR 
    #undef NDAYS
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for GDateTime ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {test_purpose_5, 1},
    {test_purpose_6, 1},
    {test_purpose_7, 1},
    {test_purpose_8, 1},
    {test_purpose_9, 1},
    {test_purpose_10, 1},
    {test_purpose_11, 1},
    {test_purpose_12, 1},
    {test_purpose_13, 1},
    {test_purpose_14, 1},
    {test_purpose_15, 1},
    {test_purpose_16, 1},
    {test_purpose_17, 1},
    {test_purpose_18, 1},
    {test_purpose_19, 1},
    {test_purpose_20, 1},
    {test_purpose_21, 1},
    {test_purpose_22, 1},
    {test_purpose_23, 1},
    {test_purpose_24, 1},
    {test_purpose_25, 1},
    {test_purpose_26, 1},
    {test_purpose_27, 1},
    {NULL,0}
};
