/*
 * GdkGeneral methods tests
 * Testing GdkGeneral methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;


/*
 * Name: gdk_screen_height
 * Params: void
 * Returns: gint
 * Assertion: Returns the height of the default screen in pixels.
 *            Returns :     the height of the default screen in pixels.
 * Params Values: void
 * Expected: gint
 * 
 * Name: gdk_screen_width
 * Params: void
 * Returns: gint
 * Assertion: Returns the width of the default screen in pixels.
 *            Returns :     the width of the default screen in pixels.
 * Params Values: void
 * Expected: gint
 * 
 * Name: gdk_screen_height_mm
 * Params: void
 * Returns: gint
 * Assertion: Returns the height of the default screen in millimeters. Note that on many X servers this value will not be correct.
 *            Returns :     the height of the default screen  in millimeters, though it is not always correct.
 * Params Values: void
 * Expected: gint
 * 
 * Name: gdk_screen_width_mm
 * Params: void
 * Returns: gint
 * Assertion: Returns the width of the default screen in millimeters. Note that on many X servers this value will not be correct.
 *            Returns :     the width of the default screen in millimeters, though it is not always correct.
 * Params Values: void
 * Expected: gint  
 */
static void test_purpose_1()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (gdk_screen_width() == 0){
        TEST_FAIL("gdk_screen_width: should not be 0", NULL);
    }
    if (gdk_screen_width_mm() == 0){
        TEST_FAIL("gdk_screen_width_mm: should not be 0", NULL);
    }
    if (gdk_screen_height() == 0){
        TEST_FAIL("gdk_screen_height: should not be 0", NULL);
    }
    if (gdk_screen_height_mm() == 0){
        TEST_FAIL("gdk_screen_height_mm: should not be 0", NULL);
    }
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_set_program_class
 * Params: const char *program_class
 * Returns: void
 * Assertion: Sets the program class. The X11 backend uses the program class to set the class name part of the WM_CLASS property on toplevel windows; see the ICCCM.
 * Params Values:  const char *program_class
 * Expected: void
 * 
 * Name: gdk_get_program_class
 * Params: void
 * Returns: const char*
 * Assertion: Gets the program class. Unless the program class has explicitly been set with gdk_set_program_class() or with the --class commandline option, the default value is the program name (determined with g_get_prgname()) with the first character converted to uppercase. 
 * Params Values: void
 * Expected: const char*  
 */
static void test_purpose_2()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (strcmp(gdk_get_program_class(), ".") != 0){
        TEST_FAIL("gdk_get_program_class: failed", NULL);
    }
    gdk_set_program_class("LSB_test_program");
    if (strcmp(gdk_get_program_class(), "LSB_test_program") != 0){
        TEST_FAIL("gdk_set_program_class: gdk_get_program_class failed", NULL);
    }
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for GdkGeneral ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {NULL,0}
};
