/*
 * GdkScreen methods tests
 * Testing GdkScreen methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;

/*
 * Name: gdk_screen_get_default
 * Params: void
 * Returns: GdkScreen*
 * Assertion: Gets the default screen for the default display. (See gdk_display_get_default()).
 *            Returns :     a GdkScreen, or NULL if there is no default display.
 * Params Values: void
 * Expected: GdkScreen*  
 */
static void test_purpose_1()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GdkScreen *gs = gdk_screen_get_default();
    if (gs == NULL){
        TEST_FAIL("gdk_screen_get_default: should not be NULL", NULL);
    }
    else{
        if (!GDK_IS_SCREEN(gs)){
            TEST_FAIL("gdk_screen_get_default: it's not a GDK_TYPE_SCREEN", NULL);
        }
    }
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_screen_get_width
 * Params: GdkScreen *screen
 * Returns: gint
 * Assertion: Gets the width of screen in pixels
 *            screen :  a GdkScreen
 *            Returns :   the width of screen in pixels.
 * Params Values: GdkScreen *screen
 * Expected: gint
 * 
 * Name: gdk_screen_get_height
 * Params: GdkScreen *screen
 * Returns: gint
 * Assertion: Gets the height of screen in pixels
 *            screen :  a GdkScreen
 *            Returns :   the height of screen in pixels.
 * Params Values: GdkScreen *screen
 * Expected: gint  
 */
static void test_purpose_2()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GdkScreen *gs = gdk_screen_get_default();
    if (gs == NULL){
        TEST_FAIL("gdk_screen_get_width: get default screen", NULL);
    }
    else{
        if (gdk_screen_get_width(gs) == 0){
            TEST_FAIL("gdk_screen_get_width: width error", NULL);
        }
        if (gdk_screen_get_height(gs) == 0){
            TEST_FAIL("gdk_screen_get_height: height error", NULL);
        }
    }
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_screen_get_width_mm
 * Params: GdkScreen *screen
 * Returns: gint
 * Assertion: Gets the width of screen in millimeters. Note that on some X servers this value will not be correct.
 *            screen :  a GdkScreen
 *            Returns :     the width of screen in pixels.
 * Params Values: GdkScreen *screen
 * Expected: gint
 * 
 * Name: gdk_screen_get_height_mm
 * Params: GdkScreen *screen
 * Returns: gint
 * Assertion: Gets the height of screen in millimeters. Note that on some X servers this value will not be correct.
 *            screen :  a GdkScreen
 *            Returns :   the height of screen in pixels.
 * Params Values: GdkScreen *screen
 * Expected: gint  
 */
static void test_purpose_3()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GdkScreen *gs = gdk_screen_get_default();
    if (gs == NULL){
        TEST_FAIL("gdk_screen_get_width_mm: get default screen", NULL);
    }
    else{
        if (gdk_screen_get_width_mm(gs) == 0){
            TEST_FAIL("gdk_screen_get_width_mm: width error", NULL);
        }
        if (gdk_screen_get_height_mm(gs) == 0){
            TEST_FAIL("gdk_screen_get_height_mm: height error", NULL);
        }
    }
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_screen_get_display
 * Params: GdkScreen *screen
 * Returns: GdkDisplay*
 * Assertion: Gets the display to which the screen belongs.
 *            screen :  a GdkScreen
 *            Returns :   the display to which screen belongs
 * Params Values: GdkScreen *screen
 * Expected: GdkDisplay*  
 */
static void test_purpose_4()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GdkScreen *gs = gdk_screen_get_default();
    if (gs == NULL){
        TEST_FAIL("gdk_screen_get_display: get default screen", NULL);
    }
    else{
        GdkDisplay* screen = gdk_screen_get_display(gs);
        if (!GDK_IS_DISPLAY(screen)){
            TEST_FAIL("gdk_screen_get_display: it's not a GDK_TYPE_DISPLAY", NULL);
        }
    }
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_screen_get_number
 * Params: GdkScreen *screen
 * Returns: gint
 * Assertion: Gets the index of screen among the screens in the display to which it belongs. (See gdk_screen_get_display())
 *            screen :  a GdkScreen
 *            Returns :   the index
 * Params Values: GdkScreen *screen
 * Expected: gint  
 */
static void test_purpose_5()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GdkScreen *gs = gdk_screen_get_default();
    if (gs == NULL){
        TEST_FAIL("gdk_screen_get_number: get default screen", NULL);
    }
    else{
        gint index = gdk_screen_get_number(gs);
        if (index < 0){
            TEST_FAIL("gdk_screen_get_number: index error", NULL);
        }
    }
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_screen_get_rgb_visual
 * Params: GdkScreen *screen
 * Returns: GdkVisual*
 * Assertion: Gets a "preferred visual" chosen by GdkRGB for rendering image data on screen. In previous versions of GDK, this was the only visual GdkRGB could use for rendering. In current versions, it's simply the visual GdkRGB would have chosen as the optimal one in those previous versions. GdkRGB can now render to drawables with any visual.
 *            screen :  a GdkScreen
 *            Returns :   The GdkVisual chosen by GdkRGB.
 * Params Values: GdkScreen *screen
 * Expected: GdkVisual*  
 */
static void test_purpose_6()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GdkScreen *gs = gdk_screen_get_default();
    if (gs == NULL){
        TEST_FAIL("gdk_screen_get_rgb_visual: get default screen", NULL);
    }
    else{
        GdkVisual* gv = gdk_screen_get_rgb_visual(gs);
        if (!GDK_IS_VISUAL(gv)){
            TEST_FAIL("gdk_screen_get_rgb_visual: it's not a GDK_TYPE_VISUAL", NULL);
        }
    }
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_screen_get_n_monitors
 * Params: GdkScreen *screen
 * Returns: gint
 * Assertion: Returns the number of monitors which screen consists of.
 *            screen :  a GdkScreen.
 *            Returns :   number of monitors which screen consists of.
 * Params Values: GdkScreen *screen
 * Expected: gint  
 */
static void test_purpose_7()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GdkScreen *gs = gdk_screen_get_default();
    if (gs == NULL){
        TEST_FAIL("gdk_screen_get_n_monitors: get default screen", NULL);
    }
    else{
        gint n_mon = gdk_screen_get_n_monitors(gs);
        if (n_mon < 0){
            TEST_FAIL("gdk_screen_get_n_monitors: failed", NULL);
        }
    }
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for GdkScreen ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {test_purpose_5, 1},
    {test_purpose_6, 1},
    {test_purpose_7, 1},
    {NULL,0}
};
