/*
 * GdkVisuals methods tests
 * Testing GdkVisuals methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;


/*
 * Name: gdk_visual_get_system
 * Params: void
 * Returns: GdkVisual*
 * Assertion: Get the system'sdefault visual for the default GDK screen. This is the visual for the root window of the display. The return value should not be freed.
 * Params Values: void
 * Expected: system visual  
 */
static void test_purpose_1()
{
    GdkVisual *visual;
    
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    visual = gdk_visual_get_system ();
    
    if (!visual)
    {
        TEST_FAIL (get_not_created_msg (), NULL);
        return;
    }
    
    if (!GDK_IS_VISUAL (visual))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL);
        return;
    }
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_visual_get_best
 * Params: void
 * Returns: GdkVisual*
 * Assertion: Get the visual with the most available colors for the default GDK screen. The return value should not be freed.
 * Params Values: void
 * Expected: best visual  
 */
static void test_purpose_2()
{
    GdkVisual *best_visual;
    GdkVisual *visual;
    
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    best_visual = gdk_visual_get_best ();
    
    if (!best_visual)
    {
        TEST_FAIL (get_not_created_msg (), NULL);
        return;
    }
    
    if (!GDK_IS_VISUAL (best_visual))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL);
        return;
    }
    
    visual = gdk_visual_get_best ();
    
    if (!visual)
    {
        TEST_FAIL (get_not_created_msg (), NULL);
        return;
    }
    
    if (!GDK_IS_VISUAL (visual))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL);
        return;
    }
    
    if (visual->depth > best_visual->depth)
        TEST_FAIL ("best visual should have the most available colors", NULL);
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gdk_visual_get_best_depth
 * Params: void
 * Returns: gint
 * Assertion: Get the best available depth for the default GDK screen. "Best" means "largest," i.e. 32 preferred over 24 preferred over 8 bits per pixel.
 * Params Values: void
 * Expected: best available depth  
 */
static void test_purpose_3()
{
    gint depth;
    GdkVisual *visual;
    
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    depth = gdk_visual_get_best_depth ();
    
    if (depth == 0)
        TEST_FAIL ("color depth cannot be 0", NULL);
    
    visual = gdk_visual_get_best ();
    
    if (!visual)
    {
        TEST_FAIL (get_not_created_msg (), NULL);
        return;
    }
    
    if (!GDK_IS_VISUAL (visual))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL);
        return;
    }
    
    if (depth != visual->depth)
        TEST_FAIL ("best color depth should be %d, not %d", visual->depth, depth, NULL);
    
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for GdkVisuals ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {NULL,0}
};
