/*
 * GtkSettings methods tests
 * Testing GtkSettings methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;

/*
 * Name:   gtk_settings_get_default
 * Params: void
 * Returns: GtkSettings*
 * Assertion: Gets the GtkSettings object for the default GDK screen, creating it if necessary.
 * Params Values: void
 * Expected: GtkSettings*  
 */
static void test_purpose_1()
{
    GtkSettings* settings;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    settings = gtk_settings_get_default ();
    
    if (!settings)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_IS_SETTINGS (settings))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:   gtk_settings_get_for_screen
 * Params: GdkScreen *screen
 * Returns: GtkSettings*
 * Assertion: Gets the GtkSettings object for screen, creating it if necessary.
 * Params Values: gdk_screen_get_default ()
 * Expected: GtkSettings*  
 */
static void test_purpose_2()
{
    #define SCREEN gdk_screen_get_default ()
    
    GtkSettings* settings;
    GdkScreen* screen;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    screen = SCREEN;
    
    settings = gtk_settings_get_for_screen (screen);
    
    if (!settings)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_IS_SETTINGS (settings))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    #undef SCREEN
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:   gtk_settings_get_for_screen
 * Params: GdkScreen *screen
 * Returns: GtkSettings*
 * Assertion: Gets the GtkSettings object for screen, creating it if necessary.
 * Params Values: gtk_widget_get_screen (gtk_button_new ())
 * Expected: GtkSettings*  
 */
static void test_purpose_3()
{
    #define SCREEN gtk_widget_get_screen (gtk_button_new ())
    
    GtkSettings* settings;
    GdkScreen* screen;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    screen = SCREEN;
    
    settings = gtk_settings_get_for_screen (screen);
    
    if (!settings)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_IS_SETTINGS (settings))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    #undef SCREEN
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:   gtk_settings_get_for_screen
 * Params: GdkScreen *screen
 * Returns: GtkSettings*
 * Assertion: Gets the GtkSettings object for screen, creating it if necessary.
 * Params Values: gtk_widget_get_screen (gtk_button_new_with_label ("button"))
 * Expected: GtkSettings*  
 */
static void test_purpose_4()
{
    #define SCREEN gtk_widget_get_screen (gtk_button_new_with_label ("button"))
    
    GtkSettings* settings;
    GdkScreen* screen;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    screen = SCREEN;
    
    settings = gtk_settings_get_for_screen (screen);
    
    if (!settings)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_IS_SETTINGS (settings))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    #undef SCREEN
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:   gtk_settings_get_for_screen
 * Params: GdkScreen *screen
 * Returns: GtkSettings*
 * Assertion: Gets the GtkSettings object for screen, creating it if necessary.
 * Params Values: gtk_widget_get_screen (gtk_window_new (GTK_WINDOW_TOPLEVEL))
 * Expected: GtkSettings*  
 */
static void test_purpose_5()
{
    #define SCREEN gtk_widget_get_screen (gtk_window_new (GTK_WINDOW_TOPLEVEL))
    
    GtkSettings* settings;
    GdkScreen* screen;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    screen = SCREEN;
    
    settings = gtk_settings_get_for_screen (screen);
    
    if (!settings)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_IS_SETTINGS (settings))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    #undef SCREEN
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:   gtk_settings_get_for_screen
 * Params: GdkScreen *screen
 * Returns: GtkSettings*
 * Assertion: Gets the GtkSettings object for screen, creating it if necessary.
 * Params Values: gtk_widget_get_screen (gtk_calendar_new ())
 * Expected: GtkSettings*  
 */
static void test_purpose_6()
{
    #define SCREEN gtk_widget_get_screen (gtk_calendar_new ())
    
    GtkSettings* settings;
    GdkScreen* screen;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    screen = SCREEN;
    
    settings = gtk_settings_get_for_screen (screen);
    
    if (!settings)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_IS_SETTINGS (settings))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    #undef SCREEN
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:   gtk_settings_get_for_screen
 * Params: GdkScreen *screen
 * Returns: GtkSettings*
 * Assertion: Gets the GtkSettings object for screen, creating it if necessary.
 * Params Values: gtk_widget_get_screen (gtk_text_view_new ())
 * Expected: GtkSettings*  
 */
static void test_purpose_7()
{
    #define SCREEN gtk_widget_get_screen (gtk_text_view_new ())
    
    GtkSettings* settings;
    GdkScreen* screen;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    screen = SCREEN;
    
    settings = gtk_settings_get_for_screen (screen);
    
    if (!settings)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_IS_SETTINGS (settings))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    #undef SCREEN
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for GtkSettings ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {test_purpose_5, 1},
    {test_purpose_6, 1},
    {test_purpose_7, 1},
    {NULL,0}
};
