/*
 * GtkSocket methods tests
 * Testing GtkSocket methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>

/*
 * Name: gtk_socket_new 
 * Params: void
 * Returns: GtkWidget*
 * Assertion: Create a new empty GtkSocket.
 * Params Values: void
 * Expected: GtkSocket*  
 */
static void test_purpose_1()
{
    GtkWidget *widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_socket_new ();
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_IS_SOCKET (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_socket_add_id
 * Params: GtkSocket *socket, GdkNativeWindow id
 * Returns: void
 * Assertion: Adds an XEMBED client, such as a GtkPlug, to the GtkSocket.
 * Params Values: GtkSocket *socket, GdkNativeWindow id
 * Expected: void  
 */
static void test_purpose_2()
{
    GtkWidget* window;
    GtkWidget* widget;
    GtkWidget* plug;
    gint  pfds[2];
    gchar buff[30];
    GdkNativeWindow id;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    
    
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    if (!window)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    widget = gtk_socket_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_IS_SOCKET (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    gtk_container_add (GTK_CONTAINER (window), widget);
    
    gtk_widget_realize (widget);
    
    plug = gtk_plug_new ((GdkNativeWindow) 0);
    if (!plug)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    gtk_socket_add_id (GTK_SOCKET (widget), gtk_plug_get_id (GTK_PLUG (plug)));
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_socket_get_id
 * Params: GtkSocket *socket, GdkNativeWindow id
 * Returns: GdkNativeWindow
 * Assertion: Gets the window ID of a GtkSocket widget, which can then be used to create a client embedded inside the socket, for instance with gtk_plug_new().
 * Params Values: GtkSocket *socket, GdkNativeWindow id
 * Expected: GdkNativeWindow  
 */
static void test_purpose_3()
{
    GtkWidget* window;
    GtkWidget* widget;
    GtkWidget* plug;
    gint  pfds[2];
    gchar buff[30];
    GdkNativeWindow id;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    
    pipe (pfds);
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    if (!window)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    widget = gtk_socket_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
    
    if (!GTK_IS_SOCKET (widget))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    gtk_container_add (GTK_CONTAINER (window), widget);
    gtk_widget_realize (widget);
    
    if (fork())
    {
        id = gtk_socket_get_id (GTK_SOCKET (widget));
        write (pfds[1], &id, sizeof (GdkNativeWindow));
    
        wait(NULL);
        close (pfds[1]);
    
    } else {
    
        read (pfds[0], &id, sizeof (GdkNativeWindow));
        plug = gtk_plug_new (id);
    
        if (!plug)
        {
            TEST_FAIL (get_not_created_msg(), NULL); 
            return;
        }
        if (!GTK_PLUG (plug)->socket_window)
        {
            TEST_FAIL ("plug->socket_window is NULL", NULL); 
        }
        close (pfds[0]);
        exit (0);
    }
    
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for GtkSocket ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {NULL,0}
};
