/*
 * SpawnProc methods tests
 * Testing SpawnProc methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;
// used for forcing to C locale for bug 1669
#include <locale.h>

static void func(gpointer data)
{
    return;
}

/*
 * Name: g_spawn_command_line_sync
 * Params: const gchar *command_line, gchar **standard_output, gchar **standard_error, gint *exit_status, GError **error
 * Returns: gboolean
 * Assertion: A simple version of g_spawn_sync() with little-used parameters removed, taking a command line instead of an argument vector. See g_spawn_sync() for full details. command_line will be parsed by g_shell_parse_argv(). Unlike g_spawn_sync(), the G_SPAWN_SEARCH_PATH flag is enabled. Note that G_SPAWN_SEARCH_PATH can have security implications, so consider using g_spawn_sync() directly if appropriate. Possible errors are those from g_spawn_sync() and those from g_shell_parse_argv().
 *            command_line : a command line
 *            standard_output : return location for child output
 *            standard_error : return location for child errors
 *            exit_status : return location for child exit status
 *            error : return location for errors
 *            Returns : TRUE on success, FALSE if an error was set
 * Params Values: const gchar *command_line, gchar **standard_output, gchar **standard_error, gint *exit_status, GError **error
 * Expected: gboolean  
 */
static void test_purpose_1()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GError *err = NULL;
    gchar *output = NULL;
    gboolean retb;
    
    // save locale
    char *cur_locale = NULL;
    cur_locale = setlocale(LC_ALL, "C");
    
    gchar fail_ret[] = "Failed to execute child process";
    
    retb = g_spawn_command_line_sync("Can not be run", NULL, NULL, NULL, &err);
    if (retb){
        TEST_FAIL("g_spawn_command_line_sync: failed", NULL);
    }
    else{
        if (g_ascii_strncasecmp(err->message, fail_ret, sizeof(fail_ret)-1) != 0){
            TEST_FAIL("g_spawn_command_line_sync: %s", err->message);
        }
    }
    g_clear_error(&err);
    
    retb = g_spawn_command_line_sync("/bin/sh -c 'echo hello'", &output, NULL, NULL, &err);
    if (!retb){
        TEST_FAIL("g_spawn_command_line_sync: failed", NULL);
    }
    else{
        if (strcmp(output, "hello\n") != 0){
            TEST_FAIL("g_spawn_command_line_sync: %s", output);
        }
    }
    g_clear_error(&err);
    
    // restore locale
    setlocale(LC_ALL, cur_locale);
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_spawn_async_with_pipes
 * Params: const gchar *working_directory, gchar **argv, gchar **envp, GSpawnFlags flags, GSpawnChildSetupFunc child_setup, gpointer user_data, GPid *child_pid, gint *standard_input, gint *standard_output, gint *standard_error, GError **error
 * Returns: gboolean
 * Assertion: Executes a child program asynchronously (your program will not block waiting for the child to exit). The child program is specified by the only argument that must be provided, argv. argv should be a NULL-terminated array of strings, to be passed as the argument vector for the child. The first string in argv is of course the name of the program to execute. By default, the name of the program must be a full path; the PATH shell variable will only be searched if you pass the G_SPAWN_SEARCH_PATH flag.
 * envp is a NULL-terminated array of strings, where each string has the form KEY=VALUE. This will become the child's environment. If envp is NULL, the child inherits its parent's environment.
 * flags should be the bitwise OR of any flags you want to affect the function's behaviour. The G_SPAWN_DO_NOT_REAP_CHILD means that the child will not automatically be reaped; you must use a GChildWatch source to be notified about the death of the child process. Eventually you must call g_spawn_close_pid() on the child_pid, in order to free resources which may be associated with the child process. (On Unix, using a GChildWatch source is equivalent to calling waitpid() or handling the SIGCHLD signal manually. On Windows, calling g_spawn_close_pid() is equivalent to calling CloseHandle() on the process handle returned in child_pid).
 * G_SPAWN_LEAVE_DESCRIPTORS_OPEN means that the parent's open file descriptors will be inherited by the child; otherwise all descriptors except stdin/stdout/stderr will be closed before calling exec() in the child. G_SPAWN_SEARCH_PATH means that argv[0] need not be an absolute path, it will be looked for in the user's PATH. G_SPAWN_STDOUT_TO_DEV_NULL means that the child's standard output will be discarded, instead of going to the same location as the parent's standard output. If you use this flag, standard_output must be NULL. G_SPAWN_STDERR_TO_DEV_NULL means that the child's standard error will be discarded, instead of going to the same location as the parent's standard error. If you use this flag, standard_error must be NULL. G_SPAWN_CHILD_INHERITS_STDIN means that the child will inherit the parent's standard input (by default, the child's standard input is attached to /dev/null). If you use this flag, standard_input must be NULL. G_SPAWN_FILE_AND_ARGV_ZERO means that the first element of argv is the file to execute, while the remaining elements are the actual argument vector to pass to the file. Normally g_spawn_async_with_pipes() uses argv[0] as the file to execute, and passes all of argv to the child.
 * child_setup and user_data are a function and user data. On POSIX platforms, the function is called in the child after GLib has performed all the setup it plans to perform (including creating pipes, closing file descriptors, etc.) but before calling exec(). That is, child_setup is called just before calling exec() in the child. Obviously actions taken in this function will only affect the child, not the parent. On Windows, there is no separate fork() and exec() functionality. Child processes are created and run right away with one API call, CreateProcess(). child_setup is called in the parent process just before creating the child process. You should carefully consider what you do in child_setup if you intend your software to be portable to Windows.
 * If non-NULL, child_pid will on Unix be filled with the child's process ID. You can use the process ID to send signals to the child, or to waitpid() if you specified the G_SPAWN_DO_NOT_REAP_CHILD flag. On Windows, child_pid will be filled with a handle to the child process only if you specified the G_SPAWN_DO_NOT_REAP_CHILD flag. You can then access the child process using the Win32 API, for example wait for its termination with the WaitFor*() functions, or examine its exit code with GetExitCodeProcess(). You should close the handle with CloseHandle() when you no longer need it.
 * If non-NULL, the standard_input, standard_output, standard_error locations will be filled with file descriptors for writing to the child's standard input or reading from its standard output or standard error. The caller of g_spawn_async_with_pipes() must close these file descriptors when they are no longer in use. If these parameters are NULL, the corresponding pipe won't be created.
 * 
 * If standard_input is NULL, the child's standard input is attached to /dev/null unless G_SPAWN_CHILD_INHERITS_STDIN is set.
 * 
 * If standard_error is NULL, the child's standard error goes to the same location as the parent's standard error unless G_SPAWN_STDERR_TO_DEV_NULL is set.
 * 
 * If standard_output is NULL, the child's standard output goes to the same location as the parent's standard output unless G_SPAWN_STDOUT_TO_DEV_NULL is set.
 * 
 * error can be NULL to ignore errors, or non-NULL to report errors. If an error is set, the function returns FALSE. Errors are reported even if they occur in the child (for example if the executable in argv[0] is not found). Typically the message field of returned errors should be displayed to users. Possible errors are those from the G_SPAWN_ERROR domain.
 * 
 * If an error occurs, child_pid, standard_input, standard_output, and standard_error will not be filled with valid values.
 * 
 * If child_pid is not NULL and an error does not occur then the returned pid must be closed using g_spawn_close_pid().
 *            working_directory :   child's current working directory, or NULL to inherit parent's
 *            argv :  child's argument vector
 *            envp :  child's environment, or NULL to inherit parent's
 *            flags : flags from GSpawnFlags
 *            child_setup :   function to run in the child just before exec()
 *            user_data : user data for child_setup
 *            child_pid : return location for child process ID, or NULL
 *            standard_input :    return location for file descriptor to write to child's stdin, or NULL
 *            standard_output :   return location for file descriptor to read child's stdout, or NULL
 *            standard_error :    return location for file descriptor to read child's stderr, or NULL
 *            error : return location for error
 *            Returns :   TRUE on success, FALSE if an error was set
 * Params Values: const gchar *working_directory, gchar **argv, gchar **envp, GSpawnFlags flags, GSpawnChildSetupFunc child_setup, gpointer user_data, GPid *child_pid, gint *standard_input, gint *standard_output, gint *standard_error, GError **erro
 * Expected: gboolean  
 */
static void test_purpose_2()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GPid gpid;
    gint s_in;
    gint s_out;
    gint s_err;
    GError *err = NULL;
    gchar *comm[] = {"ls"};
    gchar **argv = NULL;
    
    argv = g_strdupv(comm);
    if (!g_spawn_async_with_pipes("/bin/", argv, NULL, G_SPAWN_LEAVE_DESCRIPTORS_OPEN, func, NULL, &gpid, &s_in, &s_out, &s_err, &err)){
        TEST_FAIL("g_spawn_async_with_pipes: %s", err->message);
    }
    else{
        if (&gpid != NULL){
            g_spawn_close_pid(gpid);
        }
    }
    g_clear_error(&err);
    g_strfreev(argv);
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for SpawnProc ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {NULL,0}
};
