/*
 * UniMani methods tests
 * Testing UniMani methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;
#include <locale.h>

typedef struct {
    gchar *key;
    gchar *str;
} collate_key;


/*
 * Name: g_utf8_validate
 * Params: const gchar *str, gssize max_len, const gchar **end
 * Returns: gboolean
 * Assertion: Validates UTF-8 encoded text. str is the text to validate; if str is nul-terminated, then max_len can be -1, otherwise max_len should be the number of bytes to validate. If end is non-NULL, then the end of the valid range will be stored there (i.e. the start of the first invalid character if some bytes were invalid, or the end of the text being validated otherwise).
 * Note that g_utf8_validate() returns FALSE if max_len is positive and NUL is met before max_len bytes have been read.
 * Returns TRUE if all of str was valid. Many GLib and GTK+ routines require valid UTF-8 as input; so data read from a file or the network should be checked with g_utf8_validate() before doing anything else with it.
 *            str : a pointer to character data
 *            max_len : max bytes to validate, or -1 to go until NUL
 *            end : return location for end of valid data
 *            Returns : TRUE if the text was valid UTF-8 
 * Params Values: "123", 8, NULL, format = "%s"
 * Expected: "%s"  
 */
static void test_purpose_1()
{
    #define STR    "123"
    #define SIZE   8
    #define END    NULL
    #define FORMAT "%s"
    #define RETB   FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GString *str = g_string_sized_new(10);
    g_string_printf(str, FORMAT, STR);
    if (g_utf8_validate(str->str, SIZE, END) != RETB){
        TEST_FAIL("g_utf8_validate: failed", NULL);
    }
    g_string_free(str, TRUE);
    #undef STR
    #undef SIZE
    #undef END
    #undef FORMAT
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_utf8_validate
 * Params: const gchar *str, gssize max_len, const gchar **end
 * Returns: gboolean
 * Assertion: Validates UTF-8 encoded text. str is the text to validate; if str is nul-terminated, then max_len can be -1, otherwise max_len should be the number of bytes to validate. If end is non-NULL, then the end of the valid range will be stored there (i.e. the start of the first invalid character if some bytes were invalid, or the end of the text being validated otherwise).
 * Note that g_utf8_validate() returns FALSE if max_len is positive and NUL is met before max_len bytes have been read.
 * Returns TRUE if all of str was valid. Many GLib and GTK+ routines require valid UTF-8 as input; so data read from a file or the network should be checked with g_utf8_validate() before doing anything else with it.
 *            str : a pointer to character data
 *            max_len : max bytes to validate, or -1 to go until NUL
 *            end : return location for end of valid data
 *            Returns : TRUE if the text was valid UTF-8 
 * Params Values: 0xe4, 0xb8, 0xad, -1, NULL, format = "%c%c%c"
 * Expected: "%c%c%c"  
 */
static void test_purpose_2()
{
    #define STR    0xe4, 0xb8, 0xad
    #define SIZE   -1
    #define END    NULL
    #define FORMAT "%c%c%c"
    #define RETB   TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GString *str = g_string_sized_new(10);
    g_string_printf(str, FORMAT, STR);
    if (g_utf8_validate(str->str, SIZE, END) != RETB){
        TEST_FAIL("g_utf8_validate: failed", NULL);
    }
    g_string_free(str, TRUE);
    #undef STR
    #undef SIZE
    #undef END
    #undef FORMAT
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_utf8_validate
 * Params: const gchar *str, gssize max_len, const gchar **end
 * Returns: gboolean
 * Assertion: Validates UTF-8 encoded text. str is the text to validate; if str is nul-terminated, then max_len can be -1, otherwise max_len should be the number of bytes to validate. If end is non-NULL, then the end of the valid range will be stored there (i.e. the start of the first invalid character if some bytes were invalid, or the end of the text being validated otherwise).
 * Note that g_utf8_validate() returns FALSE if max_len is positive and NUL is met before max_len bytes have been read.
 * Returns TRUE if all of str was valid. Many GLib and GTK+ routines require valid UTF-8 as input; so data read from a file or the network should be checked with g_utf8_validate() before doing anything else with it.
 *            str : a pointer to character data
 *            max_len : max bytes to validate, or -1 to go until NUL
 *            end : return location for end of valid data
 *            Returns : TRUE if the text was valid UTF-8 
 * Params Values: 0xd0, 0xd6, -1, NULL, format = "%c%c"
 * Expected: "%c%c"  
 */
static void test_purpose_3()
{
    #define STR    0xd0, 0xd6
    #define SIZE   -1
    #define END    NULL
    #define FORMAT "%c%c"
    #define RETB   FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GString *str = g_string_sized_new(10);
    g_string_printf(str, FORMAT, STR);
    if (g_utf8_validate(str->str, SIZE, END) != RETB){
        TEST_FAIL("g_utf8_validate: failed", NULL);
    }
    g_string_free(str, TRUE);
    #undef STR
    #undef SIZE
    #undef END
    #undef FORMAT
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_utf8_get_char
 * Params: const gchar *p
 * Returns: gunichar
 * Assertion: Converts a sequence of bytes encoded as UTF-8 to a Unicode character. If p does not point to a valid UTF-8 encoded character, results are undefined. If you are not sure that the bytes are complete valid Unicode characters, you should use g_utf8_get_char_validated() instead.
 *            p : a pointer to Unicode character encoded as UTF-8
 *            Returns : the resulting character
 * Params Values: 0xe4, 0xb8, 0xad, Format = "%c%c%c"
 * Expected: 0x4e2d  
 */
static void test_purpose_4()
{
    #define STR    0xe4, 0xb8, 0xad
    #define FORMAT "%c%c%c"
    #define RETU   0x4e2d
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GString *gs = g_string_sized_new(10);
    g_string_printf(gs, FORMAT, STR);
    if (g_utf8_get_char(gs->str) != RETU){
        TEST_FAIL("g_utf8_get_char: failed", NULL);
    }
    g_string_free(gs, TRUE);
    #undef STR
    #undef FORMAT
    #undef RETU
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_utf8_strlen
 * Params: const gchar *p, gssize max
 * Returns: glong
 * Assertion: Returns the length of the string in characters.
 *            p :  pointer to the start of a UTF-8 encoded string.
 *            max : the maximum number of bytes to examine. If max is less than 0, then the string is assumed to be nul-terminated. If max is 0, p will not be examined and may be NULL.
 *            Returns : the length of the string in characters 
 * Params Values: 0xe4, 0xb8, 0xad, 0xe5, 0x8d, 0x8e, 0xe4, 0xba, 0xba, 0xe6, 0xb0, 0x91, 0xe5, 0x85, 0xb1, 0xe5, 0x92, 0x8c, 0xe5, 0x9b, 0xbd, Maxlen = -1, Format = "%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c"
 * Expected: 7  
 */
static void test_purpose_5()
{
    #define STR    0xe4, 0xb8, 0xad, 0xe5, 0x8d, 0x8e, 0xe4, 0xba, 0xba, 0xe6, 0xb0, 0x91, 0xe5, 0x85, 0xb1, 0xe5, 0x92, 0x8c, 0xe5, 0x9b, 0xbd
    #define MAXLEN -1
    #define FORMAT "%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c"
    #define RETU   7
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    glong len = 0;
    GString *gs = g_string_sized_new(10);
    g_string_printf(gs, FORMAT, STR);
    len = g_utf8_strlen(gs->str, MAXLEN);
    if (len != RETU){
        TEST_FAIL("g_utf8_strlen: should be %d, now is %d", RETU, len);
    }
    g_string_free(gs, TRUE);
    #undef STR
    #undef MAXLEN
    #undef FORMAT
    #undef RETU
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_utf8_strlen
 * Params: const gchar *p, gssize max
 * Returns: glong
 * Assertion: Returns the length of the string in characters.
 *            p :  pointer to the start of a UTF-8 encoded string.
 *            max : the maximum number of bytes to examine. If max is less than 0, then the string is assumed to be nul-terminated. If max is 0, p will not be examined and may be NULL.
 *            Returns : the length of the string in characters 
 * Params Values: 0xe4, 0xb8, 0xad, 0xe5, 0x8d, 0x8e, 0xe4, 0xba, 0xba, 0xe6, 0xb0, 0x91, 0xe5, 0x85, 0xb1, 0xe5, 0x92, 0x8c, 0xe5, 0x9b, 0xbd, Maxlen = 15, Format = "%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c"
 * Expected: 5  
 */
static void test_purpose_6()
{
    #define STR    0xe4, 0xb8, 0xad, 0xe5, 0x8d, 0x8e, 0xe4, 0xba, 0xba, 0xe6, 0xb0, 0x91, 0xe5, 0x85, 0xb1, 0xe5, 0x92, 0x8c, 0xe5, 0x9b, 0xbd
    #define MAXLEN 15
    #define FORMAT "%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c"
    #define RETU   5
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    glong len = 0;
    GString *gs = g_string_sized_new(10);
    g_string_printf(gs, FORMAT, STR);
    len = g_utf8_strlen(gs->str, MAXLEN);
    if (len != RETU){
        TEST_FAIL("g_utf8_strlen: should be %d, now is %d", RETU, len);
    }
    g_string_free(gs, TRUE);
    #undef STR
    #undef MAXLEN
    #undef FORMAT
    #undef RETU
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_utf8_collate
 * Params: const gchar *str1, const gchar *str2
 * Returns: gint
 * Assertion: Compares two strings for ordering using the linguistically correct rules for the current locale. When sorting a large number of strings, it will be significantly faster to obtain collation keys with g_utf8_collate_key() and compare the keys with strcmp() when sorting instead of sorting the original strings.
 *            str1 : a UTF-8 encoded string
 *            str2 : a UTF-8 encoded string
 *            Returns : < 0 if str1 compares before str2, 0 if they compare equal, > 0 if str1 compares after str2. 
 * Params Values: "a", "b", Format = -1
 * Expected: <%3%>  
 */
static void test_purpose_7()
{
    #define STR1 "a"
    #define STR2 "b"
    #define RETI -1
    
    collate_key a, b;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    a.key = g_utf8_collate_key(STR1, -1);
    a.str = STR1;
    b.key = g_utf8_collate_key(STR2, -1);
    b.str = STR2;
    
    if (g_utf8_collate(a.str, b.str) != RETI){
        TEST_FAIL("g_utf8_collate: should be %d", RETI);
    }
    
    #undef STR1
    #undef STR2
    #undef RETI
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_utf8_collate
 * Params: const gchar *str1, const gchar *str2
 * Returns: gint
 * Assertion: Compares two strings for ordering using the linguistically correct rules for the current locale. When sorting a large number of strings, it will be significantly faster to obtain collation keys with g_utf8_collate_key() and compare the keys with strcmp() when sorting instead of sorting the original strings.
 *            str1 : a UTF-8 encoded string
 *            str2 : a UTF-8 encoded string
 *            Returns : < 0 if str1 compares before str2, 0 if they compare equal, > 0 if str1 compares after str2. 
 * Params Values: "b", "a", Format = 1
 * Expected: <%3%>  
 */
static void test_purpose_8()
{
    #define STR1 "b"
    #define STR2 "a"
    #define RETI 1
    
    collate_key a, b;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    a.key = g_utf8_collate_key(STR1, -1);
    a.str = STR1;
    b.key = g_utf8_collate_key(STR2, -1);
    b.str = STR2;
    
    if (g_utf8_collate(a.str, b.str) != RETI){
        TEST_FAIL("g_utf8_collate: should be %d", RETI);
    }
    
    #undef STR1
    #undef STR2
    #undef RETI
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_utf8_collate
 * Params: const gchar *str1, const gchar *str2
 * Returns: gint
 * Assertion: Compares two strings for ordering using the linguistically correct rules for the current locale. When sorting a large number of strings, it will be significantly faster to obtain collation keys with g_utf8_collate_key() and compare the keys with strcmp() when sorting instead of sorting the original strings.
 *            str1 : a UTF-8 encoded string
 *            str2 : a UTF-8 encoded string
 *            Returns : < 0 if str1 compares before str2, 0 if they compare equal, > 0 if str1 compares after str2. 
 * Params Values: "a", "a", Format = 0
 * Expected: <%3%>  
 */
static void test_purpose_9()
{
    #define STR1 "a"
    #define STR2 "a"
    #define RETI 0
    
    collate_key a, b;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    a.key = g_utf8_collate_key(STR1, -1);
    a.str = STR1;
    b.key = g_utf8_collate_key(STR2, -1);
    b.str = STR2;
    
    if (g_utf8_collate(a.str, b.str) != RETI){
        TEST_FAIL("g_utf8_collate: should be %d", RETI);
    }
    
    #undef STR1
    #undef STR2
    #undef RETI
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isspace
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is a space, tab, or line separator (newline, carriage return, etc.). Given some UTF-8 text, obtain a character value with g_utf8_get_char().
 * (Note: don't use this to do word breaking; you have to use Pango or equivalent to get word breaking right, the algorithm is fairly complex.)
 *            c : a Unicode character
 *            Returns : TRUE if c is a punctuation character 
 * Params Values: 0x090a
 * Expected: TRUE  
 */
static void test_purpose_10()
{
    #define UNICH    0x090a
    #define RETB     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    gunichar c;
    GString *gs = g_string_sized_new(10);
    g_string_printf(gs, "%c", UNICH);
    c = g_utf8_get_char(gs->str);
    if (g_unichar_isspace(c) != RETB){
        TEST_FAIL("g_unichar_isspace: failed", NULL);
    }
    g_string_free(gs, TRUE);
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isspace
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is a space, tab, or line separator (newline, carriage return, etc.). Given some UTF-8 text, obtain a character value with g_utf8_get_char().
 * (Note: don't use this to do word breaking; you have to use Pango or equivalent to get word breaking right, the algorithm is fairly complex.)
 *            c : a Unicode character
 *            Returns : TRUE if c is a punctuation character 
 * Params Values: 0x20
 * Expected: TRUE  
 */
static void test_purpose_11()
{
    #define UNICH    0x20
    #define RETB     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    gunichar c;
    GString *gs = g_string_sized_new(10);
    g_string_printf(gs, "%c", UNICH);
    c = g_utf8_get_char(gs->str);
    if (g_unichar_isspace(c) != RETB){
        TEST_FAIL("g_unichar_isspace: failed", NULL);
    }
    g_string_free(gs, TRUE);
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_utf8_offset_to_pointer
 * Params: const gchar *str, glong offset
 * Returns: gchar*
 * Assertion: Converts from an integer character offset to a pointer to a position within the string.
 *            str : a UTF-8 encoded string
 *            offset : a character offset within str
 *            Returns : the resulting pointer 
 * Params Values: 0xe4, 0xb8, 0xad, 0xe5, 0x9b, 0xbd, "%c%c%c%c%c%c", 1
 * Expected: 0xe5, 0x9b, 0xbd  
 */
static void test_purpose_12()
{
    #define STR    0xe4, 0xb8, 0xad, 0xe5, 0x9b, 0xbd
    #define FORMAT "%c%c%c%c%c%c"
    #define OFFSET 1
    #define RETS   0xe5, 0x9b, 0xbd
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GString *gs = g_string_sized_new(10);
    GString *rets = g_string_sized_new(10);
    g_string_printf(gs, FORMAT, STR);
    g_string_printf(rets, "%c%c%c", RETS);
    if (strcmp(rets->str, g_utf8_offset_to_pointer(gs->str, OFFSET)) != 0){
        TEST_FAIL("g_utf8_offset_to_pointer: failed", NULL);
    }
    g_string_free(gs, TRUE);
    g_string_free(rets, TRUE);
    #undef STR
    #undef FORMAT
    #undef OFFSET
    #undef RETS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_utf8_pointer_to_offset
 * Params: const gchar *str, const gchar *pos
 * Returns: glong
 * Assertion: Converts from a pointer to position within a string to a integer character offset. 
 *            str : a UTF-8 encoded string
 *            pos : a pointer to a position within str
 *            Returns : the resulting character offset
 * Params Values: 0xe4, 0xb8, 0xad, 0xe5, 0x9b, 0xbd, "%c%c%c%c%c%c"
 * Expected: 1, 0xe5, 0x9b, 0xbd  
 */
static void test_purpose_13()
{
    #define STR    0xe4, 0xb8, 0xad, 0xe5, 0x9b, 0xbd
    #define FORMAT "%c%c%c%c%c%c"
    #define OFFSET 1
    #define RETS   0xe5, 0x9b, 0xbd
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    gchar *ptr;
    glong offset;
    GString *gs = g_string_sized_new(10);
    GString *rets = g_string_sized_new(10);
    g_string_printf(gs, FORMAT, STR);
    g_string_printf(rets, "%c%c%c", RETS);
    ptr = g_utf8_offset_to_pointer(gs->str, OFFSET);
    offset = g_utf8_pointer_to_offset(gs->str, ptr);
    if (offset != OFFSET){
        TEST_FAIL("g_utf8_pointer_to_offset: should be %d, now is %d", OFFSET, offset);
    }
    g_string_free(gs, TRUE);
    g_string_free(rets, TRUE);
    #undef STR
    #undef FORMAT
    #undef OFFSET
    #undef RETS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isgraph
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is printable and not a space (returns FALSE for control characters, format characters, and spaces). g_unichar_isprint() is similar, but returns TRUE for spaces. Given some UTF-8 text, obtain a character value with g_utf8_get_char().
 *            c : a Unicode character
 *            Returns : TRUE if c is printable unless it's a space 
 * Params Values: 0x0020
 * Expected: FALSE  
 */
static void test_purpose_14()
{
    #define UNICH    0x0020
    #define RETB     FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isgraph(UNICH) != RETB){
        TEST_FAIL("g_unichar_isgraph: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isgraph
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is printable and not a space (returns FALSE for control characters, format characters, and spaces). g_unichar_isprint() is similar, but returns TRUE for spaces. Given some UTF-8 text, obtain a character value with g_utf8_get_char().
 *            c : a Unicode character
 *            Returns : TRUE if c is printable unless it's a space 
 * Params Values: 0x0079
 * Expected: TRUE  
 */
static void test_purpose_15()
{
    #define UNICH    0x0079
    #define RETB     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isgraph(UNICH) != RETB){
        TEST_FAIL("g_unichar_isgraph: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_utf8_find_next_char
 * Params: const gchar *p, const gchar *end
 * Returns: gchar*
 * Assertion: Finds the start of the next UTF-8 character in the string after p.
 *            p : a pointer to a position within a UTF-8 encoded string
 *            end : a pointer to the end of the string, or NULL to indicate that the string is nul-terminated, in which case the returned value will be
 *            Returns : a pointer to the found character or NULL 
 * Params Values: 0, NULL, "%c%c%c%c%c%c%c%c%c%c%c%c"
 * Expected: 0xe5, 0x9b, 0xbd, 0xe6, 0x97, 0xa5, 0xe6, 0x9c, 0xac  
 */
static void test_purpose_16()
{
    #define P2P    0
    #define END    NULL
    #define FORM1  "%c%c%c%c%c%c%c%c%c%c%c%c"
    #define RETS   0xe5, 0x9b, 0xbd, 0xe6, 0x97, 0xa5, 0xe6, 0x9c, 0xac
    #define FORM2  "%c%c%c%c%c%c%c%c%c"
    #define STR    0xe4, 0xb8, 0xad, 0xe5, 0x9b, 0xbd, 0xe6, 0x97, 0xa5, 0xe6, 0x9c, 0xac
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GString *gs1 = g_string_sized_new(10);
    GString *gs2 = g_string_sized_new(10);
    g_string_printf(gs1, FORM1, STR);
    g_string_printf(gs2, FORM2, RETS);
    
    gchar *p = g_utf8_offset_to_pointer(gs1->str, P2P);
    if (strcmp(g_utf8_find_next_char(p, END), gs2->str) != 0){
        TEST_FAIL("g_utf8_find_next_char: failed", NULL);
    }
    
    g_string_free(gs1, TRUE);
    g_string_free(gs2, TRUE);
    #undef P2P
    #undef END
    #undef FORM1
    #undef RETS
    #undef FORM2
    #undef STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_utf8_find_next_char
 * Params: const gchar *p, const gchar *end
 * Returns: gchar*
 * Assertion: Finds the start of the next UTF-8 character in the string after p.
 *            p : a pointer to a position within a UTF-8 encoded string
 *            end : a pointer to the end of the string, or NULL to indicate that the string is nul-terminated, in which case the returned value will be
 *            Returns : a pointer to the found character or NULL 
 * Params Values: 1, NULL, "%c%c%c%c%c%c%c%c%c%c%c%c"
 * Expected: 0xe6, 0x97, 0xa5, 0xe6, 0x9c, 0xac  
 */
static void test_purpose_17()
{
    #define P2P    1
    #define END    NULL
    #define FORM1  "%c%c%c%c%c%c%c%c%c%c%c%c"
    #define RETS   0xe6, 0x97, 0xa5, 0xe6, 0x9c, 0xac
    #define FORM2  "%c%c%c%c%c%c"
    #define STR    0xe4, 0xb8, 0xad, 0xe5, 0x9b, 0xbd, 0xe6, 0x97, 0xa5, 0xe6, 0x9c, 0xac
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GString *gs1 = g_string_sized_new(10);
    GString *gs2 = g_string_sized_new(10);
    g_string_printf(gs1, FORM1, STR);
    g_string_printf(gs2, FORM2, RETS);
    
    gchar *p = g_utf8_offset_to_pointer(gs1->str, P2P);
    if (strcmp(g_utf8_find_next_char(p, END), gs2->str) != 0){
        TEST_FAIL("g_utf8_find_next_char: failed", NULL);
    }
    
    g_string_free(gs1, TRUE);
    g_string_free(gs2, TRUE);
    #undef P2P
    #undef END
    #undef FORM1
    #undef RETS
    #undef FORM2
    #undef STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isprint
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is printable. Unlike g_unichar_isgraph(), returns TRUE for spaces. Given some UTF-8 text, obtain a character value with g_utf8_get_char().
 *            c : a Unicode character
 *            Returns : TRUE if c is printable 
 * Params Values: 0x0020
 * Expected: TRUE  
 */
static void test_purpose_18()
{
    #define UNICH    0x0020
    #define RETB     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isprint(UNICH) != RETB){
        TEST_FAIL("g_unichar_isprint: failed", NULL);
    }
    #undef RETB
    #undef UNICH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isprint
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is printable. Unlike g_unichar_isgraph(), returns TRUE for spaces. Given some UTF-8 text, obtain a character value with g_utf8_get_char().
 *            c : a Unicode character
 *            Returns : TRUE if c is printable 
 * Params Values: 0x0079
 * Expected: TRUE  
 */
static void test_purpose_19()
{
    #define UNICH    0x0079
    #define RETB     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isprint(UNICH) != RETB){
        TEST_FAIL("g_unichar_isprint: failed", NULL);
    }
    #undef RETB
    #undef UNICH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isprint
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is printable. Unlike g_unichar_isgraph(), returns TRUE for spaces. Given some UTF-8 text, obtain a character value with g_utf8_get_char().
 *            c : a Unicode character
 *            Returns : TRUE if c is printable 
 * Params Values: 0x001a
 * Expected: FALSE  
 */
static void test_purpose_20()
{
    #define UNICH    0x001a
    #define RETB     FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isprint(UNICH) != RETB){
        TEST_FAIL("g_unichar_isprint: failed", NULL);
    }
    #undef RETB
    #undef UNICH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isalpha
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is alphabetic (i.e. a letter). Given some UTF-8 text, obtain a character value with g_utf8_get_char().
 *            c : a Unicode character  
 *            Returns : TRUE if c is an alphabetic character   
 * Params Values: 0x4e2d
 * Expected: TRUE  
 */
static void test_purpose_21()
{
    #define UNICH    0x4e2d
    #define RETB     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isalpha(UNICH) != RETB){
        TEST_FAIL("g_unichar_isalpha: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isalpha
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is alphabetic (i.e. a letter). Given some UTF-8 text, obtain a character value with g_utf8_get_char().
 *            c : a Unicode character  
 *            Returns : TRUE if c is an alphabetic character   
 * Params Values: 0x0020
 * Expected: FALSE  
 */
static void test_purpose_22()
{
    #define UNICH    0x0020
    #define RETB     FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isalpha(UNICH) != RETB){
        TEST_FAIL("g_unichar_isalpha: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isalpha
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is alphabetic (i.e. a letter). Given some UTF-8 text, obtain a character value with g_utf8_get_char().
 *            c : a Unicode character  
 *            Returns : TRUE if c is an alphabetic character   
 * Params Values: 0x0031
 * Expected: FALSE  
 */
static void test_purpose_23()
{
    #define UNICH    0x0031
    #define RETB     FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isalpha(UNICH) != RETB){
        TEST_FAIL("g_unichar_isalpha: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isalnum
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is alphanumeric. Given some UTF-8 text, obtain a character value with g_utf8_get_char(). 
 *            c : a Unicode character  
 *            Returns : TRUE if c is an alphanumeric character   
 * Params Values: 0x4e2d
 * Expected: TRUE  
 */
static void test_purpose_24()
{
    #define UNICH    0x4e2d
    #define RETB     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isalnum(UNICH) != RETB){
        TEST_FAIL("g_unichar_isalnum: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isalnum
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is alphanumeric. Given some UTF-8 text, obtain a character value with g_utf8_get_char(). 
 *            c : a Unicode character  
 *            Returns : TRUE if c is an alphanumeric character   
 * Params Values: 0x0020
 * Expected: FALSE  
 */
static void test_purpose_25()
{
    #define UNICH    0x0020
    #define RETB     FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isalnum(UNICH) != RETB){
        TEST_FAIL("g_unichar_isalnum: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isalnum
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is alphanumeric. Given some UTF-8 text, obtain a character value with g_utf8_get_char(). 
 *            c : a Unicode character  
 *            Returns : TRUE if c is an alphanumeric character   
 * Params Values: 0x0031
 * Expected: TRUE  
 */
static void test_purpose_26()
{
    #define UNICH    0x0031
    #define RETB     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isalnum(UNICH) != RETB){
        TEST_FAIL("g_unichar_isalnum: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isdigit
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is numeric (i.e. a digit). This covers ASCII 0-9 and also digits in other languages/scripts. Given some UTF-8 text, obtain a character value with g_utf8_get_char(). 
 *            c : a Unicode character  
 *            Returns : TRUE if c is a digit
 * Params Values: 0x4e2d
 * Expected: FALSE  
 */
static void test_purpose_27()
{
    #define UNICH    0x4e2d
    #define RETB     FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isdigit(UNICH) != RETB){
        TEST_FAIL("g_unichar_isdigit: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isdigit
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is numeric (i.e. a digit). This covers ASCII 0-9 and also digits in other languages/scripts. Given some UTF-8 text, obtain a character value with g_utf8_get_char(). 
 *            c : a Unicode character  
 *            Returns : TRUE if c is a digit
 * Params Values: 0x0020
 * Expected: FALSE  
 */
static void test_purpose_28()
{
    #define UNICH    0x0020
    #define RETB     FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isdigit(UNICH) != RETB){
        TEST_FAIL("g_unichar_isdigit: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isdigit
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is numeric (i.e. a digit). This covers ASCII 0-9 and also digits in other languages/scripts. Given some UTF-8 text, obtain a character value with g_utf8_get_char(). 
 *            c : a Unicode character  
 *            Returns : TRUE if c is a digit
 * Params Values: 0x0031
 * Expected: TRUE  
 */
static void test_purpose_29()
{
    #define UNICH    0x0031
    #define RETB     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isdigit(UNICH) != RETB){
        TEST_FAIL("g_unichar_isdigit: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_validate
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Checks whether ch is a valid Unicode character. Some possible integer values of ch will not be valid. 0 is considered a valid character, though it's normally a string terminator.
 *            c : a Unicode character  
 *            Returns : TRUE if c is a valid Unicode character 
 * Params Values: 0
 * Expected: TRUE  
 */
static void test_purpose_30()
{
    #define UNICH    0
    #define RETB     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_validate(UNICH) != RETB){
        TEST_FAIL("g_unichar_validate: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_validate
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Checks whether ch is a valid Unicode character. Some possible integer values of ch will not be valid. 0 is considered a valid character, though it's normally a string terminator.
 *            c : a Unicode character  
 *            Returns : TRUE if c is a valid Unicode character 
 * Params Values: 0x0020
 * Expected: TRUE  
 */
static void test_purpose_31()
{
    #define UNICH    0x0020
    #define RETB     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_validate(UNICH) != RETB){
        TEST_FAIL("g_unichar_validate: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_validate
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Checks whether ch is a valid Unicode character. Some possible integer values of ch will not be valid. 0 is considered a valid character, though it's normally a string terminator.
 *            c : a Unicode character  
 *            Returns : TRUE if c is a valid Unicode character 
 * Params Values: 0xd8d6
 * Expected: FALSE  
 */
static void test_purpose_32()
{
    #define UNICH    0xd8d6
    #define RETB     FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_validate(UNICH) != RETB){
        TEST_FAIL("g_unichar_validate: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isxdigit
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines if a character is a hexidecimal digit. 
 *            c : a Unicode character  
 *            Returns : TRUE if the character is a hexadecimal digit
 * Params Values: 0
 * Expected: FALSE  
 */
static void test_purpose_33()
{
    #define UNICH    0
    #define RETB     FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isxdigit(UNICH) != RETB){
        TEST_FAIL("g_unichar_isxdigit: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isxdigit
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines if a character is a hexidecimal digit. 
 *            c : a Unicode character  
 *            Returns : TRUE if the character is a hexadecimal digit
 * Params Values: 0x0066
 * Expected: TRUE  
 */
static void test_purpose_34()
{
    #define UNICH    0x0066
    #define RETB     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isxdigit(UNICH) != RETB){
        TEST_FAIL("g_unichar_isxdigit: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isupper
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines if a character is uppercase. 
 *            c : a Unicode character  
 *            Returns : TRUE if c is an uppercase character  
 * Params Values: 0x0046
 * Expected: TRUE  
 */
static void test_purpose_35()
{
    #define UNICH    0x0046
    #define RETB     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isupper(UNICH) != RETB){
        TEST_FAIL("g_unichar_isupper: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_isupper
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines if a character is uppercase. 
 *            c : a Unicode character  
 *            Returns : TRUE if c is an uppercase character  
 * Params Values: 0x0066
 * Expected: FALSE  
 */
static void test_purpose_36()
{
    #define UNICH    0x0066
    #define RETB     FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_isupper(UNICH) != RETB){
        TEST_FAIL("g_unichar_isupper: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_ispunct
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is punctuation or a symbol. Given some UTF-8 text, obtain a character value with g_utf8_get_char(). 
 *            c : a Unicode character  
 *            Returns : TRUE if c is a punctuation or symbol character 
 * Params Values: 0x002c
 * Expected: TRUE  
 */
static void test_purpose_37()
{
    #define UNICH    0x002c
    #define RETB     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_ispunct(UNICH) != RETB){
        TEST_FAIL("g_unichar_ispunct: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_ispunct
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is punctuation or a symbol. Given some UTF-8 text, obtain a character value with g_utf8_get_char(). 
 *            c : a Unicode character  
 *            Returns : TRUE if c is a punctuation or symbol character 
 * Params Values: 0x0032
 * Expected: FALSE  
 */
static void test_purpose_38()
{
    #define UNICH    0x0032
    #define RETB     FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_ispunct(UNICH) != RETB){
        TEST_FAIL("g_unichar_ispunct: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_islower
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines if a character is lowercase letter. Given some UTF-8 text, obtain a character value with g_utf8_get_char().
 *            c : a Unicode character  
 *            Returns : TRUE if c is a lowercase letter  
 * Params Values: 0x0046
 * Expected: FALSE  
 */
static void test_purpose_39()
{
    #define UNICH    0x0046
    #define RETB     FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_islower(UNICH) != RETB){
        TEST_FAIL("g_unichar_islower: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_islower
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines if a character is lowercase letter. Given some UTF-8 text, obtain a character value with g_utf8_get_char().
 *            c : a Unicode character  
 *            Returns : TRUE if c is a lowercase letter  
 * Params Values: 0x0066
 * Expected: TRUE  
 */
static void test_purpose_40()
{
    #define UNICH    0x0066
    #define RETB     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_islower(UNICH) != RETB){
        TEST_FAIL("g_unichar_islower: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_iscntrl
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is a control character. Given some UTF-8 text, obtain a character value with g_utf8_get_char(). 
 *            c : a Unicode character  
 *            Returns : TRUE if c is a control character
 * Params Values: 0x001a
 * Expected: TRUE  
 */
static void test_purpose_41()
{
    #define UNICH    0x001a
    #define RETB     TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_iscntrl(UNICH) != RETB){
        TEST_FAIL("g_unichar_iscntrl: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_iscntrl
 * Params: gunichar c
 * Returns: gboolean
 * Assertion: Determines whether a character is a control character. Given some UTF-8 text, obtain a character value with g_utf8_get_char(). 
 *            c : a Unicode character  
 *            Returns : TRUE if c is a control character
 * Params Values: 0x0066
 * Expected: FALSE  
 */
static void test_purpose_42()
{
    #define UNICH    0x0066
    #define RETB     FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_iscntrl(UNICH) != RETB){
        TEST_FAIL("g_unichar_iscntrl: failed", NULL);
    }
    #undef UNICH
    #undef RETB
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_digit_value
 * Params: gunichar c
 * Returns: gint
 * Assertion: Determines the numeric value of a character as a decimal digit. 
 *            c : a Unicode character  
 *            Returns : If c is a decimal digit (according to g_unichar_isdigit()), its numeric value. Otherwise, -1.
 * Params Values: 0x0031
 * Expected: 1  
 */
static void test_purpose_43()
{
    #define UNICH    0x0031
    #define RETI     1
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_digit_value(UNICH) != RETI){
        TEST_FAIL("g_unichar_digit_value: failed", NULL);
    }
    #undef UNICH
    #undef RETI
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_digit_value
 * Params: gunichar c
 * Returns: gint
 * Assertion: Determines the numeric value of a character as a decimal digit. 
 *            c : a Unicode character  
 *            Returns : If c is a decimal digit (according to g_unichar_isdigit()), its numeric value. Otherwise, -1.
 * Params Values: 0x0066
 * Expected: -1  
 */
static void test_purpose_44()
{
    #define UNICH    0x0066
    #define RETI     -1
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_unichar_digit_value(UNICH) != RETI){
        TEST_FAIL("g_unichar_digit_value: failed", NULL);
    }
    #undef UNICH
    #undef RETI
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_toupper
 * Params: gunichar c
 * Returns: gunichar
 * Assertion: Converts a character to uppercase.
 *            c : a Unicode character.
 *            Returns : the result of converting c to uppercase. If c is not a lowercase or titlecase character, or has no upper case equivalent c is returned unchanged. 
 * Params Values: 0x0041
 * Expected: 0x0041  
 */
static void test_purpose_45()
{
    #define UNICH    0x0041
    #define RETCH    0x0041
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    
    if (g_unichar_toupper(UNICH) != RETCH){
        TEST_FAIL("g_unichar_toupper: failed", NULL);
    }
    
    #undef UNICH
    #undef RETCH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_unichar_toupper
 * Params: gunichar c
 * Returns: gunichar
 * Assertion: Converts a character to uppercase.
 *            c : a Unicode character.
 *            Returns : the result of converting c to uppercase. If c is not a lowercase or titlecase character, or has no upper case equivalent c is returned unchanged. 
 * Params Values: 0x0061
 * Expected: 0x0041  
 */
static void test_purpose_46()
{
    #define UNICH    0x0061
    #define RETCH    0x0041
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    
    if (g_unichar_toupper(UNICH) != RETCH){
        TEST_FAIL("g_unichar_toupper: failed", NULL);
    }
    
    #undef UNICH
    #undef RETCH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_utf8_prev_char
 * Params: const gchar *p
 * Returns: gchar*
 * Assertion: Finds the previous UTF-8 character in the string before p.
 * p does not have to be at the beginning of a UTF-8 character. No check is made to see if the character found is actually valid other than it starts with an appropriate byte. If p might be the first character of the string, you must use g_utf8_find_prev_char() instead.
 *            p : a pointer to a position within a UTF-8 encoded string
 *            Returns : a pointer to the found character.
 * Params Values: 1, "%c%c%c%c%c%c%c%c%c%c%c%c"
 * Expected: 0xe4, 0xb8, 0xad, 0xe5, 0x9b, 0xbd, 0xe6, 0x97, 0xa5, 0xe6, 0x9c, 0xac, "%c%c%c%c%c%c%c%c%c%c%c%c"  
 */
static void test_purpose_47()
{
    #define P2P    1
    #define FORM1  "%c%c%c%c%c%c%c%c%c%c%c%c"
    #define RETS   0xe4, 0xb8, 0xad, 0xe5, 0x9b, 0xbd, 0xe6, 0x97, 0xa5, 0xe6, 0x9c, 0xac
    #define FORM2  "%c%c%c%c%c%c%c%c%c%c%c%c"
    #define STR    0xe4, 0xb8, 0xad, 0xe5, 0x9b, 0xbd, 0xe6, 0x97, 0xa5, 0xe6, 0x9c, 0xac
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GString *gs1 = g_string_sized_new(10);
    GString *gs2 = g_string_sized_new(10);
    g_string_printf(gs1, FORM1, STR);
    g_string_printf(gs2, FORM2, RETS);
    
    gchar *p = g_utf8_offset_to_pointer(gs1->str, P2P);
    if (strcmp(g_utf8_prev_char(p), gs2->str) != 0){
        TEST_FAIL("g_utf8_prev_char: failed", NULL);
    }
    
    g_string_free(gs1, TRUE);
    g_string_free(gs2, TRUE);
    #undef P2P
    #undef FORM1
    #undef RETS
    #undef FORM2
    #undef STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_utf8_strdown
 * Params: const gchar *str, gssize len
 * Returns: gchar*
 * Assertion: Converts all Unicode characters in the string that have a case to lowercase. The exact manner that this is done depends on the current locale, and may result in the number of characters in the string changing. 
 *            str : a UTF-8 encoded string
 *            len : length of str, in bytes, or -1 if str is nul-terminated.
 *            Returns : a newly allocated string, with all characters converted to lowercase.
 * Params Values: 0xc3, 0x8b
 * Expected: 0xc3, 0xab  
 */
static void test_purpose_48()
{
    #define UPPCHAR    0xc3, 0x8b
    #define LOWCHAR    0xc3, 0xab
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    
    GString *gs1 = g_string_sized_new(10);
    GString *gs2 = g_string_sized_new(10);
    char *current_locale = setlocale(LC_CTYPE, NULL);
    g_string_printf(gs2, "%c%c", LOWCHAR);
    g_string_printf(gs1, "%c%c", UPPCHAR);
    if (strcmp(g_utf8_strdown(gs1->str, -1), gs2->str) != 0){
        TEST_FAIL("g_utf8_strdown: failed", NULL);
    }
    g_string_free(gs1, TRUE);
    g_string_free(gs2, TRUE);
    #undef LOWCHAR
    #undef UPPCHAR
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for UniMani ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {test_purpose_5, 1},
    {test_purpose_6, 1},
    {test_purpose_7, 1},
    {test_purpose_8, 1},
    {test_purpose_9, 1},
    {test_purpose_10, 1},
    {test_purpose_11, 1},
    {test_purpose_12, 1},
    {test_purpose_13, 1},
    {test_purpose_14, 1},
    {test_purpose_15, 1},
    {test_purpose_16, 1},
    {test_purpose_17, 1},
    {test_purpose_18, 1},
    {test_purpose_19, 1},
    {test_purpose_20, 1},
    {test_purpose_21, 1},
    {test_purpose_22, 1},
    {test_purpose_23, 1},
    {test_purpose_24, 1},
    {test_purpose_25, 1},
    {test_purpose_26, 1},
    {test_purpose_27, 1},
    {test_purpose_28, 1},
    {test_purpose_29, 1},
    {test_purpose_30, 1},
    {test_purpose_31, 1},
    {test_purpose_32, 1},
    {test_purpose_33, 1},
    {test_purpose_34, 1},
    {test_purpose_35, 1},
    {test_purpose_36, 1},
    {test_purpose_37, 1},
    {test_purpose_38, 1},
    {test_purpose_39, 1},
    {test_purpose_40, 1},
    {test_purpose_41, 1},
    {test_purpose_42, 1},
    {test_purpose_43, 1},
    {test_purpose_44, 1},
    {test_purpose_45, 1},
    {test_purpose_46, 1},
    {test_purpose_47, 1},
    {test_purpose_48, 1},
    {NULL,0}
};
