/*
 * GtkTreeModelSort methods tests
 * Testing GtkTreeModelSort methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;
struct ListSort
{
  gint number1;
  gint number2;
};

struct ListSort data[] =
{
  { 3, 4 },
  { 2, 6 },
  { 0, 49 },
  { 856856, 965 },
  { 1, 34663 },
};

struct ListSort child_data[] =
{
  { 10, 149 },
  { 12, 13 },
  { 1856856, 1965 },
  { 11, 15 },
  { 14, 14 },
};

const gint NUM_TREE_ELEMS = 5;

gint global_compare_func (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer data)
{
    GValue val_a = {0,};
    GValue val_b = {0,};
    guint uint_val_a;
    guint uint_val_b;

    gtk_tree_model_get_value (model, a, 0, &val_a);
    gtk_tree_model_get_value (model, b, 0, &val_b);
    uint_val_a = g_value_get_uint (&val_a);
    uint_val_b = g_value_get_uint (&val_b);
    if (uint_val_a > uint_val_b)
    {
        return (1);
    } else {
        return (0);
    }
}


/*
 * Name: gtk_tree_model_sort_new_with_model
 * Params: GtkTreeModel *child_model
 * Returns: GtkTreeModel*
 * Assertion: Creates a new GtkTreeModel, with child_model as the child_model.
 * Params Values: GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_UINT))
 * Expected: GtkTreeModel*  
 */
static void test_purpose_1()
{
    #define CHILD_MODEL GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_UINT))
    
    GtkTreeModel *model;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = gtk_tree_model_sort_new_with_model (CHILD_MODEL);
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_MODEL_SORT (model))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    #undef CHILD_MODEL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_model_sort_new_with_model
 * Params: GtkTreeModel *child_model
 * Returns: GtkTreeModel*
 * Assertion: Creates a new GtkTreeModel, with child_model as the child_model.
 * Params Values: GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_INT)) 
 * Expected: GtkTreeModel*  
 */
static void test_purpose_2()
{
    #define CHILD_MODEL GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_INT)) 
    
    GtkTreeModel *model;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = gtk_tree_model_sort_new_with_model (CHILD_MODEL);
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_MODEL_SORT (model))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    #undef CHILD_MODEL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_model_sort_new_with_model
 * Params: GtkTreeModel *child_model
 * Returns: GtkTreeModel*
 * Assertion: Creates a new GtkTreeModel, with child_model as the child_model.
 * Params Values: GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_CHAR))
 * Expected: GtkTreeModel*  
 */
static void test_purpose_3()
{
    #define CHILD_MODEL GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_CHAR))
    
    GtkTreeModel *model;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = gtk_tree_model_sort_new_with_model (CHILD_MODEL);
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_MODEL_SORT (model))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    #undef CHILD_MODEL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_model_sort_new_with_model
 * Params: GtkTreeModel *child_model
 * Returns: GtkTreeModel*
 * Assertion: Creates a new GtkTreeModel, with child_model as the child_model.
 * Params Values: GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_DOUBLE))
 * Expected: GtkTreeModel*  
 */
static void test_purpose_4()
{
    #define CHILD_MODEL GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_DOUBLE))
    
    GtkTreeModel *model;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = gtk_tree_model_sort_new_with_model (CHILD_MODEL);
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_MODEL_SORT (model))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    #undef CHILD_MODEL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_model_sort_get_model
 * Params: GtkTreeModel *tree_model
 * Returns: GtkTreeModel*
 * Assertion: Returns the model the GtkTreeModelSort is sorting.
 * Params Values: GtkTreeModel with child model set to 'GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_UINT))'
 * Expected: GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_UINT))  
 */
static void test_purpose_5()
{
    #define CHILD_MODEL GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_UINT))
    
    GtkTreeModel *model;
    GtkTreeModel *child_model;
    GtkTreeModel *real_child_model;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    child_model = CHILD_MODEL;
    model = gtk_tree_model_sort_new_with_model (child_model);
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_MODEL_SORT (model))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    real_child_model = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
    if (real_child_model != child_model)
    {
        TEST_FAIL ("child_model differs from the one was set during the tree_model initialization", NULL);
    }
    
    #undef CHILD_MODEL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_clear_cache
 * Params: GtkTreeModelSort *tree_model_sort
 * Returns: void
 * Assertion: It clears the tree_model_sort of any cached iterators that haven't been reffed with gtk_tree_model_ref_node()'
 * Params Values: newly created GtkTreeModelSort*
 * Expected: void  
 */
static void test_purpose_6()
{
    #define DEPTH 5
    #define INDICES {7, 3, 5, 0, 1}
    
    GtkTreeStore* tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    gint indices[] = INDICES;
    guint i;
    guint j;
    GtkTreeModel* model;
    gchar* converted_path_str;
    GValue val = {0,};
    GValue saved_val = {0,};
    guint uint_value;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    
    model = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (tree_store));
    gtk_tree_model_sort_clear_cache (GTK_TREE_MODEL_SORT (model));
    
    #undef DEPTH
    #undef INDICES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_child_path_to_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: Converts child_path to a path relative to tree_model_sort.  
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath
 * 
 * Name:  gtk_tree_model_sort_convert_child_path_to_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: The returned path will point to the same row in the sorted model.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath  
 */
static void test_purpose_7()
{
    #define PATH_STR "0:0"
    #define CORRECT_CONVERTED_PATH_STR_1 "0:3"
    #define CORRECT_CONVERTED_PATH_STR_2 "4:1"
    #define CORRECT_CONVERTED_PATH_STR_3 "3:0"
    #define CORRECT_CONVERTED_PATH_STR_4 "1:4"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_child_path_to_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: Converts child_path to a path relative to tree_model_sort.  
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath
 * 
 * Name:  gtk_tree_model_sort_convert_child_path_to_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: The returned path will point to the same row in the sorted model.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath  
 */
static void test_purpose_8()
{
    #define PATH_STR "2:1"
    #define CORRECT_CONVERTED_PATH_STR_1 "2:0"
    #define CORRECT_CONVERTED_PATH_STR_2 "2:4"
    #define CORRECT_CONVERTED_PATH_STR_3 "0:2"
    #define CORRECT_CONVERTED_PATH_STR_4 "4:2"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_child_path_to_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: Converts child_path to a path relative to tree_model_sort.  
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath
 * 
 * Name:  gtk_tree_model_sort_convert_child_path_to_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: The returned path will point to the same row in the sorted model.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath  
 */
static void test_purpose_9()
{
    #define PATH_STR "4:4"
    #define CORRECT_CONVERTED_PATH_STR_1 "4:1"
    #define CORRECT_CONVERTED_PATH_STR_2 "0:3"
    #define CORRECT_CONVERTED_PATH_STR_3 "1:3"
    #define CORRECT_CONVERTED_PATH_STR_4 "3:1"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_child_path_to_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: Converts child_path to a path relative to tree_model_sort.  
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath
 * 
 * Name:  gtk_tree_model_sort_convert_child_path_to_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: The returned path will point to the same row in the sorted model.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath  
 */
static void test_purpose_10()
{
    #define PATH_STR "0:2"
    #define CORRECT_CONVERTED_PATH_STR_1 "0:4"
    #define CORRECT_CONVERTED_PATH_STR_2 "4:0"
    #define CORRECT_CONVERTED_PATH_STR_3 "3:4"
    #define CORRECT_CONVERTED_PATH_STR_4 "1:0"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_child_path_to_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: If child_path isn't a valid path on the child model, then NULL is returned.'
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath  
 */
static void test_purpose_11()
{
    #define PATH_STR "0:5"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (converted_path)
    {
        TEST_FAIL ("1: converted_path should be NULL for invalid path '%s'", PATH_STR);
        return;
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_child_path_to_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: If child_path isn't a valid path on the child model, then NULL is returned.'
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath  
 */
static void test_purpose_12()
{
    #define PATH_STR "5:5"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (converted_path)
    {
        TEST_FAIL ("1: converted_path should be NULL for invalid path '%s'", PATH_STR);
        return;
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_child_path_to_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: If child_path isn't a valid path on the child model, then NULL is returned.'
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath  
 */
static void test_purpose_13()
{
    #define PATH_STR "5:0"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (converted_path)
    {
        TEST_FAIL ("1: converted_path should be NULL for invalid path '%s'", PATH_STR);
        return;
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_child_path_to_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: If child_path isn't a valid path on the child model, then NULL is returned.'
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath  
 */
static void test_purpose_14()
{
    #define PATH_STR "1000000:1000000"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (converted_path)
    {
        TEST_FAIL ("1: converted_path should be NULL for invalid path '%s'", PATH_STR);
        return;
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: Converts sorted_path to a path on the child model of tree_model_sort.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath
 * 
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: The returned path will point to the same location in the model not being sorted.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath  
 */
static void test_purpose_15()
{
    #define PATH_STR "0:0"
    #define CORRECT_CONVERTED_PATH_STR_1 "0:1"
    #define CORRECT_CONVERTED_PATH_STR_2 "4:2"
    #define CORRECT_CONVERTED_PATH_STR_3 "2:0"
    #define CORRECT_CONVERTED_PATH_STR_4 "3:2"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreePath* path2;
    GtkTreePath* converted_path2;
    gchar* converted_path_str;
    gchar* converted_path_str2;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    path2 = gtk_tree_path_new_from_string (PATH_STR);
    converted_path2 = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path2);
    
    if (!converted_path2)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str2 = gtk_tree_path_to_string (converted_path2);
    if (strcmp (converted_path_str2, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str2, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: Converts sorted_path to a path on the child model of tree_model_sort.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath
 * 
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: The returned path will point to the same location in the model not being sorted.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath  
 */
static void test_purpose_16()
{
    #define PATH_STR "2:2"
    #define CORRECT_CONVERTED_PATH_STR_1 "2:3"
    #define CORRECT_CONVERTED_PATH_STR_2 "2:3"
    #define CORRECT_CONVERTED_PATH_STR_3 "1:1"
    #define CORRECT_CONVERTED_PATH_STR_4 "1:1"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreePath* path2;
    GtkTreePath* converted_path2;
    gchar* converted_path_str;
    gchar* converted_path_str2;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    path2 = gtk_tree_path_new_from_string (PATH_STR);
    converted_path2 = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path2);
    
    if (!converted_path2)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str2 = gtk_tree_path_to_string (converted_path2);
    if (strcmp (converted_path_str2, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str2, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: Converts sorted_path to a path on the child model of tree_model_sort.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath
 * 
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: The returned path will point to the same location in the model not being sorted.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath  
 */
static void test_purpose_17()
{
    #define PATH_STR "3:3"
    #define CORRECT_CONVERTED_PATH_STR_1 "3:0"
    #define CORRECT_CONVERTED_PATH_STR_2 "1:4"
    #define CORRECT_CONVERTED_PATH_STR_3 "0:4"
    #define CORRECT_CONVERTED_PATH_STR_4 "4:3"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreePath* path2;
    GtkTreePath* converted_path2;
    gchar* converted_path_str;
    gchar* converted_path_str2;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    path2 = gtk_tree_path_new_from_string (PATH_STR);
    converted_path2 = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path2);
    
    if (!converted_path2)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str2 = gtk_tree_path_to_string (converted_path2);
    if (strcmp (converted_path_str2, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str2, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: Converts sorted_path to a path on the child model of tree_model_sort.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath
 * 
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: The returned path will point to the same location in the model not being sorted.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath  
 */
static void test_purpose_18()
{
    #define PATH_STR "4:4"
    #define CORRECT_CONVERTED_PATH_STR_1 "4:2"
    #define CORRECT_CONVERTED_PATH_STR_2 "0:1"
    #define CORRECT_CONVERTED_PATH_STR_3 "3:2"
    #define CORRECT_CONVERTED_PATH_STR_4 "2:0"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreePath* path2;
    GtkTreePath* converted_path2;
    gchar* converted_path_str;
    gchar* converted_path_str2;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    path2 = gtk_tree_path_new_from_string (PATH_STR);
    converted_path2 = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path2);
    
    if (!converted_path2)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str2 = gtk_tree_path_to_string (converted_path2);
    if (strcmp (converted_path_str2, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str2, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: Converts sorted_path to a path on the child model of tree_model_sort.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath
 * 
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: The returned path will point to the same location in the model not being sorted.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath  
 */
static void test_purpose_19()
{
    #define PATH_STR "0:2"
    #define CORRECT_CONVERTED_PATH_STR_1 "0:3"
    #define CORRECT_CONVERTED_PATH_STR_2 "4:3"
    #define CORRECT_CONVERTED_PATH_STR_3 "2:1"
    #define CORRECT_CONVERTED_PATH_STR_4 "3:1"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreePath* path2;
    GtkTreePath* converted_path2;
    gchar* converted_path_str;
    gchar* converted_path_str2;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    path2 = gtk_tree_path_new_from_string (PATH_STR);
    converted_path2 = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path2);
    
    if (!converted_path2)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str2 = gtk_tree_path_to_string (converted_path2);
    if (strcmp (converted_path_str2, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str2, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: Converts sorted_path to a path on the child model of tree_model_sort.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath
 * 
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: The returned path will point to the same location in the model not being sorted.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath  
 */
static void test_purpose_20()
{
    #define PATH_STR "2:1"
    #define CORRECT_CONVERTED_PATH_STR_1 "2:4"
    #define CORRECT_CONVERTED_PATH_STR_2 "2:0"
    #define CORRECT_CONVERTED_PATH_STR_3 "1:3"
    #define CORRECT_CONVERTED_PATH_STR_4 "1:4"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreePath* path2;
    GtkTreePath* converted_path2;
    gchar* converted_path_str;
    gchar* converted_path_str2;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    path2 = gtk_tree_path_new_from_string (PATH_STR);
    converted_path2 = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path2);
    
    if (!converted_path2)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str2 = gtk_tree_path_to_string (converted_path2);
    if (strcmp (converted_path_str2, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str2, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: Converts sorted_path to a path on the child model of tree_model_sort.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath
 * 
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *child_path 
 * Returns: GtkTreePath*
 * Assertion: The returned path will point to the same location in the model not being sorted.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: a newly allocated GtkTreePath  
 */
static void test_purpose_21()
{
    #define PATH_STR "3:0"
    #define CORRECT_CONVERTED_PATH_STR_1 "3:1"
    #define CORRECT_CONVERTED_PATH_STR_2 "1:2"
    #define CORRECT_CONVERTED_PATH_STR_3 "0:0"
    #define CORRECT_CONVERTED_PATH_STR_4 "4:2"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreePath* path2;
    GtkTreePath* converted_path2;
    gchar* converted_path_str;
    gchar* converted_path_str2;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    path2 = gtk_tree_path_new_from_string (PATH_STR);
    converted_path2 = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path2);
    
    if (!converted_path2)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str2 = gtk_tree_path_to_string (converted_path2);
    if (strcmp (converted_path_str2, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str2, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *sorted_path 
 * Returns: GtkTreePath*
 * Assertion: If sorted_path does not point to a location in the child model, NULL is returned.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: NULL  
 */
static void test_purpose_22()
{
    #define PATH_STR "0:5"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (converted_path)
    {
        TEST_FAIL ("1: converted_path is not NULL but should be", NULL);
        return;
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *sorted_path 
 * Returns: GtkTreePath*
 * Assertion: If sorted_path does not point to a location in the child model, NULL is returned.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: NULL  
 */
static void test_purpose_23()
{
    #define PATH_STR "5:0"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (converted_path)
    {
        TEST_FAIL ("1: converted_path is not NULL but should be", NULL);
        return;
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *sorted_path 
 * Returns: GtkTreePath*
 * Assertion: If sorted_path does not point to a location in the child model, NULL is returned.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: NULL  
 */
static void test_purpose_24()
{
    #define PATH_STR "5:5"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (converted_path)
    {
        TEST_FAIL ("1: converted_path is not NULL but should be", NULL);
        return;
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_path_to_child_path
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreePath *sorted_path 
 * Returns: GtkTreePath*
 * Assertion: If sorted_path does not point to a location in the child model, NULL is returned.
 * Params Values: newly created GtkTreeModelSort*
 * Expected: NULL  
 */
static void test_purpose_25()
{
    #define PATH_STR "5555:5555"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    path = gtk_tree_path_new_from_string (PATH_STR);
    converted_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (converted_path)
    {
        TEST_FAIL ("1: converted_path is not NULL but should be", NULL);
        return;
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_child_iter_to_iter
 * Params: GtkTreeModelSort *tree_model_sort,  GtkTreeIter *sort_iter, GtkTreeIter *child_iter
 * Returns: void
 * Assertion: Sets sort_iter to point to the row in tree_model_sort that corresponds to the row pointed at by child_iter.
 * Params Values: GtkTreeModelSort*, an uninitialized GtkTreeIter*, GtkTreeIter* got from path "0:0"
 * Expected:   
 */
static void test_purpose_26()
{
    #define PATH_STR "0:0"
    #define CORRECT_CONVERTED_PATH_STR_1 "0:3"
    #define CORRECT_CONVERTED_PATH_STR_2 "4:1"
    #define CORRECT_CONVERTED_PATH_STR_3 "3:0"
    #define CORRECT_CONVERTED_PATH_STR_4 "1:4"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreeIter sort_iter;
    GtkTreeIter ch_iter;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    gtk_tree_model_get_iter_from_string (model, &ch_iter, PATH_STR);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &sort_iter,
                                                    &ch_iter);
    converted_path = gtk_tree_model_get_path (smodel, &sort_iter);
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &sort_iter,
                                                    &ch_iter);
    converted_path = gtk_tree_model_get_path (smodel, &sort_iter);
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &sort_iter,
                                                    &ch_iter);
    converted_path = gtk_tree_model_get_path (smodel, &sort_iter);
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &sort_iter,
                                                    &ch_iter);
    converted_path = gtk_tree_model_get_path (smodel, &sort_iter);
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_child_iter_to_iter
 * Params: GtkTreeModelSort *tree_model_sort,  GtkTreeIter *sort_iter, GtkTreeIter *child_iter
 * Returns: void
 * Assertion: Sets sort_iter to point to the row in tree_model_sort that corresponds to the row pointed at by child_iter.
 * Params Values: GtkTreeModelSort*, an uninitialized GtkTreeIter*, GtkTreeIter* got from path "2:1"
 * Expected:   
 */
static void test_purpose_27()
{
    #define PATH_STR "2:1"
    #define CORRECT_CONVERTED_PATH_STR_1 "2:0"
    #define CORRECT_CONVERTED_PATH_STR_2 "2:4"
    #define CORRECT_CONVERTED_PATH_STR_3 "0:2"
    #define CORRECT_CONVERTED_PATH_STR_4 "4:2"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreeIter sort_iter;
    GtkTreeIter ch_iter;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    gtk_tree_model_get_iter_from_string (model, &ch_iter, PATH_STR);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &sort_iter,
                                                    &ch_iter);
    converted_path = gtk_tree_model_get_path (smodel, &sort_iter);
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &sort_iter,
                                                    &ch_iter);
    converted_path = gtk_tree_model_get_path (smodel, &sort_iter);
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &sort_iter,
                                                    &ch_iter);
    converted_path = gtk_tree_model_get_path (smodel, &sort_iter);
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &sort_iter,
                                                    &ch_iter);
    converted_path = gtk_tree_model_get_path (smodel, &sort_iter);
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_child_iter_to_iter
 * Params: GtkTreeModelSort *tree_model_sort,  GtkTreeIter *sort_iter, GtkTreeIter *child_iter
 * Returns: void
 * Assertion: Sets sort_iter to point to the row in tree_model_sort that corresponds to the row pointed at by child_iter.
 * Params Values: GtkTreeModelSort*, an uninitialized GtkTreeIter*, GtkTreeIter* got from path "4:4"
 * Expected:   
 */
static void test_purpose_28()
{
    #define PATH_STR "4:4"
    #define CORRECT_CONVERTED_PATH_STR_1 "4:1"
    #define CORRECT_CONVERTED_PATH_STR_2 "0:3"
    #define CORRECT_CONVERTED_PATH_STR_3 "1:3"
    #define CORRECT_CONVERTED_PATH_STR_4 "3:1"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreeIter sort_iter;
    GtkTreeIter ch_iter;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    gtk_tree_model_get_iter_from_string (model, &ch_iter, PATH_STR);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &sort_iter,
                                                    &ch_iter);
    converted_path = gtk_tree_model_get_path (smodel, &sort_iter);
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &sort_iter,
                                                    &ch_iter);
    converted_path = gtk_tree_model_get_path (smodel, &sort_iter);
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &sort_iter,
                                                    &ch_iter);
    converted_path = gtk_tree_model_get_path (smodel, &sort_iter);
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &sort_iter,
                                                    &ch_iter);
    converted_path = gtk_tree_model_get_path (smodel, &sort_iter);
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_child_iter_to_iter
 * Params: GtkTreeModelSort *tree_model_sort,  GtkTreeIter *sort_iter, GtkTreeIter *child_iter
 * Returns: void
 * Assertion: Sets sort_iter to point to the row in tree_model_sort that corresponds to the row pointed at by child_iter.
 * Params Values: GtkTreeModelSort*, an uninitialized GtkTreeIter*, GtkTreeIter* got from path "0:2"
 * Expected:   
 */
static void test_purpose_29()
{
    #define PATH_STR "0:2"
    #define CORRECT_CONVERTED_PATH_STR_1 "0:4"
    #define CORRECT_CONVERTED_PATH_STR_2 "4:0"
    #define CORRECT_CONVERTED_PATH_STR_3 "3:4"
    #define CORRECT_CONVERTED_PATH_STR_4 "1:0"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreeIter sort_iter;
    GtkTreeIter ch_iter;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    gtk_tree_model_get_iter_from_string (model, &ch_iter, PATH_STR);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &sort_iter,
                                                    &ch_iter);
    converted_path = gtk_tree_model_get_path (smodel, &sort_iter);
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &sort_iter,
                                                    &ch_iter);
    converted_path = gtk_tree_model_get_path (smodel, &sort_iter);
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &sort_iter,
                                                    &ch_iter);
    converted_path = gtk_tree_model_get_path (smodel, &sort_iter);
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &sort_iter,
                                                    &ch_iter);
    converted_path = gtk_tree_model_get_path (smodel, &sort_iter);
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_iter_to_child_iter
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreeIter *child_iter, GtkTreeIter *sorted_iter
 * Returns: void
 * Assertion: Sets child_iter to point to the row pointed to by sorted_iter.
 * Params Values: GtkTreeModelSort*, an uninitialized GtkTreeIter*, GtkTreeIter* got from path "0:0"
 * Expected:   
 */
static void test_purpose_30()
{
    #define PATH_STR "0:0"
    #define CORRECT_CONVERTED_PATH_STR_1 "0:1"
    #define CORRECT_CONVERTED_PATH_STR_2 "4:2"
    #define CORRECT_CONVERTED_PATH_STR_3 "2:0"
    #define CORRECT_CONVERTED_PATH_STR_4 "3:2"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreeIter sort_iter1;
    GtkTreeIter sort_iter2;
    GtkTreeIter sort_iter3;
    GtkTreeIter sort_iter4;
    GtkTreeIter ch_iter1;
    GtkTreeIter ch_iter2;
    GtkTreeIter ch_iter3;
    GtkTreeIter ch_iter4;
    gchar* converted_path_str;
    gint i;
    gint j;
    gboolean iter_valid;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter1, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter1,
                                                    &sort_iter1);
    converted_path = gtk_tree_model_get_path (model, &ch_iter1);
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter2, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: iter is invalid", FALSE);
    }
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter2,
                                                    &sort_iter2);
    
    converted_path = gtk_tree_model_get_path (model, &ch_iter2);
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter3, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("3: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter3,
                                                    &sort_iter3);
    converted_path = gtk_tree_model_get_path (model, &ch_iter3);
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter4, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("4: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter4,
                                                    &sort_iter4);
    converted_path = gtk_tree_model_get_path (model, &ch_iter4);
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_iter_to_child_iter
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreeIter *child_iter, GtkTreeIter *sorted_iter
 * Returns: void
 * Assertion: Sets child_iter to point to the row pointed to by sorted_iter.
 * Params Values: GtkTreeModelSort*, an uninitialized GtkTreeIter*, GtkTreeIter* got from path "2:2"
 * Expected:   
 */
static void test_purpose_31()
{
    #define PATH_STR "2:2"
    #define CORRECT_CONVERTED_PATH_STR_1 "2:3"
    #define CORRECT_CONVERTED_PATH_STR_2 "2:3"
    #define CORRECT_CONVERTED_PATH_STR_3 "1:1"
    #define CORRECT_CONVERTED_PATH_STR_4 "1:1"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreeIter sort_iter1;
    GtkTreeIter sort_iter2;
    GtkTreeIter sort_iter3;
    GtkTreeIter sort_iter4;
    GtkTreeIter ch_iter1;
    GtkTreeIter ch_iter2;
    GtkTreeIter ch_iter3;
    GtkTreeIter ch_iter4;
    gchar* converted_path_str;
    gint i;
    gint j;
    gboolean iter_valid;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter1, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter1,
                                                    &sort_iter1);
    converted_path = gtk_tree_model_get_path (model, &ch_iter1);
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter2, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: iter is invalid", FALSE);
    }
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter2,
                                                    &sort_iter2);
    
    converted_path = gtk_tree_model_get_path (model, &ch_iter2);
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter3, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("3: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter3,
                                                    &sort_iter3);
    converted_path = gtk_tree_model_get_path (model, &ch_iter3);
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter4, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("4: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter4,
                                                    &sort_iter4);
    converted_path = gtk_tree_model_get_path (model, &ch_iter4);
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_iter_to_child_iter
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreeIter *child_iter, GtkTreeIter *sorted_iter
 * Returns: void
 * Assertion: Sets child_iter to point to the row pointed to by sorted_iter.
 * Params Values: GtkTreeModelSort*, an uninitialized GtkTreeIter*, GtkTreeIter* got from path "3:3"
 * Expected:   
 */
static void test_purpose_32()
{
    #define PATH_STR "3:3"
    #define CORRECT_CONVERTED_PATH_STR_1 "3:0"
    #define CORRECT_CONVERTED_PATH_STR_2 "1:4"
    #define CORRECT_CONVERTED_PATH_STR_3 "0:4"
    #define CORRECT_CONVERTED_PATH_STR_4 "4:3"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreeIter sort_iter1;
    GtkTreeIter sort_iter2;
    GtkTreeIter sort_iter3;
    GtkTreeIter sort_iter4;
    GtkTreeIter ch_iter1;
    GtkTreeIter ch_iter2;
    GtkTreeIter ch_iter3;
    GtkTreeIter ch_iter4;
    gchar* converted_path_str;
    gint i;
    gint j;
    gboolean iter_valid;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter1, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter1,
                                                    &sort_iter1);
    converted_path = gtk_tree_model_get_path (model, &ch_iter1);
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter2, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: iter is invalid", FALSE);
    }
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter2,
                                                    &sort_iter2);
    
    converted_path = gtk_tree_model_get_path (model, &ch_iter2);
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter3, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("3: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter3,
                                                    &sort_iter3);
    converted_path = gtk_tree_model_get_path (model, &ch_iter3);
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter4, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("4: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter4,
                                                    &sort_iter4);
    converted_path = gtk_tree_model_get_path (model, &ch_iter4);
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_iter_to_child_iter
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreeIter *child_iter, GtkTreeIter *sorted_iter
 * Returns: void
 * Assertion: Sets child_iter to point to the row pointed to by sorted_iter.
 * Params Values: GtkTreeModelSort*, an uninitialized GtkTreeIter*, GtkTreeIter* got from path "4:4"
 * Expected:   
 */
static void test_purpose_33()
{
    #define PATH_STR "4:4"
    #define CORRECT_CONVERTED_PATH_STR_1 "4:2"
    #define CORRECT_CONVERTED_PATH_STR_2 "0:1"
    #define CORRECT_CONVERTED_PATH_STR_3 "3:2"
    #define CORRECT_CONVERTED_PATH_STR_4 "2:0"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreeIter sort_iter1;
    GtkTreeIter sort_iter2;
    GtkTreeIter sort_iter3;
    GtkTreeIter sort_iter4;
    GtkTreeIter ch_iter1;
    GtkTreeIter ch_iter2;
    GtkTreeIter ch_iter3;
    GtkTreeIter ch_iter4;
    gchar* converted_path_str;
    gint i;
    gint j;
    gboolean iter_valid;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter1, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter1,
                                                    &sort_iter1);
    converted_path = gtk_tree_model_get_path (model, &ch_iter1);
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter2, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: iter is invalid", FALSE);
    }
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter2,
                                                    &sort_iter2);
    
    converted_path = gtk_tree_model_get_path (model, &ch_iter2);
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter3, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("3: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter3,
                                                    &sort_iter3);
    converted_path = gtk_tree_model_get_path (model, &ch_iter3);
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter4, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("4: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter4,
                                                    &sort_iter4);
    converted_path = gtk_tree_model_get_path (model, &ch_iter4);
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_iter_to_child_iter
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreeIter *child_iter, GtkTreeIter *sorted_iter
 * Returns: void
 * Assertion: Sets child_iter to point to the row pointed to by sorted_iter.
 * Params Values: GtkTreeModelSort*, an uninitialized GtkTreeIter*, GtkTreeIter* got from path "0:2"
 * Expected:   
 */
static void test_purpose_34()
{
    #define PATH_STR "0:2"
    #define CORRECT_CONVERTED_PATH_STR_1 "0:3"
    #define CORRECT_CONVERTED_PATH_STR_2 "4:3"
    #define CORRECT_CONVERTED_PATH_STR_3 "2:1"
    #define CORRECT_CONVERTED_PATH_STR_4 "3:1"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreeIter sort_iter1;
    GtkTreeIter sort_iter2;
    GtkTreeIter sort_iter3;
    GtkTreeIter sort_iter4;
    GtkTreeIter ch_iter1;
    GtkTreeIter ch_iter2;
    GtkTreeIter ch_iter3;
    GtkTreeIter ch_iter4;
    gchar* converted_path_str;
    gint i;
    gint j;
    gboolean iter_valid;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter1, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter1,
                                                    &sort_iter1);
    converted_path = gtk_tree_model_get_path (model, &ch_iter1);
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter2, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: iter is invalid", FALSE);
    }
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter2,
                                                    &sort_iter2);
    
    converted_path = gtk_tree_model_get_path (model, &ch_iter2);
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter3, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("3: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter3,
                                                    &sort_iter3);
    converted_path = gtk_tree_model_get_path (model, &ch_iter3);
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter4, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("4: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter4,
                                                    &sort_iter4);
    converted_path = gtk_tree_model_get_path (model, &ch_iter4);
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_iter_to_child_iter
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreeIter *child_iter, GtkTreeIter *sorted_iter
 * Returns: void
 * Assertion: Sets child_iter to point to the row pointed to by sorted_iter.
 * Params Values: GtkTreeModelSort*, an uninitialized GtkTreeIter*, GtkTreeIter* got from path "2:1"
 * Expected:   
 */
static void test_purpose_35()
{
    #define PATH_STR "2:1"
    #define CORRECT_CONVERTED_PATH_STR_1 "2:4"
    #define CORRECT_CONVERTED_PATH_STR_2 "2:0"
    #define CORRECT_CONVERTED_PATH_STR_3 "1:3"
    #define CORRECT_CONVERTED_PATH_STR_4 "1:4"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreeIter sort_iter1;
    GtkTreeIter sort_iter2;
    GtkTreeIter sort_iter3;
    GtkTreeIter sort_iter4;
    GtkTreeIter ch_iter1;
    GtkTreeIter ch_iter2;
    GtkTreeIter ch_iter3;
    GtkTreeIter ch_iter4;
    gchar* converted_path_str;
    gint i;
    gint j;
    gboolean iter_valid;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter1, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter1,
                                                    &sort_iter1);
    converted_path = gtk_tree_model_get_path (model, &ch_iter1);
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter2, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: iter is invalid", FALSE);
    }
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter2,
                                                    &sort_iter2);
    
    converted_path = gtk_tree_model_get_path (model, &ch_iter2);
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter3, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("3: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter3,
                                                    &sort_iter3);
    converted_path = gtk_tree_model_get_path (model, &ch_iter3);
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter4, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("4: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter4,
                                                    &sort_iter4);
    converted_path = gtk_tree_model_get_path (model, &ch_iter4);
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_convert_iter_to_child_iter
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreeIter *child_iter, GtkTreeIter *sorted_iter
 * Returns: void
 * Assertion: Sets child_iter to point to the row pointed to by sorted_iter.
 * Params Values: GtkTreeModelSort*, an uninitialized GtkTreeIter*, GtkTreeIter* got from path "3:0"
 * Expected:   
 */
static void test_purpose_36()
{
    #define PATH_STR "3:0"
    #define CORRECT_CONVERTED_PATH_STR_1 "3:1"
    #define CORRECT_CONVERTED_PATH_STR_2 "1:2"
    #define CORRECT_CONVERTED_PATH_STR_3 "0:0"
    #define CORRECT_CONVERTED_PATH_STR_4 "4:2"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    GtkTreeIter sort_iter1;
    GtkTreeIter sort_iter2;
    GtkTreeIter sort_iter3;
    GtkTreeIter sort_iter4;
    GtkTreeIter ch_iter1;
    GtkTreeIter ch_iter2;
    GtkTreeIter ch_iter3;
    GtkTreeIter ch_iter4;
    gchar* converted_path_str;
    gint i;
    gint j;
    gboolean iter_valid;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    /* we emulate the first click on the second column */
    gtk_tree_view_column_clicked (column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter1, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter1,
                                                    &sort_iter1);
    converted_path = gtk_tree_model_get_path (model, &ch_iter1);
    if (!converted_path)
    {
        TEST_FAIL ("1: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_1))
    {
        TEST_FAIL ("1: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_1);
    }
    
    /* we emulate the second click on the second column */
    gtk_tree_view_column_clicked (column1);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter2, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("2: iter is invalid", FALSE);
    }
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter2,
                                                    &sort_iter2);
    
    converted_path = gtk_tree_model_get_path (model, &ch_iter2);
    if (!converted_path)
    {
        TEST_FAIL ("2: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_2))
    {
        TEST_FAIL ("2: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_2);
    }
    
    /* we emulate the first click on the first column */
    gtk_tree_view_column_clicked (column0);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter3, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("3: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter3,
                                                    &sort_iter3);
    converted_path = gtk_tree_model_get_path (model, &ch_iter3);
    if (!converted_path)
    {
        TEST_FAIL ("3: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_3))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_3);
    }
    
    
    /* we emulate the second click on the first column */
    gtk_tree_view_column_clicked (column0);
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &sort_iter4, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("4: iter is invalid", FALSE);
    }
    
    gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (smodel),
                                                    &ch_iter4,
                                                    &sort_iter4);
    converted_path = gtk_tree_model_get_path (model, &ch_iter4);
    if (!converted_path)
    {
        TEST_FAIL ("4: converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, CORRECT_CONVERTED_PATH_STR_4))
    {
        TEST_FAIL ("4: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, CORRECT_CONVERTED_PATH_STR_4);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    #undef CORRECT_CONVERTED_PATH_STR_1
    #undef CORRECT_CONVERTED_PATH_STR_2
    #undef CORRECT_CONVERTED_PATH_STR_3
    #undef CORRECT_CONVERTED_PATH_STR_4
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_reset_default_sort_func
 * Params: GtkTreeModelSort *tree_model_sort
 * Returns: void
 * Assertion: This resets the default sort function to be in the 'unsorted' state.
 * Params Values: GtkTreeModelSort*
 * Expected: void
 * 
 * Name:  gtk_tree_model_sort_reset_default_sort_func
 * Params: GtkTreeModelSort *tree_model_sort
 * Returns: void
 * Assertion: That is, it is in the same order as the child model.
 * Params Values: GtkTreeModelSort*
 * Expected: void  
 */
static void test_purpose_37()
{
    #define PATH_STR "0:0"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    path = gtk_tree_path_new_from_string (PATH_STR);
    
    gtk_tree_model_sort_reset_default_sort_func (GTK_TREE_MODEL_SORT (smodel));
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, PATH_STR))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, PATH_STR);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_reset_default_sort_func
 * Params: GtkTreeModelSort *tree_model_sort
 * Returns: void
 * Assertion: This resets the default sort function to be in the 'unsorted' state.
 * Params Values: GtkTreeModelSort*
 * Expected: void
 * 
 * Name:  gtk_tree_model_sort_reset_default_sort_func
 * Params: GtkTreeModelSort *tree_model_sort
 * Returns: void
 * Assertion: That is, it is in the same order as the child model.
 * Params Values: GtkTreeModelSort*
 * Expected: void  
 */
static void test_purpose_38()
{
    #define PATH_STR "2:1"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    path = gtk_tree_path_new_from_string (PATH_STR);
    
    gtk_tree_model_sort_reset_default_sort_func (GTK_TREE_MODEL_SORT (smodel));
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, PATH_STR))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, PATH_STR);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_reset_default_sort_func
 * Params: GtkTreeModelSort *tree_model_sort
 * Returns: void
 * Assertion: This resets the default sort function to be in the 'unsorted' state.
 * Params Values: GtkTreeModelSort*
 * Expected: void
 * 
 * Name:  gtk_tree_model_sort_reset_default_sort_func
 * Params: GtkTreeModelSort *tree_model_sort
 * Returns: void
 * Assertion: That is, it is in the same order as the child model.
 * Params Values: GtkTreeModelSort*
 * Expected: void  
 */
static void test_purpose_39()
{
    #define PATH_STR "4:4"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    path = gtk_tree_path_new_from_string (PATH_STR);
    
    gtk_tree_model_sort_reset_default_sort_func (GTK_TREE_MODEL_SORT (smodel));
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, PATH_STR))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, PATH_STR);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_reset_default_sort_func
 * Params: GtkTreeModelSort *tree_model_sort
 * Returns: void
 * Assertion: This resets the default sort function to be in the 'unsorted' state.
 * Params Values: GtkTreeModelSort*
 * Expected: void
 * 
 * Name:  gtk_tree_model_sort_reset_default_sort_func
 * Params: GtkTreeModelSort *tree_model_sort
 * Returns: void
 * Assertion: That is, it is in the same order as the child model.
 * Params Values: GtkTreeModelSort*
 * Expected: void  
 */
static void test_purpose_40()
{
    #define PATH_STR "0:2"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    path = gtk_tree_path_new_from_string (PATH_STR);
    
    gtk_tree_model_sort_reset_default_sort_func (GTK_TREE_MODEL_SORT (smodel));
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, PATH_STR))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, PATH_STR);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name:  gtk_tree_model_sort_reset_default_sort_func
 * Params: GtkTreeModelSort *tree_model_sort
 * Returns: void
 * Assertion: This resets the default sort function to be in the 'unsorted' state.
 * Params Values: GtkTreeModelSort*
 * Expected: void
 * 
 * Name:  gtk_tree_model_sort_reset_default_sort_func
 * Params: GtkTreeModelSort *tree_model_sort
 * Returns: void
 * Assertion: That is, it is in the same order as the child model.
 * Params Values: GtkTreeModelSort*
 * Expected: void  
 */
static void test_purpose_41()
{
    #define PATH_STR "3:4"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path;
    GtkTreePath* converted_path;
    gchar* converted_path_str;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_column_set_clickable (column0, TRUE);
    gtk_tree_view_column_set_clickable (column1, TRUE);
    
    path = gtk_tree_path_new_from_string (PATH_STR);
    
    gtk_tree_model_sort_reset_default_sort_func (GTK_TREE_MODEL_SORT (smodel));
    converted_path = gtk_tree_model_sort_convert_child_path_to_path (GTK_TREE_MODEL_SORT (smodel), path);
    
    if (!converted_path)
    {
        TEST_FAIL ("converted_path is NULL", NULL);
        return;
    }
    converted_path_str = gtk_tree_path_to_string (converted_path);
    if (strcmp (converted_path_str, PATH_STR))
    {
        TEST_FAIL ("3: converted_path_str = '%s', should be '%s'", 
                   converted_path_str, PATH_STR);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_model_sort_iter_is_valid
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreeIter *iter
 * Returns: gboolean
 * Assertion: Checks if the given iter is a valid iter for this GtkTreeModelSort.
 * Params Values: newly created GtkTreeModelSort*, GtkTreeIter* pointed to "0:0"
 * Expected: FALSE in the first call and TRUE in the second  
 */
static void test_purpose_42()
{
    #define PATH_STR "0:0"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter1;
    GtkTreeIter test_iter2;
    GtkTreePath* path;
    gboolean iter_valid;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (model, &test_iter1, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: test_iter is invalid", NULL);
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &test_iter2, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: test_iter is invalid", NULL);
    }
    
    iter_valid = gtk_tree_model_sort_iter_is_valid (GTK_TREE_MODEL_SORT (smodel), 
                                                    &test_iter1);
    if (iter_valid != FALSE)
    {
        TEST_FAIL ("3: iter_valid = TRUE, should be FALSE", NULL);
    }
    
    iter_valid = gtk_tree_model_sort_iter_is_valid (GTK_TREE_MODEL_SORT (smodel), 
                                                    &test_iter2);
    if (iter_valid != TRUE)
    {
        TEST_FAIL ("4: iter_valid = FALSE, should be TRUE", NULL);
    }
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_model_sort_iter_is_valid
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreeIter *iter
 * Returns: gboolean
 * Assertion: Checks if the given iter is a valid iter for this GtkTreeModelSort.
 * Params Values: newly created GtkTreeModelSort*, GtkTreeIter* pointed to "2:1"
 * Expected: FALSE in the first call and TRUE in the second  
 */
static void test_purpose_43()
{
    #define PATH_STR "2:1"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter1;
    GtkTreeIter test_iter2;
    GtkTreePath* path;
    gboolean iter_valid;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (model, &test_iter1, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: test_iter is invalid", NULL);
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &test_iter2, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: test_iter is invalid", NULL);
    }
    
    iter_valid = gtk_tree_model_sort_iter_is_valid (GTK_TREE_MODEL_SORT (smodel), 
                                                    &test_iter1);
    if (iter_valid != FALSE)
    {
        TEST_FAIL ("3: iter_valid = TRUE, should be FALSE", NULL);
    }
    
    iter_valid = gtk_tree_model_sort_iter_is_valid (GTK_TREE_MODEL_SORT (smodel), 
                                                    &test_iter2);
    if (iter_valid != TRUE)
    {
        TEST_FAIL ("4: iter_valid = FALSE, should be TRUE", NULL);
    }
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_model_sort_iter_is_valid
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreeIter *iter
 * Returns: gboolean
 * Assertion: Checks if the given iter is a valid iter for this GtkTreeModelSort.
 * Params Values: newly created GtkTreeModelSort*, GtkTreeIter* pointed to "4:4"
 * Expected: FALSE in the first call and TRUE in the second  
 */
static void test_purpose_44()
{
    #define PATH_STR "4:4"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter1;
    GtkTreeIter test_iter2;
    GtkTreePath* path;
    gboolean iter_valid;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (model, &test_iter1, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: test_iter is invalid", NULL);
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &test_iter2, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: test_iter is invalid", NULL);
    }
    
    iter_valid = gtk_tree_model_sort_iter_is_valid (GTK_TREE_MODEL_SORT (smodel), 
                                                    &test_iter1);
    if (iter_valid != FALSE)
    {
        TEST_FAIL ("3: iter_valid = TRUE, should be FALSE", NULL);
    }
    
    iter_valid = gtk_tree_model_sort_iter_is_valid (GTK_TREE_MODEL_SORT (smodel), 
                                                    &test_iter2);
    if (iter_valid != TRUE)
    {
        TEST_FAIL ("4: iter_valid = FALSE, should be TRUE", NULL);
    }
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_model_sort_iter_is_valid
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreeIter *iter
 * Returns: gboolean
 * Assertion: Checks if the given iter is a valid iter for this GtkTreeModelSort.
 * Params Values: newly created GtkTreeModelSort*, GtkTreeIter* pointed to "4:2"
 * Expected: FALSE in the first call and TRUE in the second  
 */
static void test_purpose_45()
{
    #define PATH_STR "4:2"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter1;
    GtkTreeIter test_iter2;
    GtkTreePath* path;
    gboolean iter_valid;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (model, &test_iter1, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: test_iter is invalid", NULL);
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &test_iter2, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: test_iter is invalid", NULL);
    }
    
    iter_valid = gtk_tree_model_sort_iter_is_valid (GTK_TREE_MODEL_SORT (smodel), 
                                                    &test_iter1);
    if (iter_valid != FALSE)
    {
        TEST_FAIL ("3: iter_valid = TRUE, should be FALSE", NULL);
    }
    
    iter_valid = gtk_tree_model_sort_iter_is_valid (GTK_TREE_MODEL_SORT (smodel), 
                                                    &test_iter2);
    if (iter_valid != TRUE)
    {
        TEST_FAIL ("4: iter_valid = FALSE, should be TRUE", NULL);
    }
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_model_sort_iter_is_valid
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreeIter *iter
 * Returns: gboolean
 * Assertion: Checks if the given iter is a valid iter for this GtkTreeModelSort.
 * Params Values: newly created GtkTreeModelSort*, GtkTreeIter* pointed to "0:2"
 * Expected: FALSE in the first call and TRUE in the second  
 */
static void test_purpose_46()
{
    #define PATH_STR "0:2"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter1;
    GtkTreeIter test_iter2;
    GtkTreePath* path;
    gboolean iter_valid;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (model, &test_iter1, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: test_iter is invalid", NULL);
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &test_iter2, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: test_iter is invalid", NULL);
    }
    
    iter_valid = gtk_tree_model_sort_iter_is_valid (GTK_TREE_MODEL_SORT (smodel), 
                                                    &test_iter1);
    if (iter_valid != FALSE)
    {
        TEST_FAIL ("3: iter_valid = TRUE, should be FALSE", NULL);
    }
    
    iter_valid = gtk_tree_model_sort_iter_is_valid (GTK_TREE_MODEL_SORT (smodel), 
                                                    &test_iter2);
    if (iter_valid != TRUE)
    {
        TEST_FAIL ("4: iter_valid = FALSE, should be TRUE", NULL);
    }
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_model_sort_iter_is_valid
 * Params: GtkTreeModelSort *tree_model_sort, GtkTreeIter *iter
 * Returns: gboolean
 * Assertion: Checks if the given iter is a valid iter for this GtkTreeModelSort.
 * Params Values: newly created GtkTreeModelSort*, GtkTreeIter* pointed to "3:4"
 * Expected: FALSE in the first call and TRUE in the second  
 */
static void test_purpose_47()
{
    #define PATH_STR "3:4"
    
    GtkTreeModel* model;
    GtkTreeModel* smodel;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    GtkCellRenderer* renderer;
    GtkWidget* tree_view;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeIter test_iter1;
    GtkTreeIter test_iter2;
    GtkTreePath* path;
    gboolean iter_valid;
    gint i;
    gint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    smodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (model));
    
    if (!model)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (smodel));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    iter_valid = gtk_tree_model_get_iter_from_string (model, &test_iter1, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: test_iter is invalid", NULL);
    }
    
    iter_valid = gtk_tree_model_get_iter_from_string (smodel, &test_iter2, PATH_STR);
    if (!iter_valid)
    {
        TEST_FAIL ("1: test_iter is invalid", NULL);
    }
    
    iter_valid = gtk_tree_model_sort_iter_is_valid (GTK_TREE_MODEL_SORT (smodel), 
                                                    &test_iter1);
    if (iter_valid != FALSE)
    {
        TEST_FAIL ("3: iter_valid = TRUE, should be FALSE", NULL);
    }
    
    iter_valid = gtk_tree_model_sort_iter_is_valid (GTK_TREE_MODEL_SORT (smodel), 
                                                    &test_iter2);
    if (iter_valid != TRUE)
    {
        TEST_FAIL ("4: iter_valid = FALSE, should be TRUE", NULL);
    }
    
    #undef PATH_STR
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for GtkTreeModelSort ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {test_purpose_5, 1},
    {test_purpose_6, 1},
    {test_purpose_7, 1},
    {test_purpose_8, 1},
    {test_purpose_9, 1},
    {test_purpose_10, 1},
    {test_purpose_11, 1},
    {test_purpose_12, 1},
    {test_purpose_13, 1},
    {test_purpose_14, 1},
    {test_purpose_15, 1},
    {test_purpose_16, 1},
    {test_purpose_17, 1},
    {test_purpose_18, 1},
    {test_purpose_19, 1},
    {test_purpose_20, 1},
    {test_purpose_21, 1},
    {test_purpose_22, 1},
    {test_purpose_23, 1},
    {test_purpose_24, 1},
    {test_purpose_25, 1},
    {test_purpose_26, 1},
    {test_purpose_27, 1},
    {test_purpose_28, 1},
    {test_purpose_29, 1},
    {test_purpose_30, 1},
    {test_purpose_31, 1},
    {test_purpose_32, 1},
    {test_purpose_33, 1},
    {test_purpose_34, 1},
    {test_purpose_35, 1},
    {test_purpose_36, 1},
    {test_purpose_37, 1},
    {test_purpose_38, 1},
    {test_purpose_39, 1},
    {test_purpose_40, 1},
    {test_purpose_41, 1},
    {test_purpose_42, 1},
    {test_purpose_43, 1},
    {test_purpose_44, 1},
    {test_purpose_45, 1},
    {test_purpose_46, 1},
    {test_purpose_47, 1},
    {NULL,0}
};
