/*
 * GtkTreeView methods tests
 * Testing GtkTreeView methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;
#include <float.h>
gint global_count;

struct ListSort
{
  gint number1;
  gint number2;
};

struct ListSort data[] =
{
  { 3, 4 },
  { 2, 6 },
  { 0, 49 },
  { 856856, 965 },
  { 1, 34663 },
};

struct ListSort child_data[] =
{
  { 10, 149 },
  { 12, 13 },
  { 1856856, 1965 },
  { 11, 15 },
  { 14, 14 },
};

const gint NUM_TREE_ELEMS = 5;


gboolean 
global_search_equal_func  (GtkTreeModel *model, gint column, const gchar *key, GtkTreeIter *iter, gpointer search_data)
{
    return TRUE;
}


void
global_data_func (GtkTreeViewColumn *tree_column, GtkCellRenderer *cell, GtkTreeModel *tree_model, GtkTreeIter *iter, gpointer data)
{
}

void
global_destroy_count_func (GtkTreeView *tree_view, GtkTreePath *path, gint children, gpointer user_data)
{
    global_count++;
}

gboolean
global_column_drag_func (GtkTreeView *tree_view, GtkTreeViewColumn *column, GtkTreeViewColumn *prev_column, 
                         GtkTreeViewColumn *next_column, gpointer data)
{
    global_count++;
    return FALSE;
}


void
global_row_activated (GtkTreeView *treeview, GtkTreePath *arg1, GtkTreeViewColumn *arg2, gpointer user_data)
{
    if (arg2 == (GtkTreeViewColumn*)user_data)
    {
        global_count++;
    }
}

void
global_map_rows  (GtkTreeView *tree_view, GtkTreePath *path, gpointer user_data)
{
    global_count++;
}


/*
 * Name: gtk_tree_view_new
 * Params: void
 * Returns: GtkTreeView*
 * Assertion: Creates a new GtkTreeView widget.
 * Params Values: void
 * Expected: GtkTreeView*  
 */
static void test_purpose_1()
{
    GtkWidget *widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_new_with_model
 * Params: GtkTreeModel *model
 * Returns: GtkTreeView*
 * Assertion: Creates a new GtkTreeView widget with the model initialized to model.
 * Params Values: GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_UINT));
 * Expected: GtkTreeView* with model set to GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_UINT));  
 */
static void test_purpose_2()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_UINT));
    
    GtkWidget *widget;
    GtkTreeModel* model;
    GtkTreeModel* real_model;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = MODEL;
    widget = gtk_tree_view_new_with_model (model);
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    real_model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
    if (model == NULL)
    {
        if (real_model)
        {
            TEST_FAIL ("model should be NULL but it is not", NULL);
        } 
    } else if (gtk_tree_model_get_n_columns (real_model)
                   != gtk_tree_model_get_n_columns (model)) {
        TEST_FAIL ("real_model differs from the model was set", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef MODEL
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_new_with_model
 * Params: GtkTreeModel *model
 * Returns: GtkTreeView*
 * Assertion: Creates a new GtkTreeView widget with the model initialized to model.
 * Params Values: GTK_TREE_MODEL (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_UINT))))
 * Expected: GtkTreeView* with model set to GTK_TREE_MODEL (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_UINT))))  
 */
static void test_purpose_3()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_UINT))))
    
    GtkWidget *widget;
    GtkTreeModel* model;
    GtkTreeModel* real_model;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = MODEL;
    widget = gtk_tree_view_new_with_model (model);
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    real_model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
    if (model == NULL)
    {
        if (real_model)
        {
            TEST_FAIL ("model should be NULL but it is not", NULL);
        } 
    } else if (gtk_tree_model_get_n_columns (real_model)
                   != gtk_tree_model_get_n_columns (model)) {
        TEST_FAIL ("real_model differs from the model was set", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef MODEL
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_new_with_model
 * Params: GtkTreeModel *model
 * Returns: GtkTreeView*
 * Assertion: Creates a new GtkTreeView widget with the model initialized to model.
 * Params Values: NULL
 * Expected: GtkTreeView* with model set to NULL  
 */
static void test_purpose_4()
{
    #define MODEL NULL
    
    GtkWidget *widget;
    GtkTreeModel* model;
    GtkTreeModel* real_model;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    model = MODEL;
    widget = gtk_tree_view_new_with_model (model);
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    real_model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
    if (model == NULL)
    {
        if (real_model)
        {
            TEST_FAIL ("model should be NULL but it is not", NULL);
        } 
    } else if (gtk_tree_model_get_n_columns (real_model)
                   != gtk_tree_model_get_n_columns (model)) {
        TEST_FAIL ("real_model differs from the model was set", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef MODEL
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_model
 * Params: GtkTreeView* tree_view, GtkTreeModel* model
 * Returns: void
 * Assertion: Sets the model for a GtkTreeView.
 * Params Values: newly created GtkTreeView*, newly created GtkWidget*
 * Expected: void
 * 
 * Name: gtk_tree_view_get_model
 * Params: GtkTreeView* tree_view
 * Returns: GtkTreeModel*
 * Assertion: Returns the model the the GtkTreeView is based on.
 * Params Values: newly created GtkTreeView*
 * Expected: GtkTreeModel* that was set by gtk_tree_view_set_model ()   
 */
static void test_purpose_5()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_UINT));
    
    GtkWidget *widget;
    GtkTreeModel *new_model = NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    new_model = MODEL;
    gtk_tree_view_set_model (GTK_TREE_VIEW (widget), new_model);
    
    if (new_model != gtk_tree_view_get_model (GTK_TREE_VIEW (widget)))
    {
        TEST_FAIL("Tree view model is differ from that was set by gtk_tree_view_set_model ()", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef MODEL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_model
 * Params: GtkTreeView* tree_view, GtkTreeModel* model
 * Returns: void
 * Assertion: Sets the model for a GtkTreeView.
 * Params Values: newly created GtkTreeView*, newly created GtkWidget*
 * Expected: void
 * 
 * Name: gtk_tree_view_get_model
 * Params: GtkTreeView* tree_view
 * Returns: GtkTreeModel*
 * Assertion: Returns the model the the GtkTreeView is based on.
 * Params Values: newly created GtkTreeView*
 * Expected: GtkTreeModel* that was set by gtk_tree_view_set_model ()   
 */
static void test_purpose_6()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_INT))))
    
    GtkWidget *widget;
    GtkTreeModel *new_model = NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    new_model = MODEL;
    gtk_tree_view_set_model (GTK_TREE_VIEW (widget), new_model);
    
    if (new_model != gtk_tree_view_get_model (GTK_TREE_VIEW (widget)))
    {
        TEST_FAIL("Tree view model is differ from that was set by gtk_tree_view_set_model ()", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef MODEL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_model
 * Params: GtkTreeView* tree_view, GtkTreeModel* model
 * Returns: void
 * Assertion: If the tree_view already has a model set, it will remove it before setting the new model.
 * Params Values: newly created GtkTreeView*, newly created GtkWidget*
 * Expected: void  
 */
static void test_purpose_7()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_INT));
    
    GtkWidget *widget;
    GtkTreeModel *new_model = NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    gtk_tree_view_set_model (GTK_TREE_VIEW (widget), GTK_TREE_MODEL (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_INT)))));
    
    new_model = MODEL;
    gtk_tree_view_set_model (GTK_TREE_VIEW (widget), GTK_TREE_MODEL (new_model));
    
    if (new_model != gtk_tree_view_get_model (GTK_TREE_VIEW (widget)))
    {
        TEST_FAIL("Tree view model is differ from that was set by gtk_tree_view_set_model ()", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef MODEL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_model
 * Params: GtkTreeView* tree_view, GtkTreeModel* model
 * Returns: void
 * Assertion: If the tree_view already has a model set, it will remove it before setting the new model.
 * Params Values: newly created GtkTreeView*, newly created GtkWidget*
 * Expected: void  
 */
static void test_purpose_8()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL(gtk_tree_store_new (1, G_TYPE_CHAR))))
    
    GtkWidget *widget;
    GtkTreeModel *new_model = NULL;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    gtk_tree_view_set_model (GTK_TREE_VIEW (widget), GTK_TREE_MODEL (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_INT)))));
    
    new_model = MODEL;
    gtk_tree_view_set_model (GTK_TREE_VIEW (widget), GTK_TREE_MODEL (new_model));
    
    if (new_model != gtk_tree_view_get_model (GTK_TREE_VIEW (widget)))
    {
        TEST_FAIL("Tree view model is differ from that was set by gtk_tree_view_set_model ()", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef MODEL
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_model
 * Params: GtkTreeView* tree_view, GtkTreeModel* model
 * Returns: void
 * Assertion: If model is NULL, then it will unset the old model.
 * Params Values: newly created GtkTreeView*, newly created GtkWidget*
 * Expected: void
 * 
 * Name: gtk_tree_view_get_model
 * Params: GtkTreeView* tree_view
 * Returns: GtkTreeModel*
 * Assertion: Returns NULL if the model is unset.
 * Params Values: newly created GtkTreeView*
 * Expected: GtkTreeModel* that was set by gtk_tree_view_set_model ()   
 */
static void test_purpose_9()
{
    GtkWidget *widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    gtk_tree_view_set_model (GTK_TREE_VIEW (widget), NULL);
    
    if (gtk_tree_view_get_model (GTK_TREE_VIEW (widget)) != NULL)
    {
        TEST_FAIL("Tree view model should be NULL", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_selection
 * Params: GtkTreeView* tree_view
 * Returns: GtkTreeSelection*
 * Assertion: Gets the GtkTreeSelection associated with tree_view.
 * Params Values: newly created GtkTreeView*
 * Expected: GtkTreeSelection*    
 */
static void test_purpose_10()
{
    GtkWidget *widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_hadjustment
 * Params: GtkTreeView* tree_view, GtkAdjustment* adjustment
 * Returns: void
 * Assertion: Gets the GtkAdjustment currently being used for the horizontal aspect.
 * Params Values: newly created GtkTreeView*, gtk_adjustment_new (G_MINDOUBLE, 0.0, 0.0, 0.0, 0.0, 0.0)
 * Expected: void
 * 
 * Name: gtk_tree_view_set_hadjustment
 * Params: GtkTreeView* tree_view
 * Returns: GtkAdjustment*
 * Assertion: Sets the GtkAdjustment for the current horizontal aspect.
 * Params Values: newly created GtkTreeView*
 * Expected: GtkAdjustment* that was set by gtk_tree_view_set_hadjustment ()     
 */
static void test_purpose_11()
{
    #define ADJUSTMENT gtk_adjustment_new (G_MINDOUBLE, 0.0, 0.0, 0.0, 0.0, 0.0)
    
    GtkWidget *widget;
    GtkAdjustment *adjustment;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    adjustment = GTK_ADJUSTMENT (ADJUSTMENT);
    gtk_tree_view_set_hadjustment (GTK_TREE_VIEW (widget), adjustment);
    
    if (adjustment != gtk_tree_view_get_hadjustment (GTK_TREE_VIEW (widget)))
    {
        TEST_FAIL("Tree view hadjustment is differ from that was set by gtk_tree_view_set_hadjustment ()", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef ADJUSTMENT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_vadjustment
 * Params: GtkTreeView* tree_view, GtkAdjustment* adjustment
 * Returns: void
 * Assertion: Sets the GtkAdjustment for the current vertical aspect.
 * Params Values: newly created GtkTreeView*, gtk_adjustment_new (G_MINDOUBLE, 0.0, 0.0, 0.0, 0.0, 0.0)
 * Expected: void
 * 
 * Name: gtk_tree_view_get_vadjustment
 * Params: GtkTreeView* tree_view
 * Returns: GtkAdjustment*
 * Assertion: Gets the GtkAdjustment currently being used for the vertical aspect.
 * Params Values: newly created GtkTreeView*
 * Expected: GtkAdjustment* that was set by gtk_tree_view_set_vadjustment ()     
 */
static void test_purpose_12()
{
    #define ADJUSTMENT gtk_adjustment_new (G_MINDOUBLE, 0.0, 0.0, 0.0, 0.0, 0.0)
    
    GtkWidget *widget;
    GtkAdjustment *adjustment;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    adjustment = GTK_ADJUSTMENT (ADJUSTMENT);
    gtk_tree_view_set_vadjustment (GTK_TREE_VIEW (widget), adjustment);
    
    if (adjustment != gtk_tree_view_get_vadjustment (GTK_TREE_VIEW (widget)))
    {
        TEST_FAIL("Tree view vadjustment is differ from that was set by gtk_tree_view_set_vadjustment ()", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef ADJUSTMENT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_headers_visible
 * Params: GtkTreeView* tree_view, gboolean headers_visible
 * Returns: void
 * Assertion: Sets the the visibility state of the headers.
 * Params Values: newly created GtkTreeView*, TRUE
 * Expected: void
 * 
 * Name: gtk_tree_view_get_headers_visible
 * Params: GtkTreeView* tree_view
 * Returns: gboolean
 * Assertion: Returns TRUE if the headers on the tree_view are visible.
 * Params Values: newly created GtkTreeView*
 * Expected: TRUE  
 */
static void test_purpose_13()
{
    #define HEADERS_VISIBLE TRUE
    
    GtkWidget *widget;
    gboolean headers_visible;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (widget), HEADERS_VISIBLE);
    headers_visible = gtk_tree_view_get_headers_visible (GTK_TREE_VIEW (widget));
    if (HEADERS_VISIBLE != headers_visible)
    {
        TEST_FAIL("Tree view headers visible  = %i, should be %i", headers_visible, HEADERS_VISIBLE);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef HEADERS_VISIBLE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_headers_visible
 * Params: GtkTreeView* tree_view, gboolean headers_visible
 * Returns: void
 * Assertion: Sets the the visibility state of the headers.
 * Params Values: newly created GtkTreeView*, FALSE
 * Expected: void
 * 
 * Name: gtk_tree_view_get_headers_visible
 * Params: GtkTreeView* tree_view
 * Returns: gboolean
 * Assertion: Returns TRUE if the headers on the tree_view are visible.
 * Params Values: newly created GtkTreeView*
 * Expected: FALSE  
 */
static void test_purpose_14()
{
    #define HEADERS_VISIBLE FALSE
    
    GtkWidget *widget;
    gboolean headers_visible;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (widget), HEADERS_VISIBLE);
    headers_visible = gtk_tree_view_get_headers_visible (GTK_TREE_VIEW (widget));
    if (HEADERS_VISIBLE != headers_visible)
    {
        TEST_FAIL("Tree view headers visible  = %i, should be %i", headers_visible, HEADERS_VISIBLE);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef HEADERS_VISIBLE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_headers_clickable
 * Params: GtkTreeView* tree_view, gboolean headers_clickable
 * Returns: void
 * Assertion: Allow the column title buttons to be clicked.
 * Params Values: newly created GtkTreeView*, TRUE
 * Expected: void  
 */
static void test_purpose_15()
{
    #define HEADERS_CLICKABLE TRUE
    
    GtkWidget *widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new_with_model  
        (GTK_TREE_MODEL (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_INT)))));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (widget), HEADERS_CLICKABLE);
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef HEADERS_CLICKABLE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_headers_clickable
 * Params: GtkTreeView* tree_view, gboolean headers_clickable
 * Returns: void
 * Assertion: Allow the column title buttons to be clicked.
 * Params Values: newly created GtkTreeView*, FALSE
 * Expected: void  
 */
static void test_purpose_16()
{
    #define HEADERS_CLICKABLE FALSE
    
    GtkWidget *widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new_with_model  
        (GTK_TREE_MODEL (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_INT)))));
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (widget), HEADERS_CLICKABLE);
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef HEADERS_CLICKABLE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_column
 * Params: GtkTreeView* tree_view, gint n
 * Returns: void
 * Assertion: Gets the GtkTreeViewColumn at the given position in the tree_view.
 * Params Values: newly created GtkTreeView*, 0
 * Expected: void  
 */
static void test_purpose_17()
{
    #define N 0
    
    GtkWidget* widget;
    GtkTreeViewColumn* new_column;
    GtkTreeViewColumn* column;
    int i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    new_column = gtk_tree_view_column_new ();
    if (!new_column)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW_COLUMN (new_column))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < 101 ; i++)
    {
        gtk_tree_view_insert_column (GTK_TREE_VIEW (widget), gtk_tree_view_column_new (), i);
    }
    
    gtk_tree_view_column_set_min_width (new_column, 4917);
    gtk_tree_view_insert_column (GTK_TREE_VIEW (widget), new_column, N);
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (widget), N);
    if (N > 101)
    {
        if (column != NULL)
        {
            TEST_FAIL("The column at the %i is not NULL, but it should be.", N);
        }
    } else {
        if (gtk_tree_view_column_get_min_width (column) != 4917)
        {
            TEST_FAIL("The column at the %i position differs from that was inserted -- %i", N, gtk_tree_view_column_get_min_width (column));
        }
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef N
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_column
 * Params: GtkTreeView* tree_view, gint n
 * Returns: void
 * Assertion: Gets the GtkTreeViewColumn at the given position in the tree_view.
 * Params Values: newly created GtkTreeView*, 17
 * Expected: void  
 */
static void test_purpose_18()
{
    #define N 17
    
    GtkWidget* widget;
    GtkTreeViewColumn* new_column;
    GtkTreeViewColumn* column;
    int i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    new_column = gtk_tree_view_column_new ();
    if (!new_column)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW_COLUMN (new_column))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < 101 ; i++)
    {
        gtk_tree_view_insert_column (GTK_TREE_VIEW (widget), gtk_tree_view_column_new (), i);
    }
    
    gtk_tree_view_column_set_min_width (new_column, 4917);
    gtk_tree_view_insert_column (GTK_TREE_VIEW (widget), new_column, N);
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (widget), N);
    if (N > 101)
    {
        if (column != NULL)
        {
            TEST_FAIL("The column at the %i is not NULL, but it should be.", N);
        }
    } else {
        if (gtk_tree_view_column_get_min_width (column) != 4917)
        {
            TEST_FAIL("The column at the %i position differs from that was inserted -- %i", N, gtk_tree_view_column_get_min_width (column));
        }
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef N
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_column
 * Params: GtkTreeView* tree_view, gint n
 * Returns: void
 * Assertion: Gets the GtkTreeViewColumn at the given position in the tree_view.
 * Params Values: newly created GtkTreeView*, 101
 * Expected: void  
 */
static void test_purpose_19()
{
    #define N 101
    
    GtkWidget* widget;
    GtkTreeViewColumn* new_column;
    GtkTreeViewColumn* column;
    int i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    new_column = gtk_tree_view_column_new ();
    if (!new_column)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW_COLUMN (new_column))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < 101 ; i++)
    {
        gtk_tree_view_insert_column (GTK_TREE_VIEW (widget), gtk_tree_view_column_new (), i);
    }
    
    gtk_tree_view_column_set_min_width (new_column, 4917);
    gtk_tree_view_insert_column (GTK_TREE_VIEW (widget), new_column, N);
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (widget), N);
    if (N > 101)
    {
        if (column != NULL)
        {
            TEST_FAIL("The column at the %i is not NULL, but it should be.", N);
        }
    } else {
        if (gtk_tree_view_column_get_min_width (column) != 4917)
        {
            TEST_FAIL("The column at the %i position differs from that was inserted -- %i", N, gtk_tree_view_column_get_min_width (column));
        }
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef N
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_column
 * Params: GtkTreeView* tree_view, gint n
 * Returns: void
 * Assertion: Gets the GtkTreeViewColumn at the given position in the tree_view.
 * Params Values: newly created GtkTreeView*, 102
 * Expected: void  
 */
static void test_purpose_20()
{
    #define N 102
    
    GtkWidget* widget;
    GtkTreeViewColumn* new_column;
    GtkTreeViewColumn* column;
    int i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    new_column = gtk_tree_view_column_new ();
    if (!new_column)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW_COLUMN (new_column))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < 101 ; i++)
    {
        gtk_tree_view_insert_column (GTK_TREE_VIEW (widget), gtk_tree_view_column_new (), i);
    }
    
    gtk_tree_view_column_set_min_width (new_column, 4917);
    gtk_tree_view_insert_column (GTK_TREE_VIEW (widget), new_column, N);
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (widget), N);
    if (N > 101)
    {
        if (column != NULL)
        {
            TEST_FAIL("The column at the %i is not NULL, but it should be.", N);
        }
    } else {
        if (gtk_tree_view_column_get_min_width (column) != 4917)
        {
            TEST_FAIL("The column at the %i position differs from that was inserted -- %i", N, gtk_tree_view_column_get_min_width (column));
        }
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef N
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_column
 * Params: GtkTreeView* tree_view, gint n
 * Returns: void
 * Assertion: Gets the GtkTreeViewColumn at the given position in the tree_view.
 * Params Values: newly created GtkTreeView*, 1020
 * Expected: void  
 */
static void test_purpose_21()
{
    #define N 1020
    
    GtkWidget* widget;
    GtkTreeViewColumn* new_column;
    GtkTreeViewColumn* column;
    int i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    new_column = gtk_tree_view_column_new ();
    if (!new_column)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW_COLUMN (new_column))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < 101 ; i++)
    {
        gtk_tree_view_insert_column (GTK_TREE_VIEW (widget), gtk_tree_view_column_new (), i);
    }
    
    gtk_tree_view_column_set_min_width (new_column, 4917);
    gtk_tree_view_insert_column (GTK_TREE_VIEW (widget), new_column, N);
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (widget), N);
    if (N > 101)
    {
        if (column != NULL)
        {
            TEST_FAIL("The column at the %i is not NULL, but it should be.", N);
        }
    } else {
        if (gtk_tree_view_column_get_min_width (column) != 4917)
        {
            TEST_FAIL("The column at the %i position differs from that was inserted -- %i", N, gtk_tree_view_column_get_min_width (column));
        }
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef N
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_columns_autosize
 * Params: GtkTreeView* tree_view
 * Returns: void
 * Assertion: Resizes all columns to their optimal width.
 * Params Values: newly created GtkTreeView*
 * Expected: void  
 */
static void test_purpose_22()
{
    GtkWidget *widget;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    gtk_tree_view_columns_autosize (GTK_TREE_VIEW (widget));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_append_column
 * Params: GtkTreeView* tree_view, GtkTreeViewColumn* column 
 * Returns: gint
 * Assertion: Appends column to the list of columns.
 * Params Values: newly created GtkTreeView*, newly created GtkTreeViewColumn*, 
 * Expected: 117  
 */
static void test_purpose_23()
{
    #define N 117
    
    GtkWidget* widget;
    GtkTreeViewColumn* new_column;
    GtkTreeViewColumn* column;
    gint i;
    gint n;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < N; i++)
    {
        new_column = gtk_tree_view_column_new ();
        gtk_tree_view_column_set_min_width (new_column, 4917);
        n= gtk_tree_view_append_column (GTK_TREE_VIEW (widget), new_column);
    }
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (widget), N-1);
    
    if (gtk_tree_view_column_get_min_width (column) != 4917)
    {
        TEST_FAIL("The column at the %i position differs from that was appended. Min width of this column = %i", N, gtk_tree_view_column_get_min_width (column));
    }
    
    if (n != N)
    {
        TEST_FAIL("gtk_tree_view_append_column () return %i, should be %i.", n, N);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef N
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_remove_column
 * Params: GtkTreeView* tree_view, GtkTreeViewColumn* column 
 * Returns: gint
 * Assertion: Removes column from tree_view.
 * Params Values: newly created GtkTreeView*, newly created GtkTreeViewColumn*, 
 * Expected: 7  
 */
static void test_purpose_24()
{
    #define N 7
    #define INDEX 0
    
    GtkWidget* widget;
    GtkTreeViewColumn* new_column;
    GtkTreeViewColumn* save_column;
    gint i;
    gint n;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < N; i++)
    {
        new_column = gtk_tree_view_column_new ();
        gtk_tree_view_append_column (GTK_TREE_VIEW (widget), new_column);
        if (i == INDEX)
        {
            save_column = new_column;
        }
    }
    
    n = gtk_tree_view_remove_column (GTK_TREE_VIEW (widget), save_column);
    
    if (n != N-1)
    {
        TEST_FAIL("gtk_tree_view_remove_column () return %i, should be %i.", n, N-1);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef N
    #undef INDEX
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_remove_column
 * Params: GtkTreeView* tree_view, GtkTreeViewColumn* column 
 * Returns: gint
 * Assertion: Removes column from tree_view.
 * Params Values: newly created GtkTreeView*, newly created GtkTreeViewColumn*, 
 * Expected: 11  
 */
static void test_purpose_25()
{
    #define N 11
    #define INDEX 5
    
    GtkWidget* widget;
    GtkTreeViewColumn* new_column;
    GtkTreeViewColumn* save_column;
    gint i;
    gint n;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < N; i++)
    {
        new_column = gtk_tree_view_column_new ();
        gtk_tree_view_append_column (GTK_TREE_VIEW (widget), new_column);
        if (i == INDEX)
        {
            save_column = new_column;
        }
    }
    
    n = gtk_tree_view_remove_column (GTK_TREE_VIEW (widget), save_column);
    
    if (n != N-1)
    {
        TEST_FAIL("gtk_tree_view_remove_column () return %i, should be %i.", n, N-1);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef N
    #undef INDEX
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_remove_column
 * Params: GtkTreeView* tree_view, GtkTreeViewColumn* column 
 * Returns: gint
 * Assertion: Removes column from tree_view.
 * Params Values: newly created GtkTreeView*, newly created GtkTreeViewColumn*, 
 * Expected: 5  
 */
static void test_purpose_26()
{
    #define N 5
    #define INDEX 4
    
    GtkWidget* widget;
    GtkTreeViewColumn* new_column;
    GtkTreeViewColumn* save_column;
    gint i;
    gint n;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < N; i++)
    {
        new_column = gtk_tree_view_column_new ();
        gtk_tree_view_append_column (GTK_TREE_VIEW (widget), new_column);
        if (i == INDEX)
        {
            save_column = new_column;
        }
    }
    
    n = gtk_tree_view_remove_column (GTK_TREE_VIEW (widget), save_column);
    
    if (n != N-1)
    {
        TEST_FAIL("gtk_tree_view_remove_column () return %i, should be %i.", n, N-1);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef N
    #undef INDEX
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_insert_column
 * Params: GtkTreeView* tree_view, GtkTreeViewColumn* column, gint position
 * Returns: gint
 * Assertion: This inserts the column into the tree_view at position. 
 * Params Values: newly created GtkTreeView*, newly created GtkTreeViewColumn*, 0
 * Expected: 1  
 */
static void test_purpose_27()
{
    #define N_COLUMNS 1
    #define POSITION 0
    
    GtkWidget* widget;
    GtkTreeViewColumn* new_column;
    GtkTreeViewColumn* column;
    gint i;
    gint n;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 1; i < N_COLUMNS ; i++)
    {
        gtk_tree_view_append_column (GTK_TREE_VIEW (widget), gtk_tree_view_column_new ());
    }
    
    new_column = gtk_tree_view_column_new ();
    if (!new_column)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW_COLUMN (new_column))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    gtk_tree_view_column_set_min_width (new_column, 4917);
    n= gtk_tree_view_insert_column (GTK_TREE_VIEW (widget), new_column, POSITION);
    if (n != N_COLUMNS)
    {
        TEST_FAIL("gtk_tree_view_insert_column () return %i, should return %i", n, N_COLUMNS);
    }
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (widget), POSITION);
    if (gtk_tree_view_column_get_min_width (column) != 4917)
    {
        TEST_FAIL("The column at the %i position differs from that was inserted. Min width of this column = %i", POSITION, gtk_tree_view_column_get_min_width (column));
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef N_COLUMNS
    #undef POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_insert_column
 * Params: GtkTreeView* tree_view, GtkTreeViewColumn* column, gint position
 * Returns: gint
 * Assertion: This inserts the column into the tree_view at position. 
 * Params Values: newly created GtkTreeView*, newly created GtkTreeViewColumn*, 11
 * Expected: 17  
 */
static void test_purpose_28()
{
    #define N_COLUMNS 17
    #define POSITION 11
    
    GtkWidget* widget;
    GtkTreeViewColumn* new_column;
    GtkTreeViewColumn* column;
    gint i;
    gint n;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 1; i < N_COLUMNS ; i++)
    {
        gtk_tree_view_append_column (GTK_TREE_VIEW (widget), gtk_tree_view_column_new ());
    }
    
    new_column = gtk_tree_view_column_new ();
    if (!new_column)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW_COLUMN (new_column))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    gtk_tree_view_column_set_min_width (new_column, 4917);
    n= gtk_tree_view_insert_column (GTK_TREE_VIEW (widget), new_column, POSITION);
    if (n != N_COLUMNS)
    {
        TEST_FAIL("gtk_tree_view_insert_column () return %i, should return %i", n, N_COLUMNS);
    }
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (widget), POSITION);
    if (gtk_tree_view_column_get_min_width (column) != 4917)
    {
        TEST_FAIL("The column at the %i position differs from that was inserted. Min width of this column = %i", POSITION, gtk_tree_view_column_get_min_width (column));
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef N_COLUMNS
    #undef POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_insert_column
 * Params: GtkTreeView* tree_view, GtkTreeViewColumn* column, gint position
 * Returns: gint
 * Assertion: This inserts the column into the tree_view at position. 
 * Params Values: newly created GtkTreeView*, newly created GtkTreeViewColumn*, 10
 * Expected: 11  
 */
static void test_purpose_29()
{
    #define N_COLUMNS 11
    #define POSITION 10
    
    GtkWidget* widget;
    GtkTreeViewColumn* new_column;
    GtkTreeViewColumn* column;
    gint i;
    gint n;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 1; i < N_COLUMNS ; i++)
    {
        gtk_tree_view_append_column (GTK_TREE_VIEW (widget), gtk_tree_view_column_new ());
    }
    
    new_column = gtk_tree_view_column_new ();
    if (!new_column)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW_COLUMN (new_column))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    gtk_tree_view_column_set_min_width (new_column, 4917);
    n= gtk_tree_view_insert_column (GTK_TREE_VIEW (widget), new_column, POSITION);
    if (n != N_COLUMNS)
    {
        TEST_FAIL("gtk_tree_view_insert_column () return %i, should return %i", n, N_COLUMNS);
    }
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (widget), POSITION);
    if (gtk_tree_view_column_get_min_width (column) != 4917)
    {
        TEST_FAIL("The column at the %i position differs from that was inserted. Min width of this column = %i", POSITION, gtk_tree_view_column_get_min_width (column));
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef N_COLUMNS
    #undef POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_insert_column
 * Params: GtkTreeView* tree_view, GtkTreeViewColumn* column, gint position
 * Returns: gint
 * Assertion: If position is -1, then the column is inserted at the end.
 * Params Values: newly created GtkTreeView*, newly created GtkTreeViewColumn*, -1
 * Expected: 1  
 */
static void test_purpose_30()
{
    #define N_COLUMNS 1
    #define POSITION -1
    
    GtkWidget* widget;
    GtkTreeViewColumn* new_column;
    GtkTreeViewColumn* column;
    gint i;
    gint n;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 1; i < N_COLUMNS ; i++)
    {
        gtk_tree_view_append_column (GTK_TREE_VIEW (widget), gtk_tree_view_column_new ());
    }
    
    new_column = gtk_tree_view_column_new ();
    if (!new_column)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW_COLUMN (new_column))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    gtk_tree_view_column_set_min_width (new_column, 4917);
    n= gtk_tree_view_insert_column (GTK_TREE_VIEW (widget), new_column, POSITION);
    if (n != N_COLUMNS)
    {
        TEST_FAIL("gtk_tree_view_insert_column () return %i, should return %i", n, N_COLUMNS);
    }
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (widget), N_COLUMNS-1);
    if (gtk_tree_view_column_get_min_width (column) != 4917)
    {
        TEST_FAIL("The column at the %i position differs from that was inserted. Min width of this column = %i", POSITION, gtk_tree_view_column_get_min_width (column));
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef N_COLUMNS
    #undef POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_bin_window
 * Params: GtkTreeView *tree_view
 * Returns: GdkWindow*
 * Assertion: Returns the window that tree_view renders to.
 * Params Values: void
 * Expected: GdkWindow* if tree_view was realized, NULL otherwise  
 */
static void test_purpose_31()
{
    #define REALIZE TRUE
    
    GtkWidget *widget;
    GtkWidget *window;
    GdkWindow *gdk_window;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    if (REALIZE)
    {
        window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
        gtk_container_add (GTK_CONTAINER (window), widget);
        gtk_widget_realize (widget);
    }
    
    gdk_window = gtk_tree_view_get_bin_window (GTK_TREE_VIEW (widget));
    if (REALIZE)
    {
        if (gdk_window == NULL)
        {
            TEST_FAIL ("tree_view was realized but gdk_window = NULL", NULL);
        }
    } else {
        if (gdk_window != NULL)
        {
            TEST_FAIL ("tree_view was not realized but gdk_window != NULL", NULL);
        }
    }
    
    /******* VISUALISATION *******/
    if (!REALIZE)
    {
        TEST_VISUALIZE (widget);
    }
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef REALIZE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_bin_window
 * Params: GtkTreeView *tree_view
 * Returns: GdkWindow*
 * Assertion: Returns the window that tree_view renders to.
 * Params Values: void
 * Expected: GdkWindow* if tree_view was realized, NULL otherwise  
 */
static void test_purpose_32()
{
    #define REALIZE FALSE
    
    GtkWidget *widget;
    GtkWidget *window;
    GdkWindow *gdk_window;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    if (REALIZE)
    {
        window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
        gtk_container_add (GTK_CONTAINER (window), widget);
        gtk_widget_realize (widget);
    }
    
    gdk_window = gtk_tree_view_get_bin_window (GTK_TREE_VIEW (widget));
    if (REALIZE)
    {
        if (gdk_window == NULL)
        {
            TEST_FAIL ("tree_view was realized but gdk_window = NULL", NULL);
        }
    } else {
        if (gdk_window != NULL)
        {
            TEST_FAIL ("tree_view was not realized but gdk_window != NULL", NULL);
        }
    }
    
    /******* VISUALISATION *******/
    if (!REALIZE)
    {
        TEST_VISUALIZE (widget);
    }
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef REALIZE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_reorderable
 * Params: GtkTreeView* tree_view, gboolean reorderable
 * Returns: void
 * Assertion: This function is a convenience function to allow you to reorder models that support the GtkDragSourceIface and the GtkDragDestIface
 * Params Values: newly created GtkTreeView*, TRUE
 * Expected: void
 * 
 * Name: gtk_tree_view_get_reorderable
 * Params: GtkTreeView* tree_view
 * Returns: gboolean
 * Assertion: Retrieves whether the user can reorder the tree via drag-and-drop
 * Params Values: newly created GtkTreeView* 
 * Expected: TRUE  
 */
static void test_purpose_33()
{
    #define REORDERABLE TRUE
    
    GtkWidget* widget;
    gboolean reorderable;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    };
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    };
    
    gtk_tree_view_set_reorderable (GTK_TREE_VIEW (widget), REORDERABLE);
    reorderable = gtk_tree_view_get_reorderable (GTK_TREE_VIEW (widget));
    
    if (reorderable != REORDERABLE)
    {
        TEST_FAIL ("gtk_tree_view_get_reorderable() return %i, instead of %i.", !REORDERABLE, REORDERABLE);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef REORDERABLE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_reorderable
 * Params: GtkTreeView* tree_view, gboolean reorderable
 * Returns: void
 * Assertion: This function is a convenience function to allow you to reorder models that support the GtkDragSourceIface and the GtkDragDestIface
 * Params Values: newly created GtkTreeView*, FALSE
 * Expected: void
 * 
 * Name: gtk_tree_view_get_reorderable
 * Params: GtkTreeView* tree_view
 * Returns: gboolean
 * Assertion: Retrieves whether the user can reorder the tree via drag-and-drop
 * Params Values: newly created GtkTreeView* 
 * Expected: FALSE  
 */
static void test_purpose_34()
{
    #define REORDERABLE FALSE
    
    GtkWidget* widget;
    gboolean reorderable;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    };
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    };
    
    gtk_tree_view_set_reorderable (GTK_TREE_VIEW (widget), REORDERABLE);
    reorderable = gtk_tree_view_get_reorderable (GTK_TREE_VIEW (widget));
    
    if (reorderable != REORDERABLE)
    {
        TEST_FAIL ("gtk_tree_view_get_reorderable() return %i, instead of %i.", !REORDERABLE, REORDERABLE);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef REORDERABLE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_rules_hint
 * Params: GtkTreeView* tree_view, gboolean setting
 * Returns: void
 * Assertion: This function tells GTK+ that the user interface for your application requires users to read across tree rows and associate cells with one another
 * Params Values: newly created GtkTreeView*, TRUE
 * Expected: void
 * 
 * Name: gtk_tree_view_get_rules_hint
 * Params: GtkTreeView* tree_view
 * Returns: gboolean
 * Assertion: Gets the setting set by gtk_tree_view_set_rules_hint().
 * Params Values: newly created GtkTreeView* 
 * Expected: TRUE  
 */
static void test_purpose_35()
{
    #define SETTING TRUE
    
    GtkWidget* widget;
    gboolean setting;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    };
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    };
    
    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (widget), SETTING);
    setting = gtk_tree_view_get_rules_hint (GTK_TREE_VIEW (widget));
    
    if (setting != SETTING)
    {
        TEST_FAIL ("gtk_tree_view_get_rules_hint() return %i, instead of %i.", !SETTING, SETTING);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef SETTING
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_rules_hint
 * Params: GtkTreeView* tree_view, gboolean setting
 * Returns: void
 * Assertion: This function tells GTK+ that the user interface for your application requires users to read across tree rows and associate cells with one another
 * Params Values: newly created GtkTreeView*, FALSE
 * Expected: void
 * 
 * Name: gtk_tree_view_get_rules_hint
 * Params: GtkTreeView* tree_view
 * Returns: gboolean
 * Assertion: Gets the setting set by gtk_tree_view_set_rules_hint().
 * Params Values: newly created GtkTreeView* 
 * Expected: FALSE  
 */
static void test_purpose_36()
{
    #define SETTING FALSE
    
    GtkWidget* widget;
    gboolean setting;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    widget = gtk_tree_view_new ();
    if (!widget)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    };
    
    if (!GTK_IS_TREE_VIEW (widget))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    };
    
    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (widget), SETTING);
    setting = gtk_tree_view_get_rules_hint (GTK_TREE_VIEW (widget));
    
    if (setting != SETTING)
    {
        TEST_FAIL ("gtk_tree_view_get_rules_hint() return %i, instead of %i.", !SETTING, SETTING);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (widget);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (widget));
    
    #undef SETTING
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_create_row_drag_icon
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: GdkPixmap*
 * Assertion: Creates a GdkPixmap representation of the row at path. 
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "0:0:0"
 * Expected: GdkPixmap*  
 */
static void test_purpose_37()
{
    #define PATH "0:0:0"
    #define DEPTH 3
    #define INDICES {0, 0, 0}
    
    GtkWidget *toplevel;
    GtkWidget *window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkPixmap* pixmap;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    
    path = gtk_tree_path_new_from_string (PATH);
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    
    toplevel = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_add (GTK_CONTAINER (toplevel), window);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (toplevel);
    /******* -------------  *******/  
    
    pixmap = gtk_tree_view_create_row_drag_icon (GTK_TREE_VIEW (tree_view), path);
    
    gtk_widget_destroy (toplevel);
    #undef PATH
    #undef DEPTH
    #undef INDICES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_create_row_drag_icon
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: GdkPixmap*
 * Assertion: Creates a GdkPixmap representation of the row at path. 
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "0"
 * Expected: GdkPixmap*  
 */
static void test_purpose_38()
{
    #define PATH "0"
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    
    GtkWidget *toplevel;
    GtkWidget *window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkPixmap* pixmap;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    
    path = gtk_tree_path_new_from_string (PATH);
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    
    toplevel = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_add (GTK_CONTAINER (toplevel), window);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (toplevel);
    /******* -------------  *******/  
    
    pixmap = gtk_tree_view_create_row_drag_icon (GTK_TREE_VIEW (tree_view), path);
    
    gtk_widget_destroy (toplevel);
    #undef PATH
    #undef DEPTH
    #undef INDICES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_create_row_drag_icon
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: GdkPixmap*
 * Assertion: Creates a GdkPixmap representation of the row at path. 
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0"
 * Expected: GdkPixmap*  
 */
static void test_purpose_39()
{
    #define PATH "0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0"
    #define DEPTH 27
    #define INDICES {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
    
    GtkWidget *toplevel;
    GtkWidget *window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkPixmap* pixmap;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    
    path = gtk_tree_path_new_from_string (PATH);
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    
    toplevel = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_add (GTK_CONTAINER (toplevel), window);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (toplevel);
    /******* -------------  *******/  
    
    pixmap = gtk_tree_view_create_row_drag_icon (GTK_TREE_VIEW (tree_view), path);
    
    gtk_widget_destroy (toplevel);
    #undef PATH
    #undef DEPTH
    #undef INDICES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: Expands the row at path.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "0"
 * Expected: void
 * 
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: This will also expand all parent rows of path as necessary.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "0"
 * Expected: void  
 */
static void test_purpose_40()
{
    #define PATH "0"
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    gboolean has_parent;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    path = gtk_tree_path_new_from_string (PATH);
    
    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (tree_store, &iter, NULL);
        gtk_tree_store_set (tree_store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (tree_store, &child_iter, &iter);
            gtk_tree_store_set (tree_store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    gtk_tree_view_expand_to_path (GTK_TREE_VIEW (tree_view), path);
    
    if (strchr (PATH, ':'))
    {
        has_parent = gtk_tree_path_up (path);
        while (has_parent && gtk_tree_path_to_string (path))
        {
            if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
            {
                TEST_FAIL ("parent row at '%s' is not expanded, original path is '%s'", 
                           gtk_tree_path_to_string (path), PATH);
                gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
                return;
            }
            has_parent = gtk_tree_path_up (path);
        }
    } else {
        if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
        {
            TEST_FAIL ("row at '%s' is not expanded", PATH);
            gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
            return;
        }
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: Expands the row at path.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "0:1"
 * Expected: void
 * 
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: This will also expand all parent rows of path as necessary.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "0:1"
 * Expected: void  
 */
static void test_purpose_41()
{
    #define PATH "0:1"
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    gboolean has_parent;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    path = gtk_tree_path_new_from_string (PATH);
    
    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (tree_store, &iter, NULL);
        gtk_tree_store_set (tree_store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (tree_store, &child_iter, &iter);
            gtk_tree_store_set (tree_store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    gtk_tree_view_expand_to_path (GTK_TREE_VIEW (tree_view), path);
    
    if (strchr (PATH, ':'))
    {
        has_parent = gtk_tree_path_up (path);
        while (has_parent && gtk_tree_path_to_string (path))
        {
            if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
            {
                TEST_FAIL ("parent row at '%s' is not expanded, original path is '%s'", 
                           gtk_tree_path_to_string (path), PATH);
                gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
                return;
            }
            has_parent = gtk_tree_path_up (path);
        }
    } else {
        if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
        {
            TEST_FAIL ("row at '%s' is not expanded", PATH);
            gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
            return;
        }
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: Expands the row at path.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "2"
 * Expected: void
 * 
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: This will also expand all parent rows of path as necessary.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "2"
 * Expected: void  
 */
static void test_purpose_42()
{
    #define PATH "2"
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    gboolean has_parent;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    path = gtk_tree_path_new_from_string (PATH);
    
    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (tree_store, &iter, NULL);
        gtk_tree_store_set (tree_store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (tree_store, &child_iter, &iter);
            gtk_tree_store_set (tree_store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    gtk_tree_view_expand_to_path (GTK_TREE_VIEW (tree_view), path);
    
    if (strchr (PATH, ':'))
    {
        has_parent = gtk_tree_path_up (path);
        while (has_parent && gtk_tree_path_to_string (path))
        {
            if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
            {
                TEST_FAIL ("parent row at '%s' is not expanded, original path is '%s'", 
                           gtk_tree_path_to_string (path), PATH);
                gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
                return;
            }
            has_parent = gtk_tree_path_up (path);
        }
    } else {
        if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
        {
            TEST_FAIL ("row at '%s' is not expanded", PATH);
            gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
            return;
        }
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: Expands the row at path.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "4"
 * Expected: void
 * 
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: This will also expand all parent rows of path as necessary.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "4"
 * Expected: void  
 */
static void test_purpose_43()
{
    #define PATH "4"
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    gboolean has_parent;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    path = gtk_tree_path_new_from_string (PATH);
    
    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (tree_store, &iter, NULL);
        gtk_tree_store_set (tree_store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (tree_store, &child_iter, &iter);
            gtk_tree_store_set (tree_store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    gtk_tree_view_expand_to_path (GTK_TREE_VIEW (tree_view), path);
    
    if (strchr (PATH, ':'))
    {
        has_parent = gtk_tree_path_up (path);
        while (has_parent && gtk_tree_path_to_string (path))
        {
            if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
            {
                TEST_FAIL ("parent row at '%s' is not expanded, original path is '%s'", 
                           gtk_tree_path_to_string (path), PATH);
                gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
                return;
            }
            has_parent = gtk_tree_path_up (path);
        }
    } else {
        if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
        {
            TEST_FAIL ("row at '%s' is not expanded", PATH);
            gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
            return;
        }
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: Expands the row at path.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "0:0"
 * Expected: void
 * 
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: This will also expand all parent rows of path as necessary.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "0:0"
 * Expected: void  
 */
static void test_purpose_44()
{
    #define PATH "0:0"
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    gboolean has_parent;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    path = gtk_tree_path_new_from_string (PATH);
    
    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (tree_store, &iter, NULL);
        gtk_tree_store_set (tree_store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (tree_store, &child_iter, &iter);
            gtk_tree_store_set (tree_store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    gtk_tree_view_expand_to_path (GTK_TREE_VIEW (tree_view), path);
    
    if (strchr (PATH, ':'))
    {
        has_parent = gtk_tree_path_up (path);
        while (has_parent && gtk_tree_path_to_string (path))
        {
            if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
            {
                TEST_FAIL ("parent row at '%s' is not expanded, original path is '%s'", 
                           gtk_tree_path_to_string (path), PATH);
                gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
                return;
            }
            has_parent = gtk_tree_path_up (path);
        }
    } else {
        if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
        {
            TEST_FAIL ("row at '%s' is not expanded", PATH);
            gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
            return;
        }
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: Expands the row at path.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "0:4"
 * Expected: void
 * 
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: This will also expand all parent rows of path as necessary.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "0:4"
 * Expected: void  
 */
static void test_purpose_45()
{
    #define PATH "0:4"
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    gboolean has_parent;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    path = gtk_tree_path_new_from_string (PATH);
    
    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (tree_store, &iter, NULL);
        gtk_tree_store_set (tree_store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (tree_store, &child_iter, &iter);
            gtk_tree_store_set (tree_store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    gtk_tree_view_expand_to_path (GTK_TREE_VIEW (tree_view), path);
    
    if (strchr (PATH, ':'))
    {
        has_parent = gtk_tree_path_up (path);
        while (has_parent && gtk_tree_path_to_string (path))
        {
            if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
            {
                TEST_FAIL ("parent row at '%s' is not expanded, original path is '%s'", 
                           gtk_tree_path_to_string (path), PATH);
                gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
                return;
            }
            has_parent = gtk_tree_path_up (path);
        }
    } else {
        if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
        {
            TEST_FAIL ("row at '%s' is not expanded", PATH);
            gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
            return;
        }
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: Expands the row at path.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "1:0"
 * Expected: void
 * 
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: This will also expand all parent rows of path as necessary.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "1:0"
 * Expected: void  
 */
static void test_purpose_46()
{
    #define PATH "1:0"
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    gboolean has_parent;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    path = gtk_tree_path_new_from_string (PATH);
    
    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (tree_store, &iter, NULL);
        gtk_tree_store_set (tree_store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (tree_store, &child_iter, &iter);
            gtk_tree_store_set (tree_store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    gtk_tree_view_expand_to_path (GTK_TREE_VIEW (tree_view), path);
    
    if (strchr (PATH, ':'))
    {
        has_parent = gtk_tree_path_up (path);
        while (has_parent && gtk_tree_path_to_string (path))
        {
            if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
            {
                TEST_FAIL ("parent row at '%s' is not expanded, original path is '%s'", 
                           gtk_tree_path_to_string (path), PATH);
                gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
                return;
            }
            has_parent = gtk_tree_path_up (path);
        }
    } else {
        if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
        {
            TEST_FAIL ("row at '%s' is not expanded", PATH);
            gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
            return;
        }
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: Expands the row at path.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "1:3"
 * Expected: void
 * 
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: This will also expand all parent rows of path as necessary.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "1:3"
 * Expected: void  
 */
static void test_purpose_47()
{
    #define PATH "1:3"
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    gboolean has_parent;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    path = gtk_tree_path_new_from_string (PATH);
    
    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (tree_store, &iter, NULL);
        gtk_tree_store_set (tree_store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (tree_store, &child_iter, &iter);
            gtk_tree_store_set (tree_store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    gtk_tree_view_expand_to_path (GTK_TREE_VIEW (tree_view), path);
    
    if (strchr (PATH, ':'))
    {
        has_parent = gtk_tree_path_up (path);
        while (has_parent && gtk_tree_path_to_string (path))
        {
            if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
            {
                TEST_FAIL ("parent row at '%s' is not expanded, original path is '%s'", 
                           gtk_tree_path_to_string (path), PATH);
                gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
                return;
            }
            has_parent = gtk_tree_path_up (path);
        }
    } else {
        if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
        {
            TEST_FAIL ("row at '%s' is not expanded", PATH);
            gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
            return;
        }
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: Expands the row at path.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "1:4"
 * Expected: void
 * 
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: This will also expand all parent rows of path as necessary.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "1:4"
 * Expected: void  
 */
static void test_purpose_48()
{
    #define PATH "1:4"
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    gboolean has_parent;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    path = gtk_tree_path_new_from_string (PATH);
    
    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (tree_store, &iter, NULL);
        gtk_tree_store_set (tree_store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (tree_store, &child_iter, &iter);
            gtk_tree_store_set (tree_store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    gtk_tree_view_expand_to_path (GTK_TREE_VIEW (tree_view), path);
    
    if (strchr (PATH, ':'))
    {
        has_parent = gtk_tree_path_up (path);
        while (has_parent && gtk_tree_path_to_string (path))
        {
            if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
            {
                TEST_FAIL ("parent row at '%s' is not expanded, original path is '%s'", 
                           gtk_tree_path_to_string (path), PATH);
                gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
                return;
            }
            has_parent = gtk_tree_path_up (path);
        }
    } else {
        if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
        {
            TEST_FAIL ("row at '%s' is not expanded", PATH);
            gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
            return;
        }
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: Expands the row at path.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "4:0"
 * Expected: void
 * 
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: This will also expand all parent rows of path as necessary.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "4:0"
 * Expected: void  
 */
static void test_purpose_49()
{
    #define PATH "4:0"
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    gboolean has_parent;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    path = gtk_tree_path_new_from_string (PATH);
    
    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (tree_store, &iter, NULL);
        gtk_tree_store_set (tree_store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (tree_store, &child_iter, &iter);
            gtk_tree_store_set (tree_store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    gtk_tree_view_expand_to_path (GTK_TREE_VIEW (tree_view), path);
    
    if (strchr (PATH, ':'))
    {
        has_parent = gtk_tree_path_up (path);
        while (has_parent && gtk_tree_path_to_string (path))
        {
            if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
            {
                TEST_FAIL ("parent row at '%s' is not expanded, original path is '%s'", 
                           gtk_tree_path_to_string (path), PATH);
                gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
                return;
            }
            has_parent = gtk_tree_path_up (path);
        }
    } else {
        if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
        {
            TEST_FAIL ("row at '%s' is not expanded", PATH);
            gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
            return;
        }
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: Expands the row at path.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "4:3"
 * Expected: void
 * 
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: This will also expand all parent rows of path as necessary.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "4:3"
 * Expected: void  
 */
static void test_purpose_50()
{
    #define PATH "4:3"
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    gboolean has_parent;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    path = gtk_tree_path_new_from_string (PATH);
    
    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (tree_store, &iter, NULL);
        gtk_tree_store_set (tree_store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (tree_store, &child_iter, &iter);
            gtk_tree_store_set (tree_store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    gtk_tree_view_expand_to_path (GTK_TREE_VIEW (tree_view), path);
    
    if (strchr (PATH, ':'))
    {
        has_parent = gtk_tree_path_up (path);
        while (has_parent && gtk_tree_path_to_string (path))
        {
            if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
            {
                TEST_FAIL ("parent row at '%s' is not expanded, original path is '%s'", 
                           gtk_tree_path_to_string (path), PATH);
                gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
                return;
            }
            has_parent = gtk_tree_path_up (path);
        }
    } else {
        if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
        {
            TEST_FAIL ("row at '%s' is not expanded", PATH);
            gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
            return;
        }
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: Expands the row at path.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "4:4"
 * Expected: void
 * 
 * Name: gtk_tree_view_expand_to_path
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: void
 * Assertion: This will also expand all parent rows of path as necessary.
 * Params Values: newly created GtkTreeView*, GtkTreePath* created from "4:4"
 * Expected: void  
 */
static void test_purpose_51()
{
    #define PATH "4:4"
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    gboolean has_parent;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    path = gtk_tree_path_new_from_string (PATH);
    
    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (tree_store, &iter, NULL);
        gtk_tree_store_set (tree_store, &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (tree_store, &child_iter, &iter);
            gtk_tree_store_set (tree_store, &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    gtk_tree_view_expand_to_path (GTK_TREE_VIEW (tree_view), path);
    
    if (strchr (PATH, ':'))
    {
        has_parent = gtk_tree_path_up (path);
        while (has_parent && gtk_tree_path_to_string (path))
        {
            if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
            {
                TEST_FAIL ("parent row at '%s' is not expanded, original path is '%s'", 
                           gtk_tree_path_to_string (path), PATH);
                gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
                return;
            }
            has_parent = gtk_tree_path_up (path);
        }
    } else {
        if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
        {
            TEST_FAIL ("row at '%s' is not expanded", PATH);
            gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
            return;
        }
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_all
 * Params: GtkTreeView *tree_view
 * Returns: void
 * Assertion: Recursively expands all nodes in the tree_view. 
 * Params Values: newly created GtkTreeView*
 * Expected: void
 * 
 * Name: gtk_tree_view_collapse_all
 * Params: GtkTreeView *tree_view
 * Returns: void
 * Assertion: Recursively collapses all visible, expanded nodes in tree_view.
 * Params Values: newly created GtkTreeView*
 * Expected: void  
 */
static void test_purpose_52()
{
    #define DEPTH 2
    #define INDICES {0, 0}
    #define PATH "0"
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    path = gtk_tree_path_new_from_string (PATH);
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
    {
        TEST_FAIL ("tree_view is not expanded, but it should be expanded.", NULL);
    }
    gtk_tree_view_collapse_all (GTK_TREE_VIEW (tree_view));
    if (gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
    {
        TEST_FAIL ("tree_view is expanded, but it shouldn't be expanded.", NULL);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef DEPTH
    #undef INDICES
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_all
 * Params: GtkTreeView *tree_view
 * Returns: void
 * Assertion: Recursively expands all nodes in the tree_view. 
 * Params Values: newly created GtkTreeView*
 * Expected: void
 * 
 * Name: gtk_tree_view_collapse_all
 * Params: GtkTreeView *tree_view
 * Returns: void
 * Assertion: Recursively collapses all visible, expanded nodes in tree_view.
 * Params Values: newly created GtkTreeView*
 * Expected: void  
 */
static void test_purpose_53()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define PATH "5"
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    path = gtk_tree_path_new_from_string (PATH);
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
    {
        TEST_FAIL ("tree_view is not expanded, but it should be expanded.", NULL);
    }
    gtk_tree_view_collapse_all (GTK_TREE_VIEW (tree_view));
    if (gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
    {
        TEST_FAIL ("tree_view is expanded, but it shouldn't be expanded.", NULL);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef DEPTH
    #undef INDICES
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_all
 * Params: GtkTreeView *tree_view
 * Returns: void
 * Assertion: Recursively expands all nodes in the tree_view. 
 * Params Values: newly created GtkTreeView*
 * Expected: void
 * 
 * Name: gtk_tree_view_collapse_all
 * Params: GtkTreeView *tree_view
 * Returns: void
 * Assertion: Recursively collapses all visible, expanded nodes in tree_view.
 * Params Values: newly created GtkTreeView*
 * Expected: void  
 */
static void test_purpose_54()
{
    #define DEPTH 27
    #define INDICES {9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
    #define PATH "9:0:0" 
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    path = gtk_tree_path_new_from_string (PATH);
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
    {
        TEST_FAIL ("tree_view is not expanded, but it should be expanded.", NULL);
    }
    gtk_tree_view_collapse_all (GTK_TREE_VIEW (tree_view));
    if (gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
    {
        TEST_FAIL ("tree_view is expanded, but it shouldn't be expanded.", NULL);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef DEPTH
    #undef INDICES
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_row
 * Params: GtkTreeView *tree_view, GtkTreePath *path, gboolean open_all
 * Returns: gboolean
 * Assertion: Opens the row so its children are visible.
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "0:0:0", TRUE 
 * Expected: FALSE
 * 
 * Name: gtk_tree_view_collapse_row
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: gboolean
 * Assertion: Collapses a row (hides its child rows, if they exist).
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "0:0:0"
 * Expected: FALSE  
 */
static void test_purpose_55()
{
    #define PATH "0:0:0"
    #define DEPTH 3
    #define INDICES {0, 0, 0}
    #define OPEN_ALL TRUE 
    #define EXPAND_RESULT FALSE
    #define COLLAPSE_RESULT FALSE
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    gint indices[] = INDICES;
    guint i;
    guint j;
    gboolean result;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    
    path = gtk_tree_path_new_from_string (PATH);
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    result = gtk_tree_view_collapse_row (GTK_TREE_VIEW (tree_view), path);
    if (result != COLLAPSE_RESULT)
    {
        TEST_FAIL ("gtk_tree_view_collapse_row() return %i, instead of %i.", !COLLAPSE_RESULT, COLLAPSE_RESULT);
    }
    if (gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path) && COLLAPSE_RESULT)
    {
        TEST_FAIL ("tree_view row is not collapsed, but it should be collapsed.", NULL);
    }
    
    result = gtk_tree_view_expand_row (GTK_TREE_VIEW (tree_view), path, OPEN_ALL);
    if (result != EXPAND_RESULT)
    {
        TEST_FAIL ("gtk_tree_view_expand_row() return %i, instead of %i.", !EXPAND_RESULT, EXPAND_RESULT);
    }
    if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path) && EXPAND_RESULT)
    {
        TEST_FAIL ("tree_view row is not expanded, but it should be expanded.", NULL);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    #undef DEPTH
    #undef INDICES
    #undef OPEN_ALL
    #undef EXPAND_RESULT
    #undef COLLAPSE_RESULT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_row
 * Params: GtkTreeView *tree_view, GtkTreePath *path, gboolean open_all
 * Returns: gboolean
 * Assertion: Opens the row so its children are visible.
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "5:9", TRUE
 * Expected: FALSE
 * 
 * Name: gtk_tree_view_collapse_row
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: gboolean
 * Assertion: Collapses a row (hides its child rows, if they exist).
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "5:9"
 * Expected: FALSE  
 */
static void test_purpose_56()
{
    #define PATH "5:9"
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define OPEN_ALL TRUE
    #define EXPAND_RESULT FALSE
    #define COLLAPSE_RESULT FALSE
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    gint indices[] = INDICES;
    guint i;
    guint j;
    gboolean result;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    
    path = gtk_tree_path_new_from_string (PATH);
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    result = gtk_tree_view_collapse_row (GTK_TREE_VIEW (tree_view), path);
    if (result != COLLAPSE_RESULT)
    {
        TEST_FAIL ("gtk_tree_view_collapse_row() return %i, instead of %i.", !COLLAPSE_RESULT, COLLAPSE_RESULT);
    }
    if (gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path) && COLLAPSE_RESULT)
    {
        TEST_FAIL ("tree_view row is not collapsed, but it should be collapsed.", NULL);
    }
    
    result = gtk_tree_view_expand_row (GTK_TREE_VIEW (tree_view), path, OPEN_ALL);
    if (result != EXPAND_RESULT)
    {
        TEST_FAIL ("gtk_tree_view_expand_row() return %i, instead of %i.", !EXPAND_RESULT, EXPAND_RESULT);
    }
    if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path) && EXPAND_RESULT)
    {
        TEST_FAIL ("tree_view row is not expanded, but it should be expanded.", NULL);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    #undef DEPTH
    #undef INDICES
    #undef OPEN_ALL
    #undef EXPAND_RESULT
    #undef COLLAPSE_RESULT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_expand_row
 * Params: GtkTreeView *tree_view, GtkTreePath *path, gboolean open_all
 * Returns: gboolean
 * Assertion: Opens the row so its children are visible.
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "5:2", FALSE
 * Expected: FALSE
 * 
 * Name: gtk_tree_view_collapse_row
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: gboolean
 * Assertion: Collapses a row (hides its child rows, if they exist).
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "5:2"
 * Expected: FALSE  
 */
static void test_purpose_57()
{
    #define PATH "5:2"
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define OPEN_ALL FALSE
    #define EXPAND_RESULT FALSE
    #define COLLAPSE_RESULT FALSE
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    gint indices[] = INDICES;
    guint i;
    guint j;
    gboolean result;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    
    path = gtk_tree_path_new_from_string (PATH);
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    result = gtk_tree_view_collapse_row (GTK_TREE_VIEW (tree_view), path);
    if (result != COLLAPSE_RESULT)
    {
        TEST_FAIL ("gtk_tree_view_collapse_row() return %i, instead of %i.", !COLLAPSE_RESULT, COLLAPSE_RESULT);
    }
    if (gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path) && COLLAPSE_RESULT)
    {
        TEST_FAIL ("tree_view row is not collapsed, but it should be collapsed.", NULL);
    }
    
    result = gtk_tree_view_expand_row (GTK_TREE_VIEW (tree_view), path, OPEN_ALL);
    if (result != EXPAND_RESULT)
    {
        TEST_FAIL ("gtk_tree_view_expand_row() return %i, instead of %i.", !EXPAND_RESULT, EXPAND_RESULT);
    }
    if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path) && EXPAND_RESULT)
    {
        TEST_FAIL ("tree_view row is not expanded, but it should be expanded.", NULL);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    #undef DEPTH
    #undef INDICES
    #undef OPEN_ALL
    #undef EXPAND_RESULT
    #undef COLLAPSE_RESULT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_row_expanded
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: gboolean
 * Assertion: Returns TRUE if the node pointed to by path is expanded in tree_view.
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "0" 
 * Expected: TRUE if path is expanded.  
 */
static void test_purpose_58()
{
    #define DEPTH 2
    #define INDICES {0, 0}
    #define PATH "0"
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    path = gtk_tree_path_new_from_string (PATH);
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
    {
        TEST_FAIL ("gtk_tree_view_row_expanded () return FALSE for expanded row!", NULL);
    }
    gtk_tree_view_collapse_all (GTK_TREE_VIEW (tree_view));
    if (gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
    {
        TEST_FAIL ("gtk_tree_view_row_expanded () return TRUE for collapsed row!", NULL);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    #undef DEPTH
    #undef INDICES
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_row_expanded
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: gboolean
 * Assertion: Returns TRUE if the node pointed to by path is expanded in tree_view.
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "5" 
 * Expected: TRUE if path is expanded.  
 */
static void test_purpose_59()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define PATH "5"
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    path = gtk_tree_path_new_from_string (PATH);
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
    {
        TEST_FAIL ("gtk_tree_view_row_expanded () return FALSE for expanded row!", NULL);
    }
    gtk_tree_view_collapse_all (GTK_TREE_VIEW (tree_view));
    if (gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
    {
        TEST_FAIL ("gtk_tree_view_row_expanded () return TRUE for collapsed row!", NULL);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    #undef DEPTH
    #undef INDICES
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_row_expanded
 * Params: GtkTreeView *tree_view, GtkTreePath *path
 * Returns: gboolean
 * Assertion: Returns TRUE if the node pointed to by path is expanded in tree_view.
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "9:0:0"  
 * Expected: TRUE if path is expanded.  
 */
static void test_purpose_60()
{
    #define DEPTH 27
    #define INDICES {9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
    #define PATH "9:0:0" 
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    path = gtk_tree_path_new_from_string (PATH);
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
    {
        TEST_FAIL ("gtk_tree_view_row_expanded () return FALSE for expanded row!", NULL);
    }
    gtk_tree_view_collapse_all (GTK_TREE_VIEW (tree_view));
    if (gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
    {
        TEST_FAIL ("gtk_tree_view_row_expanded () return TRUE for collapsed row!", NULL);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    #undef DEPTH
    #undef INDICES
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_background_area
 * Params: GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GdkRectangle *rect
 * Returns: void
 * Assertion: Fills the bounding rectangle in tree window coordinates for the cell at the row specified by path and the column specified by column.
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "0", GtkTreeViewColumn from 0 position, GdkRectangle 
 * Expected: void  
 */
static void test_purpose_61()
{
    #define DEPTH 2
    #define INDICES {0, 0}
    #define PATH "0"
    #define COL_POSITION 0
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    path = gtk_tree_path_new_from_string (PATH);
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), 0);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    gtk_tree_view_get_background_area (GTK_TREE_VIEW (tree_view), path, column, &rect);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    #undef DEPTH
    #undef INDICES
    #undef PATH
    #undef COL_POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_background_area
 * Params: GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GdkRectangle *rect
 * Returns: void
 * Assertion: Fills the bounding rectangle in tree window coordinates for the cell at the row specified by path and the column specified by column.
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "5:1", GtkTreeViewColumn from 0 position, GdkRectangle 
 * Expected: void  
 */
static void test_purpose_62()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define PATH "5:1"
    #define COL_POSITION 0
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    path = gtk_tree_path_new_from_string (PATH);
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), 0);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    gtk_tree_view_get_background_area (GTK_TREE_VIEW (tree_view), path, column, &rect);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    #undef DEPTH
    #undef INDICES
    #undef PATH
    #undef COL_POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_background_area
 * Params: GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GdkRectangle *rect
 * Returns: void
 * Assertion: Fills the bounding rectangle in tree window coordinates for the cell at the row specified by path and the column specified by column.
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "9:0:0" , GtkTreeViewColumn from 0 position, GdkRectangle 
 * Expected: void  
 */
static void test_purpose_63()
{
    #define DEPTH 27
    #define INDICES {9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
    #define PATH "9:0:0" 
    #define COL_POSITION 0
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    path = gtk_tree_path_new_from_string (PATH);
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), 0);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    gtk_tree_view_get_background_area (GTK_TREE_VIEW (tree_view), path, column, &rect);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    #undef DEPTH
    #undef INDICES
    #undef PATH
    #undef COL_POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_background_area
 * Params: GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GdkRectangle *rect
 * Returns: void
 * Assertion: If path is NULL, or points to a node not found in the tree, the y and height fields of the rectangle will be filled with 0.
 * Params Values: newly created GtkTreeView*, <%2%>, GtkTreeViewColumn from <%3%> position, rectangle to fill with cell background rect
 * Expected: void  
 */
static void test_purpose_64()
{
    #define PATH NULL
    
    GtkWidget *tree_view;
    GtkTreeModel *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    if (!PATH)
    {
        path = NULL;
    } else {
        path = gtk_tree_path_new_from_string (PATH);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    gtk_tree_view_get_background_area (GTK_TREE_VIEW (tree_view), path, column1, &rect);
    
    if (rect.y !=0)
    {
        TEST_FAIL ("rect.y = %d, should be 0", rect.y);
    }
    
    if (rect.height !=0)
    {
        TEST_FAIL ("rect.height = %d, should be 0", rect.height);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_background_area
 * Params: GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GdkRectangle *rect
 * Returns: void
 * Assertion: If path is NULL, or points to a node not found in the tree, the y and height fields of the rectangle will be filled with 0.
 * Params Values: newly created GtkTreeView*, <%2%>, GtkTreeViewColumn from <%3%> position, rectangle to fill with cell background rect
 * Expected: void  
 */
static void test_purpose_65()
{
    #define PATH "55:7:3"
    
    GtkWidget *tree_view;
    GtkTreeModel *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    if (!PATH)
    {
        path = NULL;
    } else {
        path = gtk_tree_path_new_from_string (PATH);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    gtk_tree_view_get_background_area (GTK_TREE_VIEW (tree_view), path, column1, &rect);
    
    if (rect.y !=0)
    {
        TEST_FAIL ("rect.y = %d, should be 0", rect.y);
    }
    
    if (rect.height !=0)
    {
        TEST_FAIL ("rect.height = %d, should be 0", rect.height);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_background_area
 * Params: GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GdkRectangle *rect
 * Returns: void
 * Assertion: If path is NULL, or points to a node not found in the tree, the y and height fields of the rectangle will be filled with 0.
 * Params Values: newly created GtkTreeView*, <%2%>, GtkTreeViewColumn from <%3%> position, rectangle to fill with cell background rect
 * Expected: void  
 */
static void test_purpose_66()
{
    #define PATH "5:0" 
    
    GtkWidget *tree_view;
    GtkTreeModel *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    if (!PATH)
    {
        path = NULL;
    } else {
        path = gtk_tree_path_new_from_string (PATH);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    gtk_tree_view_get_background_area (GTK_TREE_VIEW (tree_view), path, column1, &rect);
    
    if (rect.y !=0)
    {
        TEST_FAIL ("rect.y = %d, should be 0", rect.y);
    }
    
    if (rect.height !=0)
    {
        TEST_FAIL ("rect.height = %d, should be 0", rect.height);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_background_area
 * Params: GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GdkRectangle *rect
 * Returns: void
 * Assertion: If column is NULL, the x and width fields will be filled with 0. 
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "9:0:0" , NULL, rectangle to fill with cell background rect
 * Expected: void  
 */
static void test_purpose_67()
{
    #define DEPTH 27
    #define INDICES {9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
    #define PATH "9:0:0" 
    #define COL_POSITION 0
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    column = NULL;
    if (!PATH)
    {
        path = NULL;
    } else {
        path = gtk_tree_path_new_from_string (PATH);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    gtk_tree_view_get_background_area (GTK_TREE_VIEW (tree_view), path, column, &rect);
    if (rect.x !=0)
    {
        TEST_FAIL ("rect.x = %d, should be 0", rect.x);
    }
    
    if (rect.width !=0)
    {
        TEST_FAIL ("rect.width = %d, should be 0", rect.width);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    #undef DEPTH
    #undef INDICES
    #undef PATH
    #undef COL_POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_cell_area
 * Params: GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GdkRectangle *rect
 * Returns: void
 * Assertion: Fills the bounding rectangle in tree window coordinates for the cell at the row specified by path and the column specified by column.
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "0", NULL, rectangle to fill with cell background rect
 * Expected: void  
 */
static void test_purpose_68()
{
    #define DEPTH 2
    #define INDICES {0, 0}
    #define PATH "0"
    #define COL_POSITION 0
    
    GtkWidget *window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    column = NULL;
    if (!PATH)
    {
        path = NULL;
    } else {
        path = gtk_tree_path_new_from_string (PATH);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column, &rect);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    #undef DEPTH
    #undef INDICES
    #undef PATH
    #undef COL_POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_cell_area
 * Params: GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GdkRectangle *rect
 * Returns: void
 * Assertion: Fills the bounding rectangle in tree window coordinates for the cell at the row specified by path and the column specified by column.
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "5:5", NULL, rectangle to fill with cell background rect
 * Expected: void  
 */
static void test_purpose_69()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define PATH "5:5"
    #define COL_POSITION 0
    
    GtkWidget *window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    column = NULL;
    if (!PATH)
    {
        path = NULL;
    } else {
        path = gtk_tree_path_new_from_string (PATH);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column, &rect);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    #undef DEPTH
    #undef INDICES
    #undef PATH
    #undef COL_POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_cell_area
 * Params: GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GdkRectangle *rect
 * Returns: void
 * Assertion: Fills the bounding rectangle in tree window coordinates for the cell at the row specified by path and the column specified by column.
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "9:0:0" , NULL, rectangle to fill with cell background rect
 * Expected: void  
 */
static void test_purpose_70()
{
    #define DEPTH 27
    #define INDICES {9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
    #define PATH "9:0:0" 
    #define COL_POSITION 0
    
    GtkWidget *window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    column = NULL;
    if (!PATH)
    {
        path = NULL;
    } else {
        path = gtk_tree_path_new_from_string (PATH);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column, &rect);
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    #undef DEPTH
    #undef INDICES
    #undef PATH
    #undef COL_POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_cell_area
 * Params: GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GdkRectangle *rect
 * Returns: void
 * Assertion: If column is NULL, the x and width fields will be filled with 0. 
 * Params Values: newly created GtkTreeView*, GtkTreePath* from "9:0:0" , NULL, rectangle to fill with cell background rect
 * Expected: void  
 */
static void test_purpose_71()
{
    #define DEPTH 27
    #define INDICES {9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
    #define PATH "9:0:0" 
    #define COL_POSITION 0
    
    GtkWidget *window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    column = NULL;
    if (!PATH)
    {
        path = NULL;
    } else {
        path = gtk_tree_path_new_from_string (PATH);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column, &rect);
    
    if (rect.x !=0)
    {
        TEST_FAIL ("rect.x = %d, should be 0", rect.x);
    }
    
    if (rect.width !=0)
    {
        TEST_FAIL ("rect.width = %d, should be 0", rect.width);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    #undef DEPTH
    #undef INDICES
    #undef PATH
    #undef COL_POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_cell_area
 * Params: GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GdkRectangle *rect
 * Returns: void
 * Assertion: If path is NULL, or points to a path not currently displayed, the y and height fields of the rectangle will be filled with 0. 
 * Params Values: newly created GtkTreeView*, NULL, GtkTreeViewColumn from position 0, rectangle to fill with cell background rect
 * Expected: void  
 */
static void test_purpose_72()
{
    #define PATH NULL
    
    GtkWidget *tree_view;
    GtkTreeModel *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    if (!PATH)
    {
        path = NULL;
    } else {
        path = gtk_tree_path_new_from_string (PATH);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column0, &rect);
    
    if (rect.y !=0)
    {
        TEST_FAIL ("rect.y = %d, should be 0", rect.y);
    }
    
    if (rect.height !=0)
    {
        TEST_FAIL ("rect.height = %d, should be 0", rect.height);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_cell_area
 * Params: GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GdkRectangle *rect
 * Returns: void
 * Assertion: If path is NULL, or points to a path not currently displayed, the y and height fields of the rectangle will be filled with 0. 
 * Params Values: newly created GtkTreeView*, "5:7:3", GtkTreeViewColumn from position 0, rectangle to fill with cell background rect
 * Expected: void  
 */
static void test_purpose_73()
{
    #define PATH "5:7:3"
    
    GtkWidget *tree_view;
    GtkTreeModel *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    if (!PATH)
    {
        path = NULL;
    } else {
        path = gtk_tree_path_new_from_string (PATH);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column0, &rect);
    
    if (rect.y !=0)
    {
        TEST_FAIL ("rect.y = %d, should be 0", rect.y);
    }
    
    if (rect.height !=0)
    {
        TEST_FAIL ("rect.height = %d, should be 0", rect.height);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_cell_area
 * Params: GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GdkRectangle *rect
 * Returns: void
 * Assertion: If path is NULL, or points to a path not currently displayed, the y and height fields of the rectangle will be filled with 0. 
 * Params Values: newly created GtkTreeView*, "5:0" , GtkTreeViewColumn from position 0, rectangle to fill with cell background rect
 * Expected: void  
 */
static void test_purpose_74()
{
    #define PATH "5:0" 
    
    GtkWidget *tree_view;
    GtkTreeModel *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    if (!PATH)
    {
        path = NULL;
    } else {
        path = gtk_tree_path_new_from_string (PATH);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column0, &rect);
    
    if (rect.y !=0)
    {
        TEST_FAIL ("rect.y = %d, should be 0", rect.y);
    }
    
    if (rect.height !=0)
    {
        TEST_FAIL ("rect.height = %d, should be 0", rect.height);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_cell_area
 * Params: GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GdkRectangle *rect
 * Returns: void
 * Assertion: If path is NULL, or points to a path not currently displayed, the y and height fields of the rectangle will be filled with 0. 
 * Params Values: newly created GtkTreeView*, "0:5", GtkTreeViewColumn from position 0, rectangle to fill with cell background rect
 * Expected: void  
 */
static void test_purpose_75()
{
    #define PATH "0:5"
    
    GtkWidget *tree_view;
    GtkTreeModel *tree_store;
    GtkTreeViewColumn *column0;
    GtkTreeViewColumn *column1;
    GtkCellRenderer* renderer;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    guint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT));
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    renderer = gtk_cell_renderer_text_new ();
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", renderer,
                                                       "text", 0,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    gtk_tree_view_column_set_sort_column_id (column0, 0);
    
    renderer = gtk_cell_renderer_text_new ();
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    gtk_tree_view_column_set_sort_column_id (column1, 1);
    
    if (!PATH)
    {
        path = NULL;
    } else {
        path = gtk_tree_path_new_from_string (PATH);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column0, &rect);
    
    if (rect.y !=0)
    {
        TEST_FAIL ("rect.y = %d, should be 0", rect.y);
    }
    
    if (rect.height !=0)
    {
        TEST_FAIL ("rect.height = %d, should be 0", rect.height);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef PATH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_enable_search
 * Params: GtkTreeView *tree_view, gboolean enable_search
 * Returns: void
 * Assertion: If enable_search is set, then the user can type in text to search through the tree interactively.
 * Params Values: newly created GtkTreeView*, TRUE
 * Expected: void
 * 
 * Name: gtk_tree_view_get_enable_search
 * Params: GtkTreeView *tree_view
 * Returns: gboolean
 * Assertion: Returns whether or not the tree allows interactive searching.
 * Params Values: newly created GtkTreeView*
 * Expected: TRUE  
 */
static void test_purpose_76()
{
    #define ENABLE_SEARCH TRUE
    
    GtkWidget *tree_view;
    gboolean enable_search;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new ();
    
    gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tree_view), ENABLE_SEARCH);
    enable_search = gtk_tree_view_get_enable_search (GTK_TREE_VIEW (tree_view));
    
    if (enable_search != ENABLE_SEARCH)
    {
        TEST_FAIL ("enable_search = %i, instead of %i", enable_search, ENABLE_SEARCH);
    }
    
    #undef ENABLE_SEARCH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_enable_search
 * Params: GtkTreeView *tree_view, gboolean enable_search
 * Returns: void
 * Assertion: If enable_search is set, then the user can type in text to search through the tree interactively.
 * Params Values: newly created GtkTreeView*, FALSE
 * Expected: void
 * 
 * Name: gtk_tree_view_get_enable_search
 * Params: GtkTreeView *tree_view
 * Returns: gboolean
 * Assertion: Returns whether or not the tree allows interactive searching.
 * Params Values: newly created GtkTreeView*
 * Expected: FALSE  
 */
static void test_purpose_77()
{
    #define ENABLE_SEARCH FALSE
    
    GtkWidget *tree_view;
    gboolean enable_search;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new ();
    
    gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tree_view), ENABLE_SEARCH);
    enable_search = gtk_tree_view_get_enable_search (GTK_TREE_VIEW (tree_view));
    
    if (enable_search != ENABLE_SEARCH)
    {
        TEST_FAIL ("enable_search = %i, instead of %i", enable_search, ENABLE_SEARCH);
    }
    
    #undef ENABLE_SEARCH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_cursor
 * Params: GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *focus_column, gboolean start_editing
 * Returns: void
 * Assertion: Sets the current keyboard focus to be at path, and selects it
 * Params Values: newly created GtkTreeView*, "9:0:0" , GtkTreeViewColumn from 0 position, TRUE
 * Expected: void
 * 
 * Name: gtk_tree_view_get_cursor
 * Params: GtkTreeView *tree_view, GtkTreePath **path, GtkTreeViewColumn **focus_column
 * Returns: void
 * Assertion: Fills in path and focus_column with the current path and focus column
 * Params Values: newly created GtkTreeView*, a pointer to be filled with the current cursor path, a pointer to be filled with the current focus column
 * Expected: void
 * 
 * Name: gtk_tree_view_get_cursor
 * Params: GtkTreeView *tree_view, GtkTreePath **path, GtkTreeViewColumn **focus_column
 * Returns: void
 * Assertion: If no column currently has focus, then* focus_column will be NULL
 * Params Values: newly created GtkTreeView*, a pointer to be filled with the current cursor path, a pointer to be filled with the current focus column
 * Expected: void  
 */
static void test_purpose_78()
{
    #define DEPTH 27
    #define INDICES {9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
    #define PATH "9:0:0" 
    #define COL_POSITION 0
    #define START_EDITING TRUE
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column;
    GtkTreeViewColumn *p_column;
    GtkTreePath *path;
    GtkTreePath *p_path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    gint indices[] = INDICES;
    guint i;
    guint j;
    char* path_str;
    char* p_path_str;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), 0);
    if (!PATH)
    {
        path = NULL;
    } else {
        path = gtk_tree_path_new_from_string (PATH);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    /* LSB bug 2700 - expand the view or p_path will return NULL */
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    gtk_tree_view_set_cursor (GTK_TREE_VIEW (tree_view), path, column, START_EDITING);
    gtk_tree_view_get_cursor (GTK_TREE_VIEW (tree_view), &p_path, &p_column);
    if (column != p_column)
    {
        TEST_FAIL ("gtk_tree_view_get_cursor () return column differ from set by gtk_tree_view_set_cursor ()", NULL);
    }
    
    path_str = gtk_tree_path_to_string (path);
    p_path_str = gtk_tree_path_to_string (p_path);
    if (strcmp (path_str, p_path_str))
    {
        TEST_FAIL ("gtk_tree_view_get_cursor () return path = %s instead of %s", p_path_str, path_str);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    #undef DEPTH
    #undef INDICES
    #undef PATH
    #undef COL_POSITION
    #undef START_EDITING
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_cursor
 * Params: GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *focus_column, gboolean start_editing
 * Returns: void
 * Assertion: Sets the current keyboard focus to be at path, and selects it
 * Params Values: newly created GtkTreeView*, "0", GtkTreeViewColumn from 0 position, FALSE
 * Expected: void
 * 
 * Name: gtk_tree_view_get_cursor
 * Params: GtkTreeView *tree_view, GtkTreePath **path, GtkTreeViewColumn **focus_column
 * Returns: void
 * Assertion: Fills in path and focus_column with the current path and focus column
 * Params Values: newly created GtkTreeView*, a pointer to be filled with the current cursor path, a pointer to be filled with the current focus column
 * Expected: void
 * 
 * Name: gtk_tree_view_get_cursor
 * Params: GtkTreeView *tree_view, GtkTreePath **path, GtkTreeViewColumn **focus_column
 * Returns: void
 * Assertion: If no column currently has focus, then* focus_column will be NULL
 * Params Values: newly created GtkTreeView*, a pointer to be filled with the current cursor path, a pointer to be filled with the current focus column
 * Expected: void  
 */
static void test_purpose_79()
{
    #define DEPTH 3
    #define INDICES {5, 7, 3} 
    #define PATH "0"
    #define COL_POSITION 0
    #define START_EDITING FALSE
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column;
    GtkTreeViewColumn *p_column;
    GtkTreePath *path;
    GtkTreePath *p_path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    gint indices[] = INDICES;
    guint i;
    guint j;
    char* path_str;
    char* p_path_str;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), 0);
    if (!PATH)
    {
        path = NULL;
    } else {
        path = gtk_tree_path_new_from_string (PATH);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    /* LSB bug 2700 - expand the view or p_path will return NULL */
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    gtk_tree_view_set_cursor (GTK_TREE_VIEW (tree_view), path, column, START_EDITING);
    gtk_tree_view_get_cursor (GTK_TREE_VIEW (tree_view), &p_path, &p_column);
    if (column != p_column)
    {
        TEST_FAIL ("gtk_tree_view_get_cursor () return column differ from set by gtk_tree_view_set_cursor ()", NULL);
    }
    
    path_str = gtk_tree_path_to_string (path);
    p_path_str = gtk_tree_path_to_string (p_path);
    if (strcmp (path_str, p_path_str))
    {
        TEST_FAIL ("gtk_tree_view_get_cursor () return path = %s instead of %s", p_path_str, path_str);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    #undef DEPTH
    #undef INDICES
    #undef PATH
    #undef COL_POSITION
    #undef START_EDITING
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_cursor
 * Params: GtkTreeView *tree_view, GtkTreePath **path, GtkTreeViewColumn **focus_column
 * Returns: void
 * Assertion: If the cursor isn't currently set, then* path will be NULL'
 * Params Values: newly created GtkTreeView*, a pointer to be filled with the current cursor path, a pointer to be filled with the current focus column
 * Expected: void  
 */
static void test_purpose_80()
{
    #define DEPTH 27
    #define INDICES {9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn *column;
    GtkTreePath *path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    gint indices[] = INDICES;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (1,G_TYPE_INT);
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    gtk_tree_view_get_cursor (GTK_TREE_VIEW (tree_view), &path, &column);
    if (path != NULL)
    {
        TEST_FAIL ("gtk_tree_view_get_cursor () return the path != NULL, but the cursor isn't currently set!", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view); 
    /******* -------------  *******/  
    
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    #undef DEPTH
    #undef INDICES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_expander_column
 * Params: GtkTreeView *tree_view, GtkTreeViewColumn *column
 * Returns: void
 * Assertion: Sets the column to draw the expander arrow at
 * Params Values: newly created GtkTreeView*, GtkTreeViewColumn from position 0
 * Expected: void
 * 
 * Name: gtk_tree_view_get_expander_column
 * Params: GtkTreeView *tree_view
 * Returns: GtkTreeViewColumn*
 * Assertion: Returns the column that is the current expander column
 * Params Values: newly created GtkTreeView*
 * Expected: GtkTreeViewColumn*  
 */
static void test_purpose_81()
{
    #define N_COLUMNS 1
    #define TYPES G_TYPE_UINT 
    #define POSITION 0
    
    GtkTreeStore *tree_store;
    GtkWidget *tree_view;
    GtkTreeViewColumn *column;
    GtkTreeViewColumn *expander_column;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (N_COLUMNS, TYPES);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
        
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), POSITION);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_set_expander_column (GTK_TREE_VIEW (tree_view), column);
    expander_column = gtk_tree_view_get_expander_column (GTK_TREE_VIEW (tree_view));
    if (column != expander_column)
    {
        TEST_FAIL ("gtk_tree_view_get_expander_column () return column that differ from set by gtk_tree_view_set_expander_column ()", NULL); 
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef N_COLUMNS
    #undef TYPES
    #undef POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_expander_column
 * Params: GtkTreeView *tree_view, GtkTreeViewColumn *column
 * Returns: void
 * Assertion: Sets the column to draw the expander arrow at
 * Params Values: newly created GtkTreeView*, GtkTreeViewColumn from position 5
 * Expected: void
 * 
 * Name: gtk_tree_view_get_expander_column
 * Params: GtkTreeView *tree_view
 * Returns: GtkTreeViewColumn*
 * Assertion: Returns the column that is the current expander column
 * Params Values: newly created GtkTreeView*
 * Expected: GtkTreeViewColumn*  
 */
static void test_purpose_82()
{
    #define N_COLUMNS 9
    #define TYPES G_TYPE_UINT, G_TYPE_INT, G_TYPE_UCHAR, G_TYPE_CHAR, G_TYPE_BOOLEAN, G_TYPE_POINTER, G_TYPE_FLOAT, G_TYPE_DOUBLE, G_TYPE_STRING
    #define POSITION 5
    
    GtkTreeStore *tree_store;
    GtkWidget *tree_view;
    GtkTreeViewColumn *column;
    GtkTreeViewColumn *expander_column;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (N_COLUMNS, TYPES);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
        
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), POSITION);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_set_expander_column (GTK_TREE_VIEW (tree_view), column);
    expander_column = gtk_tree_view_get_expander_column (GTK_TREE_VIEW (tree_view));
    if (column != expander_column)
    {
        TEST_FAIL ("gtk_tree_view_get_expander_column () return column that differ from set by gtk_tree_view_set_expander_column ()", NULL); 
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef N_COLUMNS
    #undef TYPES
    #undef POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_expander_column
 * Params: GtkTreeView *tree_view, GtkTreeViewColumn *column
 * Returns: void
 * Assertion: Sets the column to draw the expander arrow at
 * Params Values: newly created GtkTreeView*, GtkTreeViewColumn from position 11
 * Expected: void
 * 
 * Name: gtk_tree_view_get_expander_column
 * Params: GtkTreeView *tree_view
 * Returns: GtkTreeViewColumn*
 * Assertion: Returns the column that is the current expander column
 * Params Values: newly created GtkTreeView*
 * Expected: GtkTreeViewColumn*  
 */
static void test_purpose_83()
{
    #define N_COLUMNS 12
    #define TYPES G_TYPE_INT, G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_INT, G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_INT, G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_INT, G_TYPE_STRING, GDK_TYPE_PIXBUF
    #define POSITION 11
    
    GtkTreeStore *tree_store;
    GtkWidget *tree_view;
    GtkTreeViewColumn *column;
    GtkTreeViewColumn *expander_column;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (N_COLUMNS, TYPES);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
        return;
    }
        
    if (!GTK_IS_TREE_STORE (tree_store))
    {
        TEST_FAIL (get_wrong_type_msg(), NULL); 
        return;
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), POSITION);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_view_set_expander_column (GTK_TREE_VIEW (tree_view), column);
    expander_column = gtk_tree_view_get_expander_column (GTK_TREE_VIEW (tree_view));
    if (column != expander_column)
    {
        TEST_FAIL ("gtk_tree_view_get_expander_column () return column that differ from set by gtk_tree_view_set_expander_column ()", NULL); 
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef N_COLUMNS
    #undef TYPES
    #undef POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_search_column 
 * Params: GtkTreeView* tree_view, gint column
 * Returns: void
 * Assertion: Sets column as the column where the interactive search code should search in
 * Params Values: GtkTreeView* tree_view, 0
 * Expected: void
 * 
 * Name: gtk_tree_view_get_search_column 
 * Params: GtkTreeView* tree_view
 * Returns: gint
 * Assertion: Gets the column searched on by the interactive search code
 * Params Values: GtkTreeView* tree_view
 * Expected: 0  
 */
static void test_purpose_84()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_UINT))
    #define COLUMN 0
    
    GtkWidget *tree_view;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (MODEL);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    gtk_tree_view_set_search_column (GTK_TREE_VIEW (tree_view), COLUMN);
    
    if (COLUMN != gtk_tree_view_get_search_column (GTK_TREE_VIEW (tree_view)))
    {
        TEST_FAIL ("search column = %i, should be %i", gtk_tree_view_get_search_column (GTK_TREE_VIEW (tree_view)), COLUMN);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef MODEL
    #undef COLUMN
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_search_column 
 * Params: GtkTreeView* tree_view, gint column
 * Returns: void
 * Assertion: Sets column as the column where the interactive search code should search in
 * Params Values: GtkTreeView* tree_view, 2
 * Expected: void
 * 
 * Name: gtk_tree_view_get_search_column 
 * Params: GtkTreeView* tree_view
 * Returns: gint
 * Assertion: Gets the column searched on by the interactive search code
 * Params Values: GtkTreeView* tree_view
 * Expected: 2  
 */
static void test_purpose_85()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_store_new (3, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_STRING))
    #define COLUMN 2
    
    GtkWidget *tree_view;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (MODEL);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    gtk_tree_view_set_search_column (GTK_TREE_VIEW (tree_view), COLUMN);
    
    if (COLUMN != gtk_tree_view_get_search_column (GTK_TREE_VIEW (tree_view)))
    {
        TEST_FAIL ("search column = %i, should be %i", gtk_tree_view_get_search_column (GTK_TREE_VIEW (tree_view)), COLUMN);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef MODEL
    #undef COLUMN
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_search_column 
 * Params: GtkTreeView* tree_view, gint column
 * Returns: void
 * Assertion: Sets column as the column where the interactive search code should search in
 * Params Values: GtkTreeView* tree_view, 0
 * Expected: void
 * 
 * Name: gtk_tree_view_get_search_column 
 * Params: GtkTreeView* tree_view
 * Returns: gint
 * Assertion: Gets the column searched on by the interactive search code
 * Params Values: GtkTreeView* tree_view
 * Expected: 0  
 */
static void test_purpose_86()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT))))
    #define COLUMN 0
    
    GtkWidget *tree_view;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (MODEL);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    gtk_tree_view_set_search_column (GTK_TREE_VIEW (tree_view), COLUMN);
    
    if (COLUMN != gtk_tree_view_get_search_column (GTK_TREE_VIEW (tree_view)))
    {
        TEST_FAIL ("search column = %i, should be %i", gtk_tree_view_get_search_column (GTK_TREE_VIEW (tree_view)), COLUMN);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef MODEL
    #undef COLUMN
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_search_column 
 * Params: GtkTreeView* tree_view, gint column
 * Returns: void
 * Assertion: Sets column as the column where the interactive search code should search in
 * Params Values: GtkTreeView* tree_view, 1
 * Expected: void
 * 
 * Name: gtk_tree_view_get_search_column 
 * Params: GtkTreeView* tree_view
 * Returns: gint
 * Assertion: Gets the column searched on by the interactive search code
 * Params Values: GtkTreeView* tree_view
 * Expected: 1  
 */
static void test_purpose_87()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT))))
    #define COLUMN 1
    
    GtkWidget *tree_view;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (MODEL);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    gtk_tree_view_set_search_column (GTK_TREE_VIEW (tree_view), COLUMN);
    
    if (COLUMN != gtk_tree_view_get_search_column (GTK_TREE_VIEW (tree_view)))
    {
        TEST_FAIL ("search column = %i, should be %i", gtk_tree_view_get_search_column (GTK_TREE_VIEW (tree_view)), COLUMN);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef MODEL
    #undef COLUMN
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_columns 
 * Params: GtkTreeView* tree_view
 * Returns: GSList*
 * Assertion: Returns a GList of all the GtkTreeViewColumn s currently in tree_view
 * Params Values: GtkTreeView* tree_view
 * Expected: GSList*
 *   
 */
static void test_purpose_88()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_UINT))
    #define COUNT 1
    
    GtkWidget *tree_view;
    GtkTreeViewColumn *column;
    GtkCellRenderer *cell_renderer;
    GList* list;
    gint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (MODEL);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < COUNT; i++)
    {
        column = gtk_tree_view_column_new ();
        gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    list = gtk_tree_view_get_columns (GTK_TREE_VIEW (tree_view));
    
    if (COUNT != g_list_length (list))
    {
        TEST_FAIL ("count = %i, should be %i",  g_list_length (list), COUNT);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef MODEL
    #undef COUNT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_columns 
 * Params: GtkTreeView* tree_view
 * Returns: GSList*
 * Assertion: Returns a GList of all the GtkTreeViewColumn s currently in tree_view
 * Params Values: GtkTreeView* tree_view
 * Expected: GSList*
 *   
 */
static void test_purpose_89()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_store_new (3, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_STRING))
    #define COUNT 3
    
    GtkWidget *tree_view;
    GtkTreeViewColumn *column;
    GtkCellRenderer *cell_renderer;
    GList* list;
    gint i;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (MODEL);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < COUNT; i++)
    {
        column = gtk_tree_view_column_new ();
        gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    list = gtk_tree_view_get_columns (GTK_TREE_VIEW (tree_view));
    
    if (COUNT != g_list_length (list))
    {
        TEST_FAIL ("count = %i, should be %i",  g_list_length (list), COUNT);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef MODEL
    #undef COUNT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_tree_to_widget_coords
 * Params: GtkTreeView* tree_view, gint tx, gint ty, gint* wx, gint* wy
 * Returns: void
 * Assertion: Converts tree coordinates (coordinates in full scrollable area of the tree) to widget coordinates
 * Params Values: GtkTreeView* tree_view, gint tx, gint ty, gint* wx, gint* wy
 * Expected: void
 * 
 * Name: gtk_tree_view_widget_to_tree_coords
 * Params: GtkTreeView* tree_view, gint wx, gint wy, gint* tx, gint* ty
 * Returns: void
 * Assertion: Converts widget coordinates to coordinates for the tree window (the full scrollable area of the tree)
 * Params Values: GtkTreeView* tree_view, gint wx, gint wy, gint* tx, gint* ty
 * Expected: void  
 */
static void test_purpose_90()
{
    #define WIDTH 300
    #define HEIGHT 200
    #define TX 0
    #define TY 0
    
    GtkWidget *toplevel;
    GtkWidget *window;
    GtkWidget *tree_view;
    gint tx, ty, wx, wy;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (
        gtk_tree_store_new (5, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT)));
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    toplevel = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_add (GTK_CONTAINER (toplevel), window);
    gtk_window_set_default_size (GTK_WINDOW (toplevel), WIDTH, HEIGHT);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (toplevel);
    /******* -------------  *******/  
    gtk_tree_view_tree_to_widget_coords (GTK_TREE_VIEW (tree_view), TX, TY, &wx, &wy);
    gtk_tree_view_widget_to_tree_coords (GTK_TREE_VIEW (tree_view), wx, wy, &tx, &ty);
    
    if ((TX != tx) || (TY != ty))
    {
        TEST_FAIL ("coordinates were converted incorrectly (%i, %i), should be (%i, %i)", tx, ty, TX, TY);
    }
    
    gtk_widget_destroy (toplevel);
    
    #undef WIDTH
    #undef HEIGHT
    #undef TX
    #undef TY
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_tree_to_widget_coords
 * Params: GtkTreeView* tree_view, gint tx, gint ty, gint* wx, gint* wy
 * Returns: void
 * Assertion: Converts tree coordinates (coordinates in full scrollable area of the tree) to widget coordinates
 * Params Values: GtkTreeView* tree_view, gint tx, gint ty, gint* wx, gint* wy
 * Expected: void
 * 
 * Name: gtk_tree_view_widget_to_tree_coords
 * Params: GtkTreeView* tree_view, gint wx, gint wy, gint* tx, gint* ty
 * Returns: void
 * Assertion: Converts widget coordinates to coordinates for the tree window (the full scrollable area of the tree)
 * Params Values: GtkTreeView* tree_view, gint wx, gint wy, gint* tx, gint* ty
 * Expected: void  
 */
static void test_purpose_91()
{
    #define WIDTH 300
    #define HEIGHT 200
    #define TX 300
    #define TY 200
    
    GtkWidget *toplevel;
    GtkWidget *window;
    GtkWidget *tree_view;
    gint tx, ty, wx, wy;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (
        gtk_tree_store_new (5, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT)));
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    toplevel = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_add (GTK_CONTAINER (toplevel), window);
    gtk_window_set_default_size (GTK_WINDOW (toplevel), WIDTH, HEIGHT);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (toplevel);
    /******* -------------  *******/  
    gtk_tree_view_tree_to_widget_coords (GTK_TREE_VIEW (tree_view), TX, TY, &wx, &wy);
    gtk_tree_view_widget_to_tree_coords (GTK_TREE_VIEW (tree_view), wx, wy, &tx, &ty);
    
    if ((TX != tx) || (TY != ty))
    {
        TEST_FAIL ("coordinates were converted incorrectly (%i, %i), should be (%i, %i)", tx, ty, TX, TY);
    }
    
    gtk_widget_destroy (toplevel);
    
    #undef WIDTH
    #undef HEIGHT
    #undef TX
    #undef TY
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_tree_to_widget_coords
 * Params: GtkTreeView* tree_view, gint tx, gint ty, gint* wx, gint* wy
 * Returns: void
 * Assertion: Converts tree coordinates (coordinates in full scrollable area of the tree) to widget coordinates
 * Params Values: GtkTreeView* tree_view, gint tx, gint ty, gint* wx, gint* wy
 * Expected: void
 * 
 * Name: gtk_tree_view_widget_to_tree_coords
 * Params: GtkTreeView* tree_view, gint wx, gint wy, gint* tx, gint* ty
 * Returns: void
 * Assertion: Converts widget coordinates to coordinates for the tree window (the full scrollable area of the tree)
 * Params Values: GtkTreeView* tree_view, gint wx, gint wy, gint* tx, gint* ty
 * Expected: void  
 */
static void test_purpose_92()
{
    #define WIDTH 300
    #define HEIGHT 200
    #define TX 500
    #define TY 300
    
    GtkWidget *toplevel;
    GtkWidget *window;
    GtkWidget *tree_view;
    gint tx, ty, wx, wy;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (
        gtk_tree_store_new (5, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT)));
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    toplevel = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_add (GTK_CONTAINER (toplevel), window);
    gtk_window_set_default_size (GTK_WINDOW (toplevel), WIDTH, HEIGHT);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (toplevel);
    /******* -------------  *******/  
    gtk_tree_view_tree_to_widget_coords (GTK_TREE_VIEW (tree_view), TX, TY, &wx, &wy);
    gtk_tree_view_widget_to_tree_coords (GTK_TREE_VIEW (tree_view), wx, wy, &tx, &ty);
    
    if ((TX != tx) || (TY != ty))
    {
        TEST_FAIL ("coordinates were converted incorrectly (%i, %i), should be (%i, %i)", tx, ty, TX, TY);
    }
    
    gtk_widget_destroy (toplevel);
    
    #undef WIDTH
    #undef HEIGHT
    #undef TX
    #undef TY
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_tree_to_widget_coords
 * Params: GtkTreeView* tree_view, gint tx, gint ty, gint* wx, gint* wy
 * Returns: void
 * Assertion: Converts tree coordinates (coordinates in full scrollable area of the tree) to widget coordinates
 * Params Values: GtkTreeView* tree_view, gint tx, gint ty, gint* wx, gint* wy
 * Expected: void
 * 
 * Name: gtk_tree_view_widget_to_tree_coords
 * Params: GtkTreeView* tree_view, gint wx, gint wy, gint* tx, gint* ty
 * Returns: void
 * Assertion: Converts widget coordinates to coordinates for the tree window (the full scrollable area of the tree)
 * Params Values: GtkTreeView* tree_view, gint wx, gint wy, gint* tx, gint* ty
 * Expected: void  
 */
static void test_purpose_93()
{
    #define WIDTH 1000
    #define HEIGHT 1000
    #define TX 1000
    #define TY 1000
    
    GtkWidget *toplevel;
    GtkWidget *window;
    GtkWidget *tree_view;
    gint tx, ty, wx, wy;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (
        gtk_tree_store_new (5, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT)));
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    toplevel = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_add (GTK_CONTAINER (toplevel), window);
    gtk_window_set_default_size (GTK_WINDOW (toplevel), WIDTH, HEIGHT);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (toplevel);
    /******* -------------  *******/  
    gtk_tree_view_tree_to_widget_coords (GTK_TREE_VIEW (tree_view), TX, TY, &wx, &wy);
    gtk_tree_view_widget_to_tree_coords (GTK_TREE_VIEW (tree_view), wx, wy, &tx, &ty);
    
    if ((TX != tx) || (TY != ty))
    {
        TEST_FAIL ("coordinates were converted incorrectly (%i, %i), should be (%i, %i)", tx, ty, TX, TY);
    }
    
    gtk_widget_destroy (toplevel);
    
    #undef WIDTH
    #undef HEIGHT
    #undef TX
    #undef TY
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_search_equal_func
 * Params: GtkTreeView* tree_view, GtkTreeViewSearchEqualFunc func, gpointer search_user_data, GtkDestroyNotify search_destroy
 * Returns: void
 * Assertion: Sets the compare function for the interactive search capabilities
 * Params Values: GtkTreeView* tree_view, global_search_equal_func, gpointer search_user_data, GtkDestroyNotify search_destroy
 * Expected: void
 * 
 * Name: gtk_tree_view_get_search_equal_func
 * Params: GtkTreeView* tree_view
 * Returns: GtkTreeViewSearchEqualFunc
 * Assertion: Returns the compare function currently in use
 * Params Values: GtkTreeView* tree_view
 * Expected: global_search_equal_func  
 */
static void test_purpose_94()
{
    #define FUNC global_search_equal_func
    
    GtkWidget *tree_view;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (
        gtk_tree_store_new (5, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT)));
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (tree_view), FUNC, NULL, NULL);
    
    if (FUNC != gtk_tree_view_get_search_equal_func (GTK_TREE_VIEW (tree_view)))
    {
        TEST_FAIL ("'search equal function' is wrong", NULL);
    }
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef FUNC
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_insert_column_with_attributes
 * Params: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Returns: gint
 * Assertion: Creates a new GtkTreeViewColumn and inserts it into the tree_view at position
 * Params Values: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Expected: gint
 * 
 * Name: gtk_tree_view_insert_column_with_attributes
 * Params: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Returns: gint
 * Assertion: If position is -1, then the newly created column is inserted at the end
 * Params Values: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Expected: gint
 * 
 * Name: gtk_tree_view_insert_column_with_attributes
 * Params: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Returns: gint
 * Assertion: The column is initialized with the attributes given
 * Params Values: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Expected: gint  
 */
static void test_purpose_95()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_STRING))
    #define COUNT 1
    #define POS 1
    
    GtkWidget *tree_view;
    GtkTreeViewColumn *column;
    GtkCellRenderer *cell_renderer;
    GList* list;
    gint i;
    gint count;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (MODEL);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < COUNT; i++)
    {
        column = gtk_tree_view_column_new ();
        gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    }
    
    count = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tree_view), POS, "Title", gtk_cell_renderer_text_new (), NULL);
    
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    if (count != COUNT + 1)
    {
        TEST_FAIL ("count = %i, should be %i",  count, COUNT);
        return;
    }
    
    list = gtk_tree_view_get_columns (GTK_TREE_VIEW (tree_view));
    
    if (POS != -1)
    {
        column = (GtkTreeViewColumn*)g_list_nth_data (list, POS);
    
    } else {
    
        column = (GtkTreeViewColumn*)g_list_nth_data (list, COUNT);
    }
    
    if (strcmp (gtk_tree_view_column_get_title (column), "Title"))
    {
        TEST_FAIL ("Title = '%s'", gtk_tree_view_column_get_title (column));
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef MODEL
    #undef COUNT
    #undef POS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_insert_column_with_attributes
 * Params: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Returns: gint
 * Assertion: Creates a new GtkTreeViewColumn and inserts it into the tree_view at position
 * Params Values: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Expected: gint
 * 
 * Name: gtk_tree_view_insert_column_with_attributes
 * Params: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Returns: gint
 * Assertion: If position is -1, then the newly created column is inserted at the end
 * Params Values: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Expected: gint
 * 
 * Name: gtk_tree_view_insert_column_with_attributes
 * Params: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Returns: gint
 * Assertion: The column is initialized with the attributes given
 * Params Values: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Expected: gint  
 */
static void test_purpose_96()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_STRING))
    #define COUNT 1
    #define POS -1
    
    GtkWidget *tree_view;
    GtkTreeViewColumn *column;
    GtkCellRenderer *cell_renderer;
    GList* list;
    gint i;
    gint count;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (MODEL);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < COUNT; i++)
    {
        column = gtk_tree_view_column_new ();
        gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    }
    
    count = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tree_view), POS, "Title", gtk_cell_renderer_text_new (), NULL);
    
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    if (count != COUNT + 1)
    {
        TEST_FAIL ("count = %i, should be %i",  count, COUNT);
        return;
    }
    
    list = gtk_tree_view_get_columns (GTK_TREE_VIEW (tree_view));
    
    if (POS != -1)
    {
        column = (GtkTreeViewColumn*)g_list_nth_data (list, POS);
    
    } else {
    
        column = (GtkTreeViewColumn*)g_list_nth_data (list, COUNT);
    }
    
    if (strcmp (gtk_tree_view_column_get_title (column), "Title"))
    {
        TEST_FAIL ("Title = '%s'", gtk_tree_view_column_get_title (column));
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef MODEL
    #undef COUNT
    #undef POS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_insert_column_with_attributes
 * Params: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Returns: gint
 * Assertion: Creates a new GtkTreeViewColumn and inserts it into the tree_view at position
 * Params Values: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Expected: gint
 * 
 * Name: gtk_tree_view_insert_column_with_attributes
 * Params: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Returns: gint
 * Assertion: If position is -1, then the newly created column is inserted at the end
 * Params Values: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Expected: gint
 * 
 * Name: gtk_tree_view_insert_column_with_attributes
 * Params: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Returns: gint
 * Assertion: The column is initialized with the attributes given
 * Params Values: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Expected: gint  
 */
static void test_purpose_97()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING))
    #define COUNT 3
    #define POS 0
    
    GtkWidget *tree_view;
    GtkTreeViewColumn *column;
    GtkCellRenderer *cell_renderer;
    GList* list;
    gint i;
    gint count;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (MODEL);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < COUNT; i++)
    {
        column = gtk_tree_view_column_new ();
        gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    }
    
    count = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tree_view), POS, "Title", gtk_cell_renderer_text_new (), NULL);
    
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    if (count != COUNT + 1)
    {
        TEST_FAIL ("count = %i, should be %i",  count, COUNT);
        return;
    }
    
    list = gtk_tree_view_get_columns (GTK_TREE_VIEW (tree_view));
    
    if (POS != -1)
    {
        column = (GtkTreeViewColumn*)g_list_nth_data (list, POS);
    
    } else {
    
        column = (GtkTreeViewColumn*)g_list_nth_data (list, COUNT);
    }
    
    if (strcmp (gtk_tree_view_column_get_title (column), "Title"))
    {
        TEST_FAIL ("Title = '%s'", gtk_tree_view_column_get_title (column));
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef MODEL
    #undef COUNT
    #undef POS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_insert_column_with_attributes
 * Params: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Returns: gint
 * Assertion: Creates a new GtkTreeViewColumn and inserts it into the tree_view at position
 * Params Values: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Expected: gint
 * 
 * Name: gtk_tree_view_insert_column_with_attributes
 * Params: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Returns: gint
 * Assertion: If position is -1, then the newly created column is inserted at the end
 * Params Values: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Expected: gint
 * 
 * Name: gtk_tree_view_insert_column_with_attributes
 * Params: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Returns: gint
 * Assertion: The column is initialized with the attributes given
 * Params Values: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, ...
 * Expected: gint  
 */
static void test_purpose_98()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING))
    #define COUNT 3
    #define POS 2
    
    GtkWidget *tree_view;
    GtkTreeViewColumn *column;
    GtkCellRenderer *cell_renderer;
    GList* list;
    gint i;
    gint count;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (MODEL);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < COUNT; i++)
    {
        column = gtk_tree_view_column_new ();
        gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    }
    
    count = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tree_view), POS, "Title", gtk_cell_renderer_text_new (), NULL);
    
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    if (count != COUNT + 1)
    {
        TEST_FAIL ("count = %i, should be %i",  count, COUNT);
        return;
    }
    
    list = gtk_tree_view_get_columns (GTK_TREE_VIEW (tree_view));
    
    if (POS != -1)
    {
        column = (GtkTreeViewColumn*)g_list_nth_data (list, POS);
    
    } else {
    
        column = (GtkTreeViewColumn*)g_list_nth_data (list, COUNT);
    }
    
    if (strcmp (gtk_tree_view_column_get_title (column), "Title"))
    {
        TEST_FAIL ("Title = '%s'", gtk_tree_view_column_get_title (column));
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef MODEL
    #undef COUNT
    #undef POS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_insert_column_with_data_func
 * Params: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, GtkTreeCellDataFunc func, gpointer data, GDestroyNotify dnotify
 * Returns: gint
 * Assertion: Convenience function that inserts a new column into the GtkTreeView with the given cell renderer and a GtkCellDataFunc to set cell renderer attributes (normally using data from the model)
 * Params Values: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, GtkTreeCellDataFunc func, gpointer data, GDestroyNotify dnotify
 * Expected: gint  
 */
static void test_purpose_99()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_STRING))
    #define COUNT 1
    #define POS 1
    
    GtkWidget *tree_view;
    GtkTreeViewColumn *column;
    GtkCellRenderer *cell_renderer;
    GList* list;
    gint i;
    gint count;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (MODEL);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < COUNT; i++)
    {
        column = gtk_tree_view_column_new ();
        gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    }
    
    count = gtk_tree_view_insert_column_with_data_func (GTK_TREE_VIEW (tree_view), POS, "Title", gtk_cell_renderer_text_new (), global_data_func, NULL, NULL);
    
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    if (count != COUNT + 1)
    {
        TEST_FAIL ("count = %i, should be %i",  count, COUNT);
        return;
    }
    
    list = gtk_tree_view_get_columns (GTK_TREE_VIEW (tree_view));
    
    if (POS != -1)
    {
        column = (GtkTreeViewColumn*)g_list_nth_data (list, POS);
    
    } else {
    
        column = (GtkTreeViewColumn*)g_list_nth_data (list, COUNT);
    }
    
    if (strcmp (gtk_tree_view_column_get_title (column), "Title"))
    {
        TEST_FAIL ("Title = '%s'", gtk_tree_view_column_get_title (column));
    }
    
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef MODEL
    #undef COUNT
    #undef POS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_insert_column_with_data_func
 * Params: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, GtkTreeCellDataFunc func, gpointer data, GDestroyNotify dnotify
 * Returns: gint
 * Assertion: Convenience function that inserts a new column into the GtkTreeView with the given cell renderer and a GtkCellDataFunc to set cell renderer attributes (normally using data from the model)
 * Params Values: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, GtkTreeCellDataFunc func, gpointer data, GDestroyNotify dnotify
 * Expected: gint  
 */
static void test_purpose_100()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_store_new (1, G_TYPE_STRING))
    #define COUNT 1
    #define POS -1
    
    GtkWidget *tree_view;
    GtkTreeViewColumn *column;
    GtkCellRenderer *cell_renderer;
    GList* list;
    gint i;
    gint count;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (MODEL);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < COUNT; i++)
    {
        column = gtk_tree_view_column_new ();
        gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    }
    
    count = gtk_tree_view_insert_column_with_data_func (GTK_TREE_VIEW (tree_view), POS, "Title", gtk_cell_renderer_text_new (), global_data_func, NULL, NULL);
    
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    if (count != COUNT + 1)
    {
        TEST_FAIL ("count = %i, should be %i",  count, COUNT);
        return;
    }
    
    list = gtk_tree_view_get_columns (GTK_TREE_VIEW (tree_view));
    
    if (POS != -1)
    {
        column = (GtkTreeViewColumn*)g_list_nth_data (list, POS);
    
    } else {
    
        column = (GtkTreeViewColumn*)g_list_nth_data (list, COUNT);
    }
    
    if (strcmp (gtk_tree_view_column_get_title (column), "Title"))
    {
        TEST_FAIL ("Title = '%s'", gtk_tree_view_column_get_title (column));
    }
    
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef MODEL
    #undef COUNT
    #undef POS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_insert_column_with_data_func
 * Params: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, GtkTreeCellDataFunc func, gpointer data, GDestroyNotify dnotify
 * Returns: gint
 * Assertion: Convenience function that inserts a new column into the GtkTreeView with the given cell renderer and a GtkCellDataFunc to set cell renderer attributes (normally using data from the model)
 * Params Values: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, GtkTreeCellDataFunc func, gpointer data, GDestroyNotify dnotify
 * Expected: gint  
 */
static void test_purpose_101()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING))
    #define COUNT 3
    #define POS 0
    
    GtkWidget *tree_view;
    GtkTreeViewColumn *column;
    GtkCellRenderer *cell_renderer;
    GList* list;
    gint i;
    gint count;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (MODEL);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < COUNT; i++)
    {
        column = gtk_tree_view_column_new ();
        gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    }
    
    count = gtk_tree_view_insert_column_with_data_func (GTK_TREE_VIEW (tree_view), POS, "Title", gtk_cell_renderer_text_new (), global_data_func, NULL, NULL);
    
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    if (count != COUNT + 1)
    {
        TEST_FAIL ("count = %i, should be %i",  count, COUNT);
        return;
    }
    
    list = gtk_tree_view_get_columns (GTK_TREE_VIEW (tree_view));
    
    if (POS != -1)
    {
        column = (GtkTreeViewColumn*)g_list_nth_data (list, POS);
    
    } else {
    
        column = (GtkTreeViewColumn*)g_list_nth_data (list, COUNT);
    }
    
    if (strcmp (gtk_tree_view_column_get_title (column), "Title"))
    {
        TEST_FAIL ("Title = '%s'", gtk_tree_view_column_get_title (column));
    }
    
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef MODEL
    #undef COUNT
    #undef POS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_insert_column_with_data_func
 * Params: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, GtkTreeCellDataFunc func, gpointer data, GDestroyNotify dnotify
 * Returns: gint
 * Assertion: Convenience function that inserts a new column into the GtkTreeView with the given cell renderer and a GtkCellDataFunc to set cell renderer attributes (normally using data from the model)
 * Params Values: GtkTreeView* tree_view, gint position, const gchar* title, GtkCellRenderer* cell, GtkTreeCellDataFunc func, gpointer data, GDestroyNotify dnotify
 * Expected: gint  
 */
static void test_purpose_102()
{
    #define MODEL GTK_TREE_MODEL (gtk_tree_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING))
    #define COUNT 3
    #define POS 2
    
    GtkWidget *tree_view;
    GtkTreeViewColumn *column;
    GtkCellRenderer *cell_renderer;
    GList* list;
    gint i;
    gint count;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_view = gtk_tree_view_new_with_model (MODEL);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    for (i = 0; i < COUNT; i++)
    {
        column = gtk_tree_view_column_new ();
        gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    }
    
    count = gtk_tree_view_insert_column_with_data_func (GTK_TREE_VIEW (tree_view), POS, "Title", gtk_cell_renderer_text_new (), global_data_func, NULL, NULL);
    
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    if (count != COUNT + 1)
    {
        TEST_FAIL ("count = %i, should be %i",  count, COUNT);
        return;
    }
    
    list = gtk_tree_view_get_columns (GTK_TREE_VIEW (tree_view));
    
    if (POS != -1)
    {
        column = (GtkTreeViewColumn*)g_list_nth_data (list, POS);
    
    } else {
    
        column = (GtkTreeViewColumn*)g_list_nth_data (list, COUNT);
    }
    
    if (strcmp (gtk_tree_view_column_get_title (column), "Title"))
    {
        TEST_FAIL ("Title = '%s'", gtk_tree_view_column_get_title (column));
    }
    
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef MODEL
    #undef COUNT
    #undef POS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_destroy_count_func
 * Params: GtkTreeView* tree_view, GtkTreeDestroyCountFunc func, gpointer data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: It is meant for private use by ATK for determining the number of visible children that are removed when the user collapses a row, or a row is deleted
 * Params Values: GtkTreeView* tree_view, GtkTreeDestroyCountFunc func, gpointer data, GtkDestroyNotify destroy
 * Expected: void  
 */
static void test_purpose_103()
{
    #define DEPTH 5
    #define INDICES {2, 3, 5, 2, 1}
    #define FUNC global_destroy_count_func
    
    GtkWidget *window;
    GtkWidget *scroll;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    gint indices[] = INDICES;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    gtk_tree_view_set_destroy_count_func (GTK_TREE_VIEW (tree_view), FUNC, NULL, NULL);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_model_row_deleted (GTK_TREE_MODEL (tree_store), gtk_tree_path_new_first ());
    
    if (!global_count && FUNC)
    {
        TEST_FAIL ("'destroy count function' wasn't called %i", global_count);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef FUNC
    #undef DEPTH
    #undef INDICES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_destroy_count_func
 * Params: GtkTreeView* tree_view, GtkTreeDestroyCountFunc func, gpointer data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: It is meant for private use by ATK for determining the number of visible children that are removed when the user collapses a row, or a row is deleted
 * Params Values: GtkTreeView* tree_view, GtkTreeDestroyCountFunc func, gpointer data, GtkDestroyNotify destroy
 * Expected: void  
 */
static void test_purpose_104()
{
    #define DEPTH 5
    #define INDICES {7, 3, 5, 2, 1}
    #define FUNC NULL
    
    GtkWidget *window;
    GtkWidget *scroll;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    gint indices[] = INDICES;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    gtk_tree_view_set_destroy_count_func (GTK_TREE_VIEW (tree_view), FUNC, NULL, NULL);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    gtk_tree_model_row_deleted (GTK_TREE_MODEL (tree_store), gtk_tree_path_new_first ());
    
    if (!global_count && FUNC)
    {
        TEST_FAIL ("'destroy count function' wasn't called %i", global_count);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef FUNC
    #undef DEPTH
    #undef INDICES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_column_drag_function
 * Params: GtkTreeView* tree_view, GtkTreeViewColumnDropFunc func, gpointer data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: Sets a user function for determining where a column may be dropped when dragged
 * Params Values: GtkTreeView* tree_view, GtkTreeViewColumnDropFunc func, gpointer data, GtkDestroyNotify destroy
 * Expected: void
 * 
 * Name: gtk_tree_view_set_column_drag_function
 * Params: GtkTreeView* tree_view, GtkTreeViewColumnDropFunc func, gpointer data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: If func is set to be NULL, then tree_view reverts to the default behavior of allowing all columns to be dropped everywhere
 * Params Values: GtkTreeView* tree_view, GtkTreeViewColumnDropFunc func, gpointer data, GtkDestroyNotify destroy
 * Expected: void  
 */
static void test_purpose_105()
{
    #define DEPTH 5
    #define INDICES {7, 3, 5, 2, 1}
    #define FUNC global_column_drag_func
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    gint indices[] = INDICES;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    gtk_tree_view_set_column_drag_function (GTK_TREE_VIEW (tree_view), FUNC, NULL, NULL);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    /*
    gtk_tree_view_set_drag_dest_row (GTK_TREE_VIEW (tree_view), gtk_tree_path_new_first (), GTK_TREE_VIEW_DROP_INTO_OR_AFTER); 
    */
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef FUNC
    #undef DEPTH
    #undef INDICES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_set_column_drag_function
 * Params: GtkTreeView* tree_view, GtkTreeViewColumnDropFunc func, gpointer data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: Sets a user function for determining where a column may be dropped when dragged
 * Params Values: GtkTreeView* tree_view, GtkTreeViewColumnDropFunc func, gpointer data, GtkDestroyNotify destroy
 * Expected: void
 * 
 * Name: gtk_tree_view_set_column_drag_function
 * Params: GtkTreeView* tree_view, GtkTreeViewColumnDropFunc func, gpointer data, GtkDestroyNotify destroy
 * Returns: void
 * Assertion: If func is set to be NULL, then tree_view reverts to the default behavior of allowing all columns to be dropped everywhere
 * Params Values: GtkTreeView* tree_view, GtkTreeViewColumnDropFunc func, gpointer data, GtkDestroyNotify destroy
 * Expected: void  
 */
static void test_purpose_106()
{
    #define DEPTH 5
    #define INDICES {7, 3, 5, 2, 1}
    #define FUNC NULL
    
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    gint indices[] = INDICES;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    gtk_tree_view_set_column_drag_function (GTK_TREE_VIEW (tree_view), FUNC, NULL, NULL);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (tree_view);
    /******* -------------  *******/  
    
    /*
    gtk_tree_view_set_drag_dest_row (GTK_TREE_VIEW (tree_view), gtk_tree_path_new_first (), GTK_TREE_VIEW_DROP_INTO_OR_AFTER); 
    */
    
    gtk_widget_destroy (gtk_widget_get_toplevel (tree_view));
    
    #undef FUNC
    #undef DEPTH
    #undef INDICES
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_visible_rect
 * Params: GtkTreeView* tree_view, GdkRectangle* visible_rect
 * Returns: void
 * Assertion: Fills visible_rect with the currently-visible region of the buffer, in tree coordinates
 * Params Values: GtkTreeView* tree_view, GdkRectangle* visible_rect
 * Expected: void  
 */
static void test_purpose_107()
{
    #define DEPTH 5
    #define INDICES {7, 3, 5, 2, 1}
    #define WIDTH 1
    #define HEIGHT 1
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    gint indices[] = INDICES;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    GdkRectangle geom;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), WIDTH, HEIGHT);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &rect); 
    gdk_window_get_geometry (tree_view->window, &geom.x, &geom.y, &geom.width, &geom.height, NULL);
    
    if (!gdk_region_equal (gdk_region_rectangle (&rect), gdk_region_rectangle (&geom)))
    {
        TEST_FAIL ("\nrect = (%i, %i, %i, %i)\n", rect.x, rect. y, rect.width, rect.height);
        TEST_FAIL ("\ngeom = (%i, %i, %i, %i)\n", geom.x, geom. y, geom.width, geom.height);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef DEPTH
    #undef INDICES
    #undef WIDTH
    #undef HEIGHT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_visible_rect
 * Params: GtkTreeView* tree_view, GdkRectangle* visible_rect
 * Returns: void
 * Assertion: Fills visible_rect with the currently-visible region of the buffer, in tree coordinates
 * Params Values: GtkTreeView* tree_view, GdkRectangle* visible_rect
 * Expected: void  
 */
static void test_purpose_108()
{
    #define DEPTH 5
    #define INDICES {7, 3, 5, 2, 1}
    #define WIDTH 20
    #define HEIGHT 20
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    gint indices[] = INDICES;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    GdkRectangle geom;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), WIDTH, HEIGHT);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &rect); 
    gdk_window_get_geometry (tree_view->window, &geom.x, &geom.y, &geom.width, &geom.height, NULL);
    
    if (!gdk_region_equal (gdk_region_rectangle (&rect), gdk_region_rectangle (&geom)))
    {
        TEST_FAIL ("\nrect = (%i, %i, %i, %i)\n", rect.x, rect. y, rect.width, rect.height);
        TEST_FAIL ("\ngeom = (%i, %i, %i, %i)\n", geom.x, geom. y, geom.width, geom.height);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef DEPTH
    #undef INDICES
    #undef WIDTH
    #undef HEIGHT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_visible_rect
 * Params: GtkTreeView* tree_view, GdkRectangle* visible_rect
 * Returns: void
 * Assertion: Fills visible_rect with the currently-visible region of the buffer, in tree coordinates
 * Params Values: GtkTreeView* tree_view, GdkRectangle* visible_rect
 * Expected: void  
 */
static void test_purpose_109()
{
    #define DEPTH 5
    #define INDICES {7, 3, 5, 2, 1}
    #define WIDTH 40
    #define HEIGHT 40
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    gint indices[] = INDICES;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    GdkRectangle geom;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), WIDTH, HEIGHT);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &rect); 
    gdk_window_get_geometry (tree_view->window, &geom.x, &geom.y, &geom.width, &geom.height, NULL);
    
    if (!gdk_region_equal (gdk_region_rectangle (&rect), gdk_region_rectangle (&geom)))
    {
        TEST_FAIL ("\nrect = (%i, %i, %i, %i)\n", rect.x, rect. y, rect.width, rect.height);
        TEST_FAIL ("\ngeom = (%i, %i, %i, %i)\n", geom.x, geom. y, geom.width, geom.height);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef DEPTH
    #undef INDICES
    #undef WIDTH
    #undef HEIGHT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_visible_rect
 * Params: GtkTreeView* tree_view, GdkRectangle* visible_rect
 * Returns: void
 * Assertion: Fills visible_rect with the currently-visible region of the buffer, in tree coordinates
 * Params Values: GtkTreeView* tree_view, GdkRectangle* visible_rect
 * Expected: void  
 */
static void test_purpose_110()
{
    #define DEPTH 5
    #define INDICES {7, 3, 5, 2, 1}
    #define WIDTH 200
    #define HEIGHT 100
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    gint indices[] = INDICES;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    GdkRectangle geom;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), WIDTH, HEIGHT);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &rect); 
    gdk_window_get_geometry (tree_view->window, &geom.x, &geom.y, &geom.width, &geom.height, NULL);
    
    if (!gdk_region_equal (gdk_region_rectangle (&rect), gdk_region_rectangle (&geom)))
    {
        TEST_FAIL ("\nrect = (%i, %i, %i, %i)\n", rect.x, rect. y, rect.width, rect.height);
        TEST_FAIL ("\ngeom = (%i, %i, %i, %i)\n", geom.x, geom. y, geom.width, geom.height);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef DEPTH
    #undef INDICES
    #undef WIDTH
    #undef HEIGHT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_visible_rect
 * Params: GtkTreeView* tree_view, GdkRectangle* visible_rect
 * Returns: void
 * Assertion: Fills visible_rect with the currently-visible region of the buffer, in tree coordinates
 * Params Values: GtkTreeView* tree_view, GdkRectangle* visible_rect
 * Expected: void  
 */
static void test_purpose_111()
{
    #define DEPTH 5
    #define INDICES {7, 3, 5, 2, 1}
    #define WIDTH 2000
    #define HEIGHT 1000
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    gint indices[] = INDICES;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GdkRectangle rect;
    GdkRectangle geom;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < DEPTH; i++)
    {
        for (j = 0; j <= indices[i]; j++)
        {
            if (i == 0)
            {
                gtk_tree_store_append (tree_store, &iter, NULL);
            } else {
                gtk_tree_store_append (tree_store, &child_iter, &iter);            
            }
        }
        if (i != 0)
        {
            iter = child_iter;
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), WIDTH, HEIGHT);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &rect); 
    gdk_window_get_geometry (tree_view->window, &geom.x, &geom.y, &geom.width, &geom.height, NULL);
    
    if (!gdk_region_equal (gdk_region_rectangle (&rect), gdk_region_rectangle (&geom)))
    {
        TEST_FAIL ("\nrect = (%i, %i, %i, %i)\n", rect.x, rect. y, rect.width, rect.height);
        TEST_FAIL ("\ngeom = (%i, %i, %i, %i)\n", geom.x, geom. y, geom.width, geom.height);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef DEPTH
    #undef INDICES
    #undef WIDTH
    #undef HEIGHT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_row_activated
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column
 * Returns: void
 * Assertion: Activates the cell determined by path and column
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column
 * Expected: void  
 */
static void test_purpose_112()
{
    #define PATH gtk_tree_path_new_from_string ("0")
    #define COLUMN column0
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreePath* path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (),
                                                       "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (),
                                                       "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    
    g_signal_connect (G_OBJECT (tree_view), "row-activated", (GCallback)global_row_activated, COLUMN);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    path = PATH;
    gtk_tree_view_row_activated (GTK_TREE_VIEW (tree_view), path, COLUMN);
    
    if (!global_count)
    {
        TEST_FAIL ("'row-activated' signal wasn't catched", NULL);
    }
    
    gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef COLUMN
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_row_activated
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column
 * Returns: void
 * Assertion: Activates the cell determined by path and column
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column
 * Expected: void  
 */
static void test_purpose_113()
{
    #define PATH gtk_tree_path_new_from_string ("1")
    #define COLUMN column0
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreePath* path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (),
                                                       "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (),
                                                       "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    
    g_signal_connect (G_OBJECT (tree_view), "row-activated", (GCallback)global_row_activated, COLUMN);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    path = PATH;
    gtk_tree_view_row_activated (GTK_TREE_VIEW (tree_view), path, COLUMN);
    
    if (!global_count)
    {
        TEST_FAIL ("'row-activated' signal wasn't catched", NULL);
    }
    
    gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef COLUMN
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_row_activated
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column
 * Returns: void
 * Assertion: Activates the cell determined by path and column
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column
 * Expected: void  
 */
static void test_purpose_114()
{
    #define PATH gtk_tree_path_new_from_string ("4")
    #define COLUMN column0
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreePath* path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (),
                                                       "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (),
                                                       "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    
    g_signal_connect (G_OBJECT (tree_view), "row-activated", (GCallback)global_row_activated, COLUMN);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    path = PATH;
    gtk_tree_view_row_activated (GTK_TREE_VIEW (tree_view), path, COLUMN);
    
    if (!global_count)
    {
        TEST_FAIL ("'row-activated' signal wasn't catched", NULL);
    }
    
    gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef COLUMN
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_row_activated
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column
 * Returns: void
 * Assertion: Activates the cell determined by path and column
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column
 * Expected: void  
 */
static void test_purpose_115()
{
    #define PATH gtk_tree_path_new_from_string ("1:2")
    #define COLUMN column1
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreePath* path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (),
                                                       "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (),
                                                       "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    
    g_signal_connect (G_OBJECT (tree_view), "row-activated", (GCallback)global_row_activated, COLUMN);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    path = PATH;
    gtk_tree_view_row_activated (GTK_TREE_VIEW (tree_view), path, COLUMN);
    
    if (!global_count)
    {
        TEST_FAIL ("'row-activated' signal wasn't catched", NULL);
    }
    
    gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef COLUMN
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_row_activated
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column
 * Returns: void
 * Assertion: Activates the cell determined by path and column
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column
 * Expected: void  
 */
static void test_purpose_116()
{
    #define PATH gtk_tree_path_new_from_string ("3:2")
    #define COLUMN column1
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreePath* path;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (),
                                                       "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (),
                                                       "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    
    g_signal_connect (G_OBJECT (tree_view), "row-activated", (GCallback)global_row_activated, COLUMN);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    path = PATH;
    gtk_tree_view_row_activated (GTK_TREE_VIEW (tree_view), path, COLUMN);
    
    if (!global_count)
    {
        TEST_FAIL ("'row-activated' signal wasn't catched", NULL);
    }
    
    gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef COLUMN
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: Finds the path at the point (x, y), relative to widget coordinates 
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: If column is non-NULL, then it will be filled with the column at that point
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: If path is non-NULL, then it will be filled with the GtkTreePath at that point
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: x and y must come from an event on the tree_view only where event->window == gtk_tree_view_get_bin()
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean  
 */
static void test_purpose_117()
{
    #define X 1
    #define Y 1
    #define RESULT TRUE
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column = NULL;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    gint x = 0;
    gint y = 0;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (),
                                                       "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (),
                                                       "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    gtk_widget_show_all (container);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    
    if (RESULT != gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (tree_view), X, Y, &path, &column, &x, &y))
    {
        TEST_FAIL ("wrong result", NULL);
        gtk_widget_destroy (gtk_widget_get_toplevel (container));
        return;
    }
    
    if (RESULT && !path)
    {
        TEST_FAIL ("path is NULL", NULL);
    }
    
    if (RESULT && !column)
    {
        TEST_FAIL ("column is NULL", NULL);
    }
    
    if (path) 
    {
        gtk_tree_path_free (path);
    }
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef X
    #undef Y
    #undef RESULT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: Finds the path at the point (x, y), relative to widget coordinates 
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: If column is non-NULL, then it will be filled with the column at that point
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: If path is non-NULL, then it will be filled with the GtkTreePath at that point
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: x and y must come from an event on the tree_view only where event->window == gtk_tree_view_get_bin()
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean  
 */
static void test_purpose_118()
{
    #define X 20
    #define Y 1
    #define RESULT TRUE
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column = NULL;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    gint x = 0;
    gint y = 0;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (),
                                                       "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (),
                                                       "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    gtk_widget_show_all (container);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    
    if (RESULT != gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (tree_view), X, Y, &path, &column, &x, &y))
    {
        TEST_FAIL ("wrong result", NULL);
        gtk_widget_destroy (gtk_widget_get_toplevel (container));
        return;
    }
    
    if (RESULT && !path)
    {
        TEST_FAIL ("path is NULL", NULL);
    }
    
    if (RESULT && !column)
    {
        TEST_FAIL ("column is NULL", NULL);
    }
    
    if (path) 
    {
        gtk_tree_path_free (path);
    }
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef X
    #undef Y
    #undef RESULT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: Finds the path at the point (x, y), relative to widget coordinates 
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: If column is non-NULL, then it will be filled with the column at that point
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: If path is non-NULL, then it will be filled with the GtkTreePath at that point
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: x and y must come from an event on the tree_view only where event->window == gtk_tree_view_get_bin()
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean  
 */
static void test_purpose_119()
{
    #define X 5
    #define Y 20
    #define RESULT TRUE
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column = NULL;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    gint x = 0;
    gint y = 0;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (),
                                                       "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (),
                                                       "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    gtk_widget_show_all (container);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    
    if (RESULT != gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (tree_view), X, Y, &path, &column, &x, &y))
    {
        TEST_FAIL ("wrong result", NULL);
        gtk_widget_destroy (gtk_widget_get_toplevel (container));
        return;
    }
    
    if (RESULT && !path)
    {
        TEST_FAIL ("path is NULL", NULL);
    }
    
    if (RESULT && !column)
    {
        TEST_FAIL ("column is NULL", NULL);
    }
    
    if (path) 
    {
        gtk_tree_path_free (path);
    }
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef X
    #undef Y
    #undef RESULT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: Finds the path at the point (x, y), relative to widget coordinates 
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: If column is non-NULL, then it will be filled with the column at that point
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: If path is non-NULL, then it will be filled with the GtkTreePath at that point
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: x and y must come from an event on the tree_view only where event->window == gtk_tree_view_get_bin()
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean  
 */
static void test_purpose_120()
{
    #define X 5
    #define Y 30
    #define RESULT TRUE
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column = NULL;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    gint x = 0;
    gint y = 0;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (),
                                                       "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (),
                                                       "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    gtk_widget_show_all (container);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    
    if (RESULT != gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (tree_view), X, Y, &path, &column, &x, &y))
    {
        TEST_FAIL ("wrong result", NULL);
        gtk_widget_destroy (gtk_widget_get_toplevel (container));
        return;
    }
    
    if (RESULT && !path)
    {
        TEST_FAIL ("path is NULL", NULL);
    }
    
    if (RESULT && !column)
    {
        TEST_FAIL ("column is NULL", NULL);
    }
    
    if (path) 
    {
        gtk_tree_path_free (path);
    }
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef X
    #undef Y
    #undef RESULT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: Finds the path at the point (x, y), relative to widget coordinates 
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: If column is non-NULL, then it will be filled with the column at that point
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: If path is non-NULL, then it will be filled with the GtkTreePath at that point
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: x and y must come from an event on the tree_view only where event->window == gtk_tree_view_get_bin()
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean  
 */
static void test_purpose_121()
{
    #define X 10
    #define Y 20
    #define RESULT TRUE
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column = NULL;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    gint x = 0;
    gint y = 0;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (),
                                                       "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (),
                                                       "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    gtk_widget_show_all (container);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    
    if (RESULT != gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (tree_view), X, Y, &path, &column, &x, &y))
    {
        TEST_FAIL ("wrong result", NULL);
        gtk_widget_destroy (gtk_widget_get_toplevel (container));
        return;
    }
    
    if (RESULT && !path)
    {
        TEST_FAIL ("path is NULL", NULL);
    }
    
    if (RESULT && !column)
    {
        TEST_FAIL ("column is NULL", NULL);
    }
    
    if (path) 
    {
        gtk_tree_path_free (path);
    }
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef X
    #undef Y
    #undef RESULT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: Finds the path at the point (x, y), relative to widget coordinates 
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: If column is non-NULL, then it will be filled with the column at that point
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: If path is non-NULL, then it will be filled with the GtkTreePath at that point
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: x and y must come from an event on the tree_view only where event->window == gtk_tree_view_get_bin()
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean  
 */
static void test_purpose_122()
{
    #define X 10
    #define Y 30
    #define RESULT TRUE
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column = NULL;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    gint x = 0;
    gint y = 0;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (),
                                                       "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (),
                                                       "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    gtk_widget_show_all (container);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    
    if (RESULT != gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (tree_view), X, Y, &path, &column, &x, &y))
    {
        TEST_FAIL ("wrong result", NULL);
        gtk_widget_destroy (gtk_widget_get_toplevel (container));
        return;
    }
    
    if (RESULT && !path)
    {
        TEST_FAIL ("path is NULL", NULL);
    }
    
    if (RESULT && !column)
    {
        TEST_FAIL ("column is NULL", NULL);
    }
    
    if (path) 
    {
        gtk_tree_path_free (path);
    }
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef X
    #undef Y
    #undef RESULT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: Finds the path at the point (x, y), relative to widget coordinates 
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: If column is non-NULL, then it will be filled with the column at that point
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: If path is non-NULL, then it will be filled with the GtkTreePath at that point
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: x and y must come from an event on the tree_view only where event->window == gtk_tree_view_get_bin()
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean  
 */
static void test_purpose_123()
{
    #define X 500
    #define Y 500
    #define RESULT FALSE
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column = NULL;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    gint x = 0;
    gint y = 0;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (),
                                                       "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (),
                                                       "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    gtk_widget_show_all (container);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    
    if (RESULT != gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (tree_view), X, Y, &path, &column, &x, &y))
    {
        TEST_FAIL ("wrong result", NULL);
        gtk_widget_destroy (gtk_widget_get_toplevel (container));
        return;
    }
    
    if (RESULT && !path)
    {
        TEST_FAIL ("path is NULL", NULL);
    }
    
    if (RESULT && !column)
    {
        TEST_FAIL ("column is NULL", NULL);
    }
    
    if (path) 
    {
        gtk_tree_path_free (path);
    }
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef X
    #undef Y
    #undef RESULT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: Finds the path at the point (x, y), relative to widget coordinates 
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: If column is non-NULL, then it will be filled with the column at that point
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: If path is non-NULL, then it will be filled with the GtkTreePath at that point
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean
 * 
 * Name: gtk_tree_view_get_path_at_pos
 * Params: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Returns: gboolean
 * Assertion: x and y must come from an event on the tree_view only where event->window == gtk_tree_view_get_bin()
 * Params Values: GtkTreeView* tree_view, gint x, gint y, GtkTreePath** path, GtkTreeViewColumn** column, gint* cell_x, gint* cell_y
 * Expected: gboolean  
 */
static void test_purpose_124()
{
    #define X -1
    #define Y -1
    #define RESULT FALSE
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column = NULL;
    GtkTreeViewColumn* column0;
    GtkTreeViewColumn* column1;
    gint x = 0;
    gint y = 0;
    guint i;
    guint j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    column0 = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (),
                                                       "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column0);
    
    column1 = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (),
                                                       "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column1);
    
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    gtk_widget_show_all (container);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    
    if (RESULT != gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (tree_view), X, Y, &path, &column, &x, &y))
    {
        TEST_FAIL ("wrong result", NULL);
        gtk_widget_destroy (gtk_widget_get_toplevel (container));
        return;
    }
    
    if (RESULT && !path)
    {
        TEST_FAIL ("path is NULL", NULL);
    }
    
    if (RESULT && !column)
    {
        TEST_FAIL ("column is NULL", NULL);
    }
    
    if (path) 
    {
        gtk_tree_path_free (path);
    }
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef X
    #undef Y
    #undef RESULT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_map_expanded_rows
 * Params: GtkTreeView* tree_view, GtkTreeViewMappingFunc func, gpointer data
 * Returns: void
 * Assertion: Calls func on all expanded rows
 * Params Values: GtkTreeView* tree_view, GtkTreeViewMappingFunc func, gpointer data
 * Expected: void  
 */
static void test_purpose_125()
{
    #define PATH "1"
    #define RESULT TRUE
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column;
    guint i, j, k;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (3, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    column = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (),
                                                       "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (),
                                                       "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    path = gtk_tree_path_new_from_string (PATH);
    if (RESULT != gtk_tree_view_expand_row (GTK_TREE_VIEW (tree_view), path, FALSE))
    {
        TEST_FAIL ("wrong result for path '%s'", PATH);
    }
    
    gtk_tree_view_map_expanded_rows (GTK_TREE_VIEW (tree_view), global_map_rows, NULL);
    
    if (RESULT && global_count != 1)
    {
        TEST_FAIL ("global_count = %i", global_count);
    }
    
    if (path) gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef RESULT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_map_expanded_rows
 * Params: GtkTreeView* tree_view, GtkTreeViewMappingFunc func, gpointer data
 * Returns: void
 * Assertion: Calls func on all expanded rows
 * Params Values: GtkTreeView* tree_view, GtkTreeViewMappingFunc func, gpointer data
 * Expected: void  
 */
static void test_purpose_126()
{
    #define PATH "3"
    #define RESULT TRUE
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column;
    guint i, j, k;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (3, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    column = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (),
                                                       "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (),
                                                       "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    path = gtk_tree_path_new_from_string (PATH);
    if (RESULT != gtk_tree_view_expand_row (GTK_TREE_VIEW (tree_view), path, FALSE))
    {
        TEST_FAIL ("wrong result for path '%s'", PATH);
    }
    
    gtk_tree_view_map_expanded_rows (GTK_TREE_VIEW (tree_view), global_map_rows, NULL);
    
    if (RESULT && global_count != 1)
    {
        TEST_FAIL ("global_count = %i", global_count);
    }
    
    if (path) gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef RESULT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_map_expanded_rows
 * Params: GtkTreeView* tree_view, GtkTreeViewMappingFunc func, gpointer data
 * Returns: void
 * Assertion: Calls func on all expanded rows
 * Params Values: GtkTreeView* tree_view, GtkTreeViewMappingFunc func, gpointer data
 * Expected: void  
 */
static void test_purpose_127()
{
    #define PATH "1:1"
    #define RESULT FALSE
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column;
    guint i, j, k;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (3, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    column = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (),
                                                       "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (),
                                                       "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    path = gtk_tree_path_new_from_string (PATH);
    if (RESULT != gtk_tree_view_expand_row (GTK_TREE_VIEW (tree_view), path, FALSE))
    {
        TEST_FAIL ("wrong result for path '%s'", PATH);
    }
    
    gtk_tree_view_map_expanded_rows (GTK_TREE_VIEW (tree_view), global_map_rows, NULL);
    
    if (RESULT && global_count != 1)
    {
        TEST_FAIL ("global_count = %i", global_count);
    }
    
    if (path) gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef RESULT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_map_expanded_rows
 * Params: GtkTreeView* tree_view, GtkTreeViewMappingFunc func, gpointer data
 * Returns: void
 * Assertion: Calls func on all expanded rows
 * Params Values: GtkTreeView* tree_view, GtkTreeViewMappingFunc func, gpointer data
 * Expected: void  
 */
static void test_purpose_128()
{
    #define PATH "1:3"
    #define RESULT FALSE
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column;
    guint i, j, k;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (3, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    column = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (),
                                                       "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (),
                                                       "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    path = gtk_tree_path_new_from_string (PATH);
    if (RESULT != gtk_tree_view_expand_row (GTK_TREE_VIEW (tree_view), path, FALSE))
    {
        TEST_FAIL ("wrong result for path '%s'", PATH);
    }
    
    gtk_tree_view_map_expanded_rows (GTK_TREE_VIEW (tree_view), global_map_rows, NULL);
    
    if (RESULT && global_count != 1)
    {
        TEST_FAIL ("global_count = %i", global_count);
    }
    
    if (path) gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef RESULT
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_move_column_after
 * Params: GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* base_column
 * Returns: void
 * Assertion: Moves column to be after to base_column
 * Params Values: GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* base_column
 * Expected: void
 * 
 * Name: gtk_tree_view_move_column_after
 * Params: GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* base_column
 * Returns: void
 * Assertion: If base_column is NULL, then column is placed in the first position
 * Params Values: GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* base_column
 * Expected: void  
 */
static void test_purpose_129()
{
    #define COUNT 5
    #define POS -1
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn* column;
    guint i, j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    
    for (i = 0; i < COUNT; i++)
    {
        column = gtk_tree_view_column_new_with_attributes ("Column", gtk_cell_renderer_text_new (), "text", i, NULL);
        gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    }
    
    column = gtk_tree_view_column_new_with_attributes ("New column", gtk_cell_renderer_text_new (), "text", POS,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    if (POS >= 0)
    {
        gtk_tree_view_move_column_after (GTK_TREE_VIEW (tree_view), column, 
                                         gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), POS));
    } else {
    
        gtk_tree_view_move_column_after (GTK_TREE_VIEW (tree_view), column, NULL);
    }
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    if (strcmp ("New column", gtk_tree_view_column_get_title (gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), POS+1))))
    {
        TEST_FAIL ("Column moved incorrectly", NULL);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef COUNT
    #undef POS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_move_column_after
 * Params: GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* base_column
 * Returns: void
 * Assertion: Moves column to be after to base_column
 * Params Values: GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* base_column
 * Expected: void
 * 
 * Name: gtk_tree_view_move_column_after
 * Params: GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* base_column
 * Returns: void
 * Assertion: If base_column is NULL, then column is placed in the first position
 * Params Values: GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* base_column
 * Expected: void  
 */
static void test_purpose_130()
{
    #define COUNT 5
    #define POS 0
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn* column;
    guint i, j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    
    for (i = 0; i < COUNT; i++)
    {
        column = gtk_tree_view_column_new_with_attributes ("Column", gtk_cell_renderer_text_new (), "text", i, NULL);
        gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    }
    
    column = gtk_tree_view_column_new_with_attributes ("New column", gtk_cell_renderer_text_new (), "text", POS,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    if (POS >= 0)
    {
        gtk_tree_view_move_column_after (GTK_TREE_VIEW (tree_view), column, 
                                         gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), POS));
    } else {
    
        gtk_tree_view_move_column_after (GTK_TREE_VIEW (tree_view), column, NULL);
    }
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    if (strcmp ("New column", gtk_tree_view_column_get_title (gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), POS+1))))
    {
        TEST_FAIL ("Column moved incorrectly", NULL);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef COUNT
    #undef POS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_move_column_after
 * Params: GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* base_column
 * Returns: void
 * Assertion: Moves column to be after to base_column
 * Params Values: GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* base_column
 * Expected: void
 * 
 * Name: gtk_tree_view_move_column_after
 * Params: GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* base_column
 * Returns: void
 * Assertion: If base_column is NULL, then column is placed in the first position
 * Params Values: GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* base_column
 * Expected: void  
 */
static void test_purpose_131()
{
    #define COUNT 5
    #define POS 4
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn* column;
    guint i, j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    
    for (i = 0; i < COUNT; i++)
    {
        column = gtk_tree_view_column_new_with_attributes ("Column", gtk_cell_renderer_text_new (), "text", i, NULL);
        gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    }
    
    column = gtk_tree_view_column_new_with_attributes ("New column", gtk_cell_renderer_text_new (), "text", POS,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    if (POS >= 0)
    {
        gtk_tree_view_move_column_after (GTK_TREE_VIEW (tree_view), column, 
                                         gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), POS));
    } else {
    
        gtk_tree_view_move_column_after (GTK_TREE_VIEW (tree_view), column, NULL);
    }
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    if (strcmp ("New column", gtk_tree_view_column_get_title (gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), POS+1))))
    {
        TEST_FAIL ("Column moved incorrectly", NULL);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef COUNT
    #undef POS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_move_column_after
 * Params: GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* base_column
 * Returns: void
 * Assertion: Moves column to be after to base_column
 * Params Values: GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* base_column
 * Expected: void
 * 
 * Name: gtk_tree_view_move_column_after
 * Params: GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* base_column
 * Returns: void
 * Assertion: If base_column is NULL, then column is placed in the first position
 * Params Values: GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* base_column
 * Expected: void  
 */
static void test_purpose_132()
{
    #define COUNT 50
    #define POS 40
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeViewColumn* column;
    guint i, j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    global_count = 0;
    tree_store = gtk_tree_store_new (1, G_TYPE_UINT);
    
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    
    
    for (i = 0; i < COUNT; i++)
    {
        column = gtk_tree_view_column_new_with_attributes ("Column", gtk_cell_renderer_text_new (), "text", i, NULL);
        gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    }
    
    column = gtk_tree_view_column_new_with_attributes ("New column", gtk_cell_renderer_text_new (), "text", POS,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    if (POS >= 0)
    {
        gtk_tree_view_move_column_after (GTK_TREE_VIEW (tree_view), column, 
                                         gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), POS));
    } else {
    
        gtk_tree_view_move_column_after (GTK_TREE_VIEW (tree_view), column, NULL);
    }
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 300, 200);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    if (strcmp ("New column", gtk_tree_view_column_get_title (gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), POS+1))))
    {
        TEST_FAIL ("Column moved incorrectly", NULL);
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef COUNT
    #undef POS
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_scroll_to_point
 * Params: GtkTreeView* tree_view, gint tree_x, gint tree_y
 * Returns: void
 * Assertion: Scrolls the tree view such that the top-left corner of the visible area is tree_x, tree_y, where tree_x and tree_y are specified in tree window coordinates
 * Params Values: GtkTreeView* tree_view, gint tree_x, gint tree_y
 * Expected: void  
 */
static void test_purpose_133()
{
    #define PATH "0"
    #define COL_POSITION 0
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column;
    GdkRectangle rect;
    GdkRectangle cell;
    guint i, j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    column = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (), "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (), "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), COL_POSITION);
    path = gtk_tree_path_new_from_string (PATH);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 200, 200);
    gtk_widget_show_all (container);
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column, &cell);
    gtk_tree_view_scroll_to_point (GTK_TREE_VIEW (tree_view), cell.x, cell.y);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &rect); 
    
    if (!gdk_rectangle_intersect (&rect, &cell, &cell))
    {
        TEST_FAIL ("visible rect = (%d, %d, %d, %d)", rect.x, rect.y, rect.width, rect.height);   
        TEST_FAIL ("cell         = (%d, %d, %d, %d)", cell.x, cell.y, cell.width, cell.height);   
    }
    
    if (path) gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef COL_POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_scroll_to_point
 * Params: GtkTreeView* tree_view, gint tree_x, gint tree_y
 * Returns: void
 * Assertion: Scrolls the tree view such that the top-left corner of the visible area is tree_x, tree_y, where tree_x and tree_y are specified in tree window coordinates
 * Params Values: GtkTreeView* tree_view, gint tree_x, gint tree_y
 * Expected: void  
 */
static void test_purpose_134()
{
    #define PATH "1"
    #define COL_POSITION 0
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column;
    GdkRectangle rect;
    GdkRectangle cell;
    guint i, j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    column = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (), "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (), "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), COL_POSITION);
    path = gtk_tree_path_new_from_string (PATH);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 200, 200);
    gtk_widget_show_all (container);
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column, &cell);
    gtk_tree_view_scroll_to_point (GTK_TREE_VIEW (tree_view), cell.x, cell.y);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &rect); 
    
    if (!gdk_rectangle_intersect (&rect, &cell, &cell))
    {
        TEST_FAIL ("visible rect = (%d, %d, %d, %d)", rect.x, rect.y, rect.width, rect.height);   
        TEST_FAIL ("cell         = (%d, %d, %d, %d)", cell.x, cell.y, cell.width, cell.height);   
    }
    
    if (path) gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef COL_POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_scroll_to_point
 * Params: GtkTreeView* tree_view, gint tree_x, gint tree_y
 * Returns: void
 * Assertion: Scrolls the tree view such that the top-left corner of the visible area is tree_x, tree_y, where tree_x and tree_y are specified in tree window coordinates
 * Params Values: GtkTreeView* tree_view, gint tree_x, gint tree_y
 * Expected: void  
 */
static void test_purpose_135()
{
    #define PATH "2:1"
    #define COL_POSITION 1
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column;
    GdkRectangle rect;
    GdkRectangle cell;
    guint i, j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    column = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (), "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (), "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), COL_POSITION);
    path = gtk_tree_path_new_from_string (PATH);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 200, 200);
    gtk_widget_show_all (container);
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column, &cell);
    gtk_tree_view_scroll_to_point (GTK_TREE_VIEW (tree_view), cell.x, cell.y);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &rect); 
    
    if (!gdk_rectangle_intersect (&rect, &cell, &cell))
    {
        TEST_FAIL ("visible rect = (%d, %d, %d, %d)", rect.x, rect.y, rect.width, rect.height);   
        TEST_FAIL ("cell         = (%d, %d, %d, %d)", cell.x, cell.y, cell.width, cell.height);   
    }
    
    if (path) gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef COL_POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_scroll_to_point
 * Params: GtkTreeView* tree_view, gint tree_x, gint tree_y
 * Returns: void
 * Assertion: Scrolls the tree view such that the top-left corner of the visible area is tree_x, tree_y, where tree_x and tree_y are specified in tree window coordinates
 * Params Values: GtkTreeView* tree_view, gint tree_x, gint tree_y
 * Expected: void  
 */
static void test_purpose_136()
{
    #define PATH "2:2"
    #define COL_POSITION 1
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column;
    GdkRectangle rect;
    GdkRectangle cell;
    guint i, j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    column = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (), "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (), "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), COL_POSITION);
    path = gtk_tree_path_new_from_string (PATH);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 200, 200);
    gtk_widget_show_all (container);
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column, &cell);
    gtk_tree_view_scroll_to_point (GTK_TREE_VIEW (tree_view), cell.x, cell.y);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &rect); 
    
    if (!gdk_rectangle_intersect (&rect, &cell, &cell))
    {
        TEST_FAIL ("visible rect = (%d, %d, %d, %d)", rect.x, rect.y, rect.width, rect.height);   
        TEST_FAIL ("cell         = (%d, %d, %d, %d)", cell.x, cell.y, cell.width, cell.height);   
    }
    
    if (path) gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef COL_POSITION
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_scroll_to_point
 * Params: GtkTreeView* tree_view, gint tree_x, gint tree_y
 * Returns: void
 * Assertion: If either tree_x or tree_y are -1, then that direction isn't scrolled'
 * Params Values: GtkTreeView* tree_view, gint tree_x, gint tree_y
 * Expected: void  
 */
static void test_purpose_137()
{
    #define DX -1
    #define DY 200
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeViewColumn* column;
    GdkRectangle rect;
    GdkRectangle cell;
    guint i, j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    column = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (), "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (), "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 100, 100);
    gtk_widget_show_all (container);
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &cell); 
    gtk_tree_view_scroll_to_point (GTK_TREE_VIEW (tree_view), DX, DY);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &rect); 
    
    if ((DX == -1) && (rect.x != cell.x))
    {
        TEST_FAIL ("befor = (%d, %d, %d, %d)", cell.x, cell.y, cell.width, cell.height);   
        TEST_FAIL ("after = (%d, %d, %d, %d)", rect.x, rect.y, rect.width, rect.height);   
    }
    
    if ((DY == -1) && (rect.y != cell.y))
    {
        TEST_FAIL ("befor = (%d, %d, %d, %d)", cell.x, cell.y, cell.width, cell.height);   
        TEST_FAIL ("after = (%d, %d, %d, %d)", rect.x, rect.y, rect.width, rect.height);   
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef DX
    #undef DY
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_scroll_to_point
 * Params: GtkTreeView* tree_view, gint tree_x, gint tree_y
 * Returns: void
 * Assertion: If either tree_x or tree_y are -1, then that direction isn't scrolled'
 * Params Values: GtkTreeView* tree_view, gint tree_x, gint tree_y
 * Expected: void  
 */
static void test_purpose_138()
{
    #define DX 200
    #define DY -1
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreeViewColumn* column;
    GdkRectangle rect;
    GdkRectangle cell;
    guint i, j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    column = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (), "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (), "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 100, 100);
    gtk_widget_show_all (container);
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &cell); 
    gtk_tree_view_scroll_to_point (GTK_TREE_VIEW (tree_view), DX, DY);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &rect); 
    
    if ((DX == -1) && (rect.x != cell.x))
    {
        TEST_FAIL ("befor = (%d, %d, %d, %d)", cell.x, cell.y, cell.width, cell.height);   
        TEST_FAIL ("after = (%d, %d, %d, %d)", rect.x, rect.y, rect.width, rect.height);   
    }
    
    if ((DY == -1) && (rect.y != cell.y))
    {
        TEST_FAIL ("befor = (%d, %d, %d, %d)", cell.x, cell.y, cell.width, cell.height);   
        TEST_FAIL ("after = (%d, %d, %d, %d)", rect.x, rect.y, rect.width, rect.height);   
    }
    
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef DX
    #undef DY
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: Moves the alignments of tree_view to the position specified by column and path
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: This function only works if the model is set, and path is a valid row on the model
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If use_align is FALSE, then the alignment arguments are ignored, and the tree does the minimum amount of work to scroll the cell onto the screen
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If the model changes before the tree_view is realized, the centered path will be modified to reflect this change
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If the cell is currently visible on the screen, nothing is done
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void  
 */
static void test_purpose_139()
{
    #define PATH "0"
    #define COL_POSITION 0
    #define USE_ALIGN TRUE
    #define ROW_ALIGN 0.0
    #define COL_ALIGN 0.0
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column;
    GdkRectangle rect;
    GdkRectangle cell;
    guint i, j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    column = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (), "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (), "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), COL_POSITION);
    path = gtk_tree_path_new_from_string (PATH);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 200, 200);
    gtk_widget_show_all (container);
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column, &cell);
    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (tree_view), path, column, USE_ALIGN, ROW_ALIGN, COL_ALIGN);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &rect); 
    
    if (!gdk_rectangle_intersect (&rect, &cell, &cell))
    {
        TEST_FAIL ("visible rect = (%d, %d, %d, %d)", rect.x, rect.y, rect.width, rect.height);   
        TEST_FAIL ("cell         = (%d, %d, %d, %d)", cell.x, cell.y, cell.width, cell.height);   
    }
    
    if (path) gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef COL_POSITION
    #undef USE_ALIGN 
    #undef ROW_ALIGN 
    #undef COL_ALIGN 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: Moves the alignments of tree_view to the position specified by column and path
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: This function only works if the model is set, and path is a valid row on the model
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If use_align is FALSE, then the alignment arguments are ignored, and the tree does the minimum amount of work to scroll the cell onto the screen
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If the model changes before the tree_view is realized, the centered path will be modified to reflect this change
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If the cell is currently visible on the screen, nothing is done
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void  
 */
static void test_purpose_140()
{
    #define PATH "0"
    #define COL_POSITION 0
    #define USE_ALIGN TRUE
    #define ROW_ALIGN 0.1
    #define COL_ALIGN 0.1
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column;
    GdkRectangle rect;
    GdkRectangle cell;
    guint i, j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    column = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (), "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (), "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), COL_POSITION);
    path = gtk_tree_path_new_from_string (PATH);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 200, 200);
    gtk_widget_show_all (container);
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column, &cell);
    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (tree_view), path, column, USE_ALIGN, ROW_ALIGN, COL_ALIGN);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &rect); 
    
    if (!gdk_rectangle_intersect (&rect, &cell, &cell))
    {
        TEST_FAIL ("visible rect = (%d, %d, %d, %d)", rect.x, rect.y, rect.width, rect.height);   
        TEST_FAIL ("cell         = (%d, %d, %d, %d)", cell.x, cell.y, cell.width, cell.height);   
    }
    
    if (path) gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef COL_POSITION
    #undef USE_ALIGN 
    #undef ROW_ALIGN 
    #undef COL_ALIGN 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: Moves the alignments of tree_view to the position specified by column and path
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: This function only works if the model is set, and path is a valid row on the model
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If use_align is FALSE, then the alignment arguments are ignored, and the tree does the minimum amount of work to scroll the cell onto the screen
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If the model changes before the tree_view is realized, the centered path will be modified to reflect this change
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If the cell is currently visible on the screen, nothing is done
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void  
 */
static void test_purpose_141()
{
    #define PATH "1"
    #define COL_POSITION 0
    #define USE_ALIGN TRUE
    #define ROW_ALIGN 0.5
    #define COL_ALIGN 0.5
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column;
    GdkRectangle rect;
    GdkRectangle cell;
    guint i, j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    column = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (), "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (), "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), COL_POSITION);
    path = gtk_tree_path_new_from_string (PATH);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 200, 200);
    gtk_widget_show_all (container);
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column, &cell);
    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (tree_view), path, column, USE_ALIGN, ROW_ALIGN, COL_ALIGN);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &rect); 
    
    if (!gdk_rectangle_intersect (&rect, &cell, &cell))
    {
        TEST_FAIL ("visible rect = (%d, %d, %d, %d)", rect.x, rect.y, rect.width, rect.height);   
        TEST_FAIL ("cell         = (%d, %d, %d, %d)", cell.x, cell.y, cell.width, cell.height);   
    }
    
    if (path) gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef COL_POSITION
    #undef USE_ALIGN 
    #undef ROW_ALIGN 
    #undef COL_ALIGN 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: Moves the alignments of tree_view to the position specified by column and path
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: This function only works if the model is set, and path is a valid row on the model
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If use_align is FALSE, then the alignment arguments are ignored, and the tree does the minimum amount of work to scroll the cell onto the screen
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If the model changes before the tree_view is realized, the centered path will be modified to reflect this change
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If the cell is currently visible on the screen, nothing is done
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void  
 */
static void test_purpose_142()
{
    #define PATH "2"
    #define COL_POSITION 0
    #define USE_ALIGN FALSE
    #define ROW_ALIGN 0.0
    #define COL_ALIGN 0.0
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column;
    GdkRectangle rect;
    GdkRectangle cell;
    guint i, j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    column = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (), "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (), "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), COL_POSITION);
    path = gtk_tree_path_new_from_string (PATH);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 200, 200);
    gtk_widget_show_all (container);
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column, &cell);
    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (tree_view), path, column, USE_ALIGN, ROW_ALIGN, COL_ALIGN);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &rect); 
    
    if (!gdk_rectangle_intersect (&rect, &cell, &cell))
    {
        TEST_FAIL ("visible rect = (%d, %d, %d, %d)", rect.x, rect.y, rect.width, rect.height);   
        TEST_FAIL ("cell         = (%d, %d, %d, %d)", cell.x, cell.y, cell.width, cell.height);   
    }
    
    if (path) gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef COL_POSITION
    #undef USE_ALIGN 
    #undef ROW_ALIGN 
    #undef COL_ALIGN 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: Moves the alignments of tree_view to the position specified by column and path
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: This function only works if the model is set, and path is a valid row on the model
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If use_align is FALSE, then the alignment arguments are ignored, and the tree does the minimum amount of work to scroll the cell onto the screen
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If the model changes before the tree_view is realized, the centered path will be modified to reflect this change
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If the cell is currently visible on the screen, nothing is done
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void  
 */
static void test_purpose_143()
{
    #define PATH "2:0"
    #define COL_POSITION 1
    #define USE_ALIGN TRUE
    #define ROW_ALIGN 0.1
    #define COL_ALIGN 0.1
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column;
    GdkRectangle rect;
    GdkRectangle cell;
    guint i, j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    column = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (), "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (), "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), COL_POSITION);
    path = gtk_tree_path_new_from_string (PATH);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 200, 200);
    gtk_widget_show_all (container);
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column, &cell);
    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (tree_view), path, column, USE_ALIGN, ROW_ALIGN, COL_ALIGN);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &rect); 
    
    if (!gdk_rectangle_intersect (&rect, &cell, &cell))
    {
        TEST_FAIL ("visible rect = (%d, %d, %d, %d)", rect.x, rect.y, rect.width, rect.height);   
        TEST_FAIL ("cell         = (%d, %d, %d, %d)", cell.x, cell.y, cell.width, cell.height);   
    }
    
    if (path) gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef COL_POSITION
    #undef USE_ALIGN 
    #undef ROW_ALIGN 
    #undef COL_ALIGN 
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: Moves the alignments of tree_view to the position specified by column and path
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: This function only works if the model is set, and path is a valid row on the model
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If use_align is FALSE, then the alignment arguments are ignored, and the tree does the minimum amount of work to scroll the cell onto the screen
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If the model changes before the tree_view is realized, the centered path will be modified to reflect this change
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void
 * 
 * Name: gtk_tree_view_scroll_to_cell
 * Params: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Returns: void
 * Assertion: If the cell is currently visible on the screen, nothing is done
 * Params Values: GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gboolean use_align, gfloat row_align, gfloat col_align
 * Expected: void  
 */
static void test_purpose_144()
{
    #define PATH "2:3"
    #define COL_POSITION 1
    #define USE_ALIGN TRUE
    #define ROW_ALIGN 0.1
    #define COL_ALIGN 0.1
    
    GtkWidget* container;
    GtkWidget* window;
    GtkWidget *tree_view;
    GtkTreeStore *tree_store;
    GtkTreeIter iter;
    GtkTreeIter child_iter;
    GtkTreePath* path = NULL;
    GtkTreeViewColumn* column;
    GdkRectangle rect;
    GdkRectangle cell;
    guint i, j;
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    tree_store = gtk_tree_store_new (2, G_TYPE_UINT, G_TYPE_UINT);
    if (!tree_store)
    {
        TEST_FAIL (get_not_created_msg(), NULL); 
    }
    
    for (i = 0; i < NUM_TREE_ELEMS; i++)
    {
        gint j;
        
        gtk_tree_store_append (GTK_TREE_STORE (tree_store), &iter, NULL);
        gtk_tree_store_set (GTK_TREE_STORE (tree_store), &iter,
                            0, data[i].number1,
                            1, data[i].number2,
                            -1);
        for (j = 0; j < NUM_TREE_ELEMS; j++)
        {
            gtk_tree_store_append (GTK_TREE_STORE (tree_store), &child_iter, &iter);
            gtk_tree_store_set (GTK_TREE_STORE (tree_store), &child_iter,
                                0, child_data[j].number1,
                                1, child_data[j].number2,
                                -1);
        }
    }
    
    tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    if (!tree_view)
    {
        TEST_FAIL (get_not_created_msg (), NULL); 
        return;
    }
    if (!GTK_IS_TREE_VIEW (tree_view))
    {
        TEST_FAIL (get_wrong_type_msg (), NULL); 
        return;
    }
    column = gtk_tree_view_column_new_with_attributes ("Number 1", gtk_cell_renderer_text_new (), "text", 0, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_column_new_with_attributes ("Number 2", gtk_cell_renderer_text_new (), "text", 1,  NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
    
    column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree_view), COL_POSITION);
    path = gtk_tree_path_new_from_string (PATH);
    
    gtk_tree_view_expand_all (GTK_TREE_VIEW (tree_view));
    
    container = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (window), tree_view);
    gtk_container_add (GTK_CONTAINER (container), window);
    gtk_window_resize (GTK_WINDOW (container), 200, 200);
    gtk_widget_show_all (container);
    gtk_tree_view_get_cell_area (GTK_TREE_VIEW (tree_view), path, column, &cell);
    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (tree_view), path, column, USE_ALIGN, ROW_ALIGN, COL_ALIGN);
    
    /******* VISUALISATION *******/
    TEST_VISUALIZE (container);
    /******* -------------  *******/  
    
    gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (tree_view), &rect); 
    
    if (!gdk_rectangle_intersect (&rect, &cell, &cell))
    {
        TEST_FAIL ("visible rect = (%d, %d, %d, %d)", rect.x, rect.y, rect.width, rect.height);   
        TEST_FAIL ("cell         = (%d, %d, %d, %d)", cell.x, cell.y, cell.width, cell.height);   
    }
    
    if (path) gtk_tree_path_free (path);
    gtk_widget_destroy (gtk_widget_get_toplevel (container));
    
    #undef PATH
    #undef COL_POSITION
    #undef USE_ALIGN 
    #undef ROW_ALIGN 
    #undef COL_ALIGN 
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for GtkTreeView ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {test_purpose_5, 1},
    {test_purpose_6, 1},
    {test_purpose_7, 1},
    {test_purpose_8, 1},
    {test_purpose_9, 1},
    {test_purpose_10, 1},
    {test_purpose_11, 1},
    {test_purpose_12, 1},
    {test_purpose_13, 1},
    {test_purpose_14, 1},
    {test_purpose_15, 1},
    {test_purpose_16, 1},
    {test_purpose_17, 1},
    {test_purpose_18, 1},
    {test_purpose_19, 1},
    {test_purpose_20, 1},
    {test_purpose_21, 1},
    {test_purpose_22, 1},
    {test_purpose_23, 1},
    {test_purpose_24, 1},
    {test_purpose_25, 1},
    {test_purpose_26, 1},
    {test_purpose_27, 1},
    {test_purpose_28, 1},
    {test_purpose_29, 1},
    {test_purpose_30, 1},
    {test_purpose_31, 1},
    {test_purpose_32, 1},
    {test_purpose_33, 1},
    {test_purpose_34, 1},
    {test_purpose_35, 1},
    {test_purpose_36, 1},
    {test_purpose_37, 1},
    {test_purpose_38, 1},
    {test_purpose_39, 1},
    {test_purpose_40, 1},
    {test_purpose_41, 1},
    {test_purpose_42, 1},
    {test_purpose_43, 1},
    {test_purpose_44, 1},
    {test_purpose_45, 1},
    {test_purpose_46, 1},
    {test_purpose_47, 1},
    {test_purpose_48, 1},
    {test_purpose_49, 1},
    {test_purpose_50, 1},
    {test_purpose_51, 1},
    {test_purpose_52, 1},
    {test_purpose_53, 1},
    {test_purpose_54, 1},
    {test_purpose_55, 1},
    {test_purpose_56, 1},
    {test_purpose_57, 1},
    {test_purpose_58, 1},
    {test_purpose_59, 1},
    {test_purpose_60, 1},
    {test_purpose_61, 1},
    {test_purpose_62, 1},
    {test_purpose_63, 1},
    {test_purpose_64, 1},
    {test_purpose_65, 1},
    {test_purpose_66, 1},
    {test_purpose_67, 1},
    {test_purpose_68, 1},
    {test_purpose_69, 1},
    {test_purpose_70, 1},
    {test_purpose_71, 1},
    {test_purpose_72, 1},
    {test_purpose_73, 1},
    {test_purpose_74, 1},
    {test_purpose_75, 1},
    {test_purpose_76, 1},
    {test_purpose_77, 1},
    {test_purpose_78, 1},
    {test_purpose_79, 1},
    {test_purpose_80, 1},
    {test_purpose_81, 1},
    {test_purpose_82, 1},
    {test_purpose_83, 1},
    {test_purpose_84, 1},
    {test_purpose_85, 1},
    {test_purpose_86, 1},
    {test_purpose_87, 1},
    {test_purpose_88, 1},
    {test_purpose_89, 1},
    {test_purpose_90, 1},
    {test_purpose_91, 1},
    {test_purpose_92, 1},
    {test_purpose_93, 1},
    {test_purpose_94, 1},
    {test_purpose_95, 1},
    {test_purpose_96, 1},
    {test_purpose_97, 1},
    {test_purpose_98, 1},
    {test_purpose_99, 1},
    {test_purpose_100, 1},
    {test_purpose_101, 1},
    {test_purpose_102, 1},
    {test_purpose_103, 1},
    {test_purpose_104, 1},
    {test_purpose_105, 1},
    {test_purpose_106, 1},
    {test_purpose_107, 1},
    {test_purpose_108, 1},
    {test_purpose_109, 1},
    {test_purpose_110, 1},
    {test_purpose_111, 1},
    {test_purpose_112, 1},
    {test_purpose_113, 1},
    {test_purpose_114, 1},
    {test_purpose_115, 1},
    {test_purpose_116, 1},
    {test_purpose_117, 1},
    {test_purpose_118, 1},
    {test_purpose_119, 1},
    {test_purpose_120, 1},
    {test_purpose_121, 1},
    {test_purpose_122, 1},
    {test_purpose_123, 1},
    {test_purpose_124, 1},
    {test_purpose_125, 1},
    {test_purpose_126, 1},
    {test_purpose_127, 1},
    {test_purpose_128, 1},
    {test_purpose_129, 1},
    {test_purpose_130, 1},
    {test_purpose_131, 1},
    {test_purpose_132, 1},
    {test_purpose_133, 1},
    {test_purpose_134, 1},
    {test_purpose_135, 1},
    {test_purpose_136, 1},
    {test_purpose_137, 1},
    {test_purpose_138, 1},
    {test_purpose_139, 1},
    {test_purpose_140, 1},
    {test_purpose_141, 1},
    {test_purpose_142, 1},
    {test_purpose_143, 1},
    {test_purpose_144, 1},
    {NULL,0}
};
